Package: SWATH2stats
Type: Package
Title: Transform and Filter SWATH Data for Statistical Packages
Version: 1.42.0
Date: 2025-12-17
Authors@R: c(person("Peter", "Blattmann", role = c("aut", "cre"),
                    email="peter_blattmann@bluewin.ch",
                    comment = c(ORCID = "0000-0001-9105-6381")), 
             person("Moritz", "Heusel", role = c("aut"),
                    comment = c(ORCID = "0000-0002-8506-530X")),
             person("Ruedi", "Aebersold", role = c("aut"),
                    comment = c(ORCID = "0000-0002-9576-3267")))
Author: Peter Blattmann [aut, cre]
  Moritz Heusel [aut]
  Ruedi Aebersold [aut]
Maintainer: Peter Blattmann <peter_blattmann@bluewin.ch>
Description: This package is intended to transform SWATH data from the
    OpenSWATH software into a format readable by other statistics
    packages while performing filtering, annotation and FDR estimation.
License: GPL-3
Depends: R(>= 2.10.0)
Imports: data.table, reshape2, ggplot2, stats, grDevices, graphics,
        utils, biomaRt, methods
Suggests: testthat, knitr, rmarkdown
Enhances: MSstats, PECA, aLFQ
biocViews: Proteomics, Annotation, ExperimentalDesign, Preprocessing,
        MassSpectrometry, ImmunoOncology
NeedsCompilation: no
VignetteBuilder: knitr
RoxygenNote: 7.3.3
Encoding: UTF-8
URL: https://peterblattmann.github.io/SWATH2stats/
BugReports: https://github.com/peterblattmann/SWATH2stats
git_url: https://git.bioconductor.org/packages/SWATH2stats
git_branch: RELEASE_3_23
git_last_commit: 402b2fc
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
Packaged: 2026-04-29 01:04:26 UTC; biocbuild

# Generated by roxygen2: do not edit by hand

S3method(plot,fdr_cube)
S3method(plot,fdr_table)
export(JPP_update)
export(add_genesymbol)
export(assess_decoy_rate)
export(assess_fdr_byrun)
export(assess_fdr_overall)
export(convert4MSstats)
export(convert4PECA)
export(convert4aLFQ)
export(convert4mapDIA)
export(convert4pythonscript)
export(convert_protein_ids)
export(count_analytes)
export(disaggregate)
export(filter_all_peptides)
export(filter_mscore)
export(filter_mscore_condition)
export(filter_mscore_fdr)
export(filter_mscore_freqobs)
export(filter_on_max_peptides)
export(filter_on_min_peptides)
export(filter_proteotypic_peptides)
export(import_data)
export(load_mart)
export(mscore4assayfdr)
export(mscore4pepfdr)
export(mscore4protfdr)
export(plot_correlation_between_samples)
export(plot_variation)
export(plot_variation_vs_total)
export(reduce_OpenSWATH_output)
export(removeDecoyProteins)
export(sample_annotation)
export(transform_MSstats_OpenSWATH)
export(unifyProteinGroupLabels)
export(validate_columns)
export(write_matrix_peptides)
export(write_matrix_proteins)
importFrom(biomaRt,getBM)
importFrom(biomaRt,listDatasets)
importFrom(biomaRt,listMarts)
importFrom(biomaRt,useMart)
importFrom(data.table,data.table)
importFrom(data.table,fread)
importFrom(data.table,setkey)
importFrom(data.table,setkeyv)
importFrom(data.table,setnames)
importFrom(ggplot2,aes)
importFrom(ggplot2,aes_string)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_boxplot)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,geom_tile)
importFrom(ggplot2,geom_violin)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_fill_gradient)
importFrom(ggplot2,scale_x_discrete)
importFrom(ggplot2,scale_y_discrete)
importFrom(ggplot2,stat_summary)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(grDevices,dev.off)
importFrom(grDevices,pdf)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,barplot)
importFrom(graphics,grid)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,points)
importFrom(methods,is)
importFrom(reshape2,colsplit)
importFrom(reshape2,dcast)
importFrom(reshape2,melt)
importFrom(stats,aggregate)
importFrom(stats,cor)
importFrom(stats,density)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,sd)
importFrom(utils,head)
importFrom(utils,write.csv)
importFrom(utils,write.table)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# SWATH2stats

This package is intended to transform extracted SWATH/DIA data from the
OpenSWATH or other (e.g. Spectronaut) software into a format
directly-usable by statistics packages (e.g. mapDIA, PECA, MSstats)
while performing filtering, annotation and FDR assessment if necessary.

## Analyzing SWATH/DIA data

How to extract SWATH/DIA data before using SWATH2stats with OpenSWATH
can be found here: <http://openswath.org>

## Usage

SWATH2stats is a Bioconductor package. Go to
<https://www.bioconductor.org/packages/release/bioc/html/SWATH2stats.html>
to see all information related to installation. Importantly there exists
both a release and development version.

## Contribution

Please feel free to comment and post issues or pull requests on github.

## Publication

For the publication describing this package, see:
<https://doi.org/10.1371/journal.pone.0153160>

## References

  - Blattmann P, Heusel M, Aebersold R. SWATH2stats: An R/Bioconductor
    Package to Process and Convert Quantitative SWATH-MS Proteomics Data
    for Downstream Analysis Tools. PLoS ONE 11(4): e0153160 (2016). doi:
    10.1371/journal.pone.0153160.

  - Rost HL, Rosenberger G, Navarro P, Gillet L, Miladinovic SM,
    Schubert OT, Wolski W, Collins BC, Malmstrom J, Malmstrom L,
    Aebersold R. OpenSWATH enables automated, targeted analysis of
    data-independent acquisition MS data. Nature Biotechnology. 2014
    Mar;32(3):219-23. doi: 10.1038/nbt.2841.

  - Choi M, Chang CY, Clough T, Broudy D, Killeen T, MacLean B, Vitek O.
    MSstats: an R package for statistical analysis of quantitative mass
    spectrometry-based proteomic experiments.Bioinformatics. 2014 Sep
    1;30(17):2524-6. doi: 10.1093/bioinformatics/btu305.

  - Rosenberger G, Ludwig C, Rost HL, Aebersold R, Malmstrom L. aLFQ: an
    R-package for estimating absolute protein quantities from label-free
    LC-MS/MS proteomics data. Bioinformatics. 2014 Sep 1;30(17):2511-3.
    doi: 10.1093/bioinformatics/btu200.

  - Suomi T., Corthals G., Nevalainen O.S., and Elo L.L. (2015). Using
    Peptide-Level Proteomics Data for Detecting Differentially Expressed
    Proteins. J Proteome Res. Nov 6;14(11):4564-70. doi:
    10.1021/acs.jproteome.5b00363.

  - Suomi, T. and Elo L.L. (2017). Enhanced differential expression
    statistics for data-independent acquisition proteomics" Scientific
    Reports 7, Article number: 5869.doi:10.1038/s41598-017-05949-y

---
output: github_document
---

<!-- README.md is generated from README.Rmd. Please edit that file -->

```{r, echo = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.path = "man/figures/"
)
```

# SWATH2stats 

This package is intended to transform extracted SWATH/DIA data from the OpenSWATH or other  (e.g. Spectronaut) software into a format directly-usable by statistics packages (e.g. mapDIA, PECA, MSstats) while performing filtering, annotation and FDR assessment if necessary.

## Analyzing SWATH/DIA data

How to extract SWATH/DIA data before using SWATH2stats with OpenSWATH can be found here: <http://openswath.org>


## Usage

SWATH2stats is a Bioconductor package. Go to <https://www.bioconductor.org/packages/release/bioc/html/SWATH2stats.html> to see all information related to installation. Importantly, there exist both a release and development version of every Bioconductor R package. 


## Contribution

Please feel free to comment and post issues or pull requests on github (direct link in upper right corner). 


## Publication

The publication describing this package you can find here: <https://doi.org/10.1371/journal.pone.0153160>



## References

+ Blattmann P, Heusel M, Aebersold R. SWATH2stats: An R/Bioconductor Package to Process and Convert
  Quantitative SWATH-MS Proteomics Data for Downstream Analysis Tools. PLoS ONE 11(4): e0153160 (2016). doi: 10.1371/journal.pone.0153160.
  
+ Rost HL, Rosenberger G, Navarro P, Gillet L, Miladinovic SM, Schubert OT, Wolski W, Collins BC, Malmstrom J, Malmstrom L, Aebersold R. OpenSWATH enables automated, targeted analysis of data-independent acquisition MS data. Nature Biotechnology. 2014 Mar;32(3):219-23. doi: 10.1038/nbt.2841.
  
+ Choi M, Chang CY, Clough T, Broudy D, Killeen T, MacLean B, Vitek O. MSstats: an R package for statistical analysis of quantitative mass spectrometry-based proteomic experiments.Bioinformatics. 2014 Sep 1;30(17):2524-6. doi: 10.1093/bioinformatics/btu305.
  
+ Rosenberger G, Ludwig C, Rost HL, Aebersold R, Malmstrom L. aLFQ: an R-package for estimating absolute protein quantities from label-free LC-MS/MS proteomics data. Bioinformatics. 2014 Sep 1;30(17):2511-3. doi: 10.1093/bioinformatics/btu200.
  
+ Suomi T., Corthals G., Nevalainen O.S., and Elo L.L. (2015). Using Peptide-Level Proteomics Data for Detecting Differentially Expressed Proteins. J Proteome Res. Nov 6;14(11):4564-70. doi: 10.1021/acs.jproteome.5b00363.
  
+ Suomi, T. and Elo L.L. (2017). Enhanced differential expression statistics for data-independent acquisition proteomics" Scientific Reports 7, Article number:
5869.doi:10.1038/s41598-017-05949-y

#' Filter openSWATH output for proteins that are identified by a minimum of n independent peptides.
#'
#' This function removes entries mapping to proteins that are identified by less
#' than n_peptides.
#' Removing single-hit proteins from an analysis can significantly increase the
#' sensitivity under strict protein fdr criteria, as evaluated by
#' e.g. assess_fdr_overall.
#'
#' @param data Data table that is produced by the OpenSWATH/iPortal workflow.
#' @param n_peptides Number of minimal number of peptide IDs associated with a
#'   protein ID in order to be kept in the dataset.
#' @param protein_col Column with protein identifiers. Default: ProteinName
#' @param peptide_col Column with peptide identifiers. Default: Peptide.Sequence or FullPeptideName
#' @param rm.decoy  Option to remove the decoys during filtering.
#' @return Returns the filtered data frame with only peptides that map to
#'   proteins with >= n_peptides peptides.
#' @author Moritz Heusel, Peter Blattmann
#' @examples{
#'  data("OpenSWATH_data", package="SWATH2stats")
#'  data("Study_design", package="SWATH2stats")
#'  data <- sample_annotation(OpenSWATH_data, Study_design)
#'  data.filtered <- filter_mscore_freqobs(data, 0.01,0.8)
#'  data.max <- filter_on_max_peptides(data.filtered, 5)
#'  data.min.max <- filter_on_min_peptides(data.max, 3)
#'  }
#' @export
filter_on_min_peptides <- function(data, 
                                   n_peptides, 
                                   protein_col = "ProteinName",
                                   peptide_col = c("Peptide.Sequence", "FullPeptideName"),
                                   rm.decoy = TRUE) {
    data <- unifyProteinGroupLabels(data)
    if (isTRUE(rm.decoy)) {
        data <- removeDecoyProteins(data)
    }
    
    # select valid columns
    columns <- validate_columns(data, list(Protein = protein_col,
                                        Peptide = peptide_col))
    
    # rename columns
    colnames(data) <- gsub(columns[["Protein"]], "PROTEIN", colnames(data))
    colnames(data) <- gsub(columns[["Peptide"]], "PEPTIDE", colnames(data))
    
    data.prot.pep <- unique(data[, c("PROTEIN", "PEPTIDE")])

    data.prot.pep.n <- tapply(data.prot.pep$PEPTIDE, 
                              data.prot.pep$PROTEIN,
                              length)

    prot.pep.names <- names(data.prot.pep.n[data.prot.pep.n >= n_peptides & 
                                                !is.na(data.prot.pep.n)])
    data.filtered <- data[data$PROTEIN %in% prot.pep.names, ]

    message("Before filtering: ", "\n", 
            "  Number of proteins: ", length(unique(data$PROTEIN)), "\n", 
            "  Number of peptides: ", length(unique(data$PEPTIDE)), "\n\n",
            "Percentage of peptides removed: ", round((length(unique(data$PEPTIDE)) -
            length(unique(data.filtered$PEPTIDE)))/length(unique(data$PEPTIDE)) *
            100, digits = 2), "%", "\n\n", 
            "After filtering: ", "\n", "  Number of proteins: ",
            length(unique(data.filtered$PROTEIN)), "\n", 
            "  Number of peptides: ", length(unique(data.filtered$PEPTIDE)), "\n")
   
    # rename columns to original names
    colnames(data.filtered) <- gsub("PROTEIN", columns[["Protein"]], colnames(data.filtered))
    colnames(data.filtered) <- gsub("PEPTIDE", columns[["Peptide"]], colnames(data.filtered))
    
    return(data.filtered)
}

context("data conversion")

test_that("data conversion", {
  data(OpenSWATH_data, package="SWATH2stats")
  data <- OpenSWATH_data
  data(Study_design, package="SWATH2stats")

  data <- sample_annotation(data, Study_design)
  data.filtered.mscore <- filter_mscore_freqobs(data, 0.01, 0.8)

  # test if gives warning when data is in data.table format
  data2 <- data.table::data.table(data)
  expect_error(sample_annotation(data2, Study_design))

  data.proteotypic <- filter_proteotypic_peptides(data.filtered.mscore)
  expect_that(filter_proteotypic_peptides(data.filtered.mscore), shows_message("Number of proteins detected: 11"))
  expect_that(filter_proteotypic_peptides(data.filtered.mscore), shows_message("Number of proteotypic peptides detected: 131"))
  expect_that(dim(data.proteotypic), equals(c(972,62)))


  data.all <- filter_all_peptides(data.filtered.mscore)
  expect_that(filter_proteotypic_peptides(data.filtered.mscore), shows_message("Number of proteins detected: 11"))
  expect_that(dim(data.all), equals(c(984,62)))

  data.max <- filter_on_max_peptides(data.filtered.mscore, 5)
  expect_that(dim(data.max), equals(c(300,62)))
  expect_that(filter_on_max_peptides(data.filtered.mscore, 5), shows_message("Before filtering: \n  Number of proteins: 10\n  Number of peptides: 133\n\nPercentage of peptides removed: 69.17%\n\nAfter filtering: \n  Number of proteins: 10\n  Number of peptides: 41\n\n"))

  data.min <- filter_on_min_peptides(data.filtered.mscore, 5)
  expect_that(dim(data.min), equals(c(918,62)))

  data.python <- convert4pythonscript(data.max)
  expect_true(identical(grep("UniMod\\:", data.max[,"aggr_Fragment_Annotation"]), grep("UniMod\\_", data.python[,"aggr_Fragment_Annotation"])))
  expect_false(identical(grep("UniMod\\:", data.max[,"aggr_Fragment_Annotation"]), grep("UniMod\\_", data.max[,"aggr_Fragment_Annotation"])))
  expect_true(identical(grep("UniMod\\:", data.max[,"FullPeptideName"]), grep("UniMod\\_", data.python[,"FullPeptideName"])))
  expect_false(identical(grep("UniMod\\:", data.max[,"FullPeptideName"]), grep("UniMod\\_", data.max[,"FullPeptideName"])))

  raw <- disaggregate(data.max)
  expect_true(subset(raw, FragmentIon == "1069078_FIIDPAAVITGR_2" & Run == 6)[,"Intensity"] == 188569)
  expect_true(subset(raw, FragmentIon == "279546_FPSIVGVAR_2" & Run == 5)[,"Intensity"] == 10805)

  # test error if transitions and data don't match
  data.max.test1 <- data.max
  data.max.test1$aggr_Peak_Area <- as.character(data.max.test1$aggr_Peak_Area)
  data.max.test1[data.max.test1$Run == 6 & data.max.test1$BioReplicate == 3 & data.max.test1$Peptide_Charge == "FIIDPAAVITGR 2","aggr_Peak_Area"] <- "188569.000000;81947.000000;77295.000000;50218.000000"
  expect_error(disaggregate(data.max.test1))
  #expect_that(disaggregate(data.max.test1), throws_error("Error in disaggregate(data.max.test1) : \n  The number of transitions annotated and measured do not match in the following transitions:\n 1069078_FIIDPAAVITGR_2, 1069082_FIIDPAAVITGR_2, 1069080_FIIDPAAVITGR_2, 1069071_FIIDPAAVITGR_2, 1069084_FIIDPAAVITGR_2, 1069076_FIIDPAAVITGR_2\n", fixed=TRUE))
  expect_that(disaggregate(data.max.test1), throws_error("The number of transitions annotated and measured do not match in the following transitions:\n 1069078_FIIDPAAVITGR_2, 1069082_FIIDPAAVITGR_2, 1069080_FIIDPAAVITGR_2, 1069071_FIIDPAAVITGR_2, 1069084_FIIDPAAVITGR_2, 1069076_FIIDPAAVITGR_2", fixed=FALSE))
  
  # test if spectral library does not contain same number of transitions
  data.max.test2 <- data.max.test1
  data.max.test2$aggr_Fragment_Annotation <- as.character(data.max.test2$aggr_Fragment_Annotation)
  data.max.test2[data.max.test1$Run == 6 & data.max.test1$BioReplicate == 3 & data.max.test1$Peptide_Charge == "FIIDPAAVITGR 2","aggr_Fragment_Annotation"] <- "1069078_FIIDPAAVITGR_2;1069082_FIIDPAAVITGR_2;1069080_FIIDPAAVITGR_2;1069071_FIIDPAAVITGR_2"
  expect_that(disaggregate(data.max.test2), shows_message("The library contains between 4 and 6 transitions per precursor.", fixed=TRUE))
  expect_that(dim(disaggregate(data.max.test2)), equals(c(1798,10)))

  data.MSstats <- convert4MSstats(raw)
  expect_that(dim(data.MSstats), equals(c(1800,10)))

  data.mapDIA <- convert4mapDIA(raw, RT = TRUE)
  expect_that(dim(data.mapDIA), equals(c(300,10)))

  data.aLFQ <- convert4aLFQ(raw)
  expect_that(dim(data.aLFQ), equals(c(1800,8)))
  
  expect_warning(convert4aLFQ(raw[c(1:6),]), "The aLFQ package should only be used with transition-level data.")
  
  # test if warning is displayed when there are several values for a data point
  raw2 <- raw
  raw2$Intensity <- raw$Intensity + 20
  raw2 <- rbind(raw[raw$FragmentIon == "1069078_FIIDPAAVITGR_2" & raw$Condition %in% c("Hela_Treatment"),], raw2)
  expect_warning(convert4mapDIA(raw2, RT = TRUE), "Data contains several intensity values per condition")

  # test if warning is displayed when column is missing
  raw2 <- raw
  raw2$RT <- NULL
  expect_warning(convert4mapDIA(raw2, RT = TRUE))
  
  # test if extraction of Precursor charge works
  raw2 <- raw
  raw2$PrecursorCharge <- NULL
  expect_message(convert4MSstats(raw2), 
                 "Missing columns: PrecursorCharge")
  data.MSstats <- convert4MSstats(raw2)
  expect_true(all.equal(data.MSstats[,"PrecursorCharge"], gsub(".*_([[:digit:]])$", "\\1", data.MSstats[, "FragmentIon"])))
  
  # test if negative values are replaced by NA
  raw2 <- raw
  raw2[raw2$FragmentIon == "1069078_FIIDPAAVITGR_2" & raw2$Run == 3, "Intensity"] <- -100
  data.MSstats <- convert4MSstats(raw2)
  expect_warning(convert4MSstats(raw2), 
                 "Negative intensity values were replaced by NA")
  expect_true(is.na(data.MSstats[data.MSstats$FragmentIon == "1069078_FIIDPAAVITGR_2" & data.MSstats$Run == 3, "Intensity"]))
  

})


test_that("non-proteotypic peptides", {
  data(OpenSWATH_data, package="SWATH2stats")
  data <- OpenSWATH_data
  data$ProteinName <- gsub("1/Protein6", "2/Protein6/DECOY_ProteinX", data$ProteinName)
  data$ProteinName <- gsub("1/Protein5", "2/DECOY_ProteinX/Protein6", data$ProteinName)
  
  expect_true(sum(data$ProteinName == "2/Protein6/DECOY_ProteinX")>1)
  expect_true(sum(data$ProteinName == "2/DECOY_ProteinX/Protein6")>1)
  
  data <- unifyProteinGroupLabels(data)
  
  expect_false(sum(data$ProteinName == "2/Protein6/DECOY_ProteinX")>1)
  expect_true(sum(data$ProteinName == "2/DECOY_ProteinX/Protein6")>1)
  expect_false(sum(data$ProteinName == "1/Protein6")>1)
  
  data <- removeDecoyProteins(data)
  
  expect_false(sum(data$ProteinName == "2/Protein6/DECOY_ProteinX")>1)
  expect_false(sum(data$ProteinName == "2/DECOY_ProteinX/Protein6")>1)
  expect_true(sum(data$ProteinName == "1/Protein6")>1)
})

#' Validate columns for a data.frame
#'
#' This function looks at the different possible column names given and chooses 
#' the one present in a data.frame. If none of the column names fit or if 
#' multiple names fit the function stops with an appropriate error  message.
#' The functions returns a list with the column names existing that can be used.
#' 
#' @name validate_columns
#' @param data data.frame to check for columns.
#' @param columns List of column names to be checked if they exist. 
#' @param verbose Logical if message should be printed. Default = FALSE
#' @return Returns list of columns that are present
#' @author Peter Blattmann
#' @examples{
#'  validate_columns(cars, list(Speed = c("speed")))
#'  
#'  # if out of two possible column one exists
#'  validate_columns(cars, list(Speed = c("speed", "velocity")))
#'  validate_columns(cars, list(Speed = c("speed", "velocity")), verbose = TRUE)
#'  
#'  }
#' @export
validate_columns <- function(data, columns, verbose = FALSE){
  columns_checked <- columns
  for(i in seq_len(length(columns))){
    if(sum(columns[[i]] %in% colnames(data) == 1)){
      if(length(columns[[i]]) > 1){
        columns_checked[[i]] <- columns[[i]][columns[[i]] %in% colnames(data)]
        if(verbose){
          message(paste0(columns_checked[[i]], " is used as ", names(columns[i]), " column."))
        }
      } else (
        columns_checked[i] <- columns[i])
    } else if(sum(columns[[i]]  %in% colnames(data) == 0)){
      if(is.null(names(columns[i]))){
        stop(paste0("Column ", i, " does not exist in data"))
      } else (stop(paste0(names(columns[i]), " column does not exist in data")))
    } else if(sum(columns[[i]]  %in% colnames(data) > 1)){
      if(is.null(names(columns[i]))){
        stop(paste0("Both names from column ", i, 
                    " exist in data. Please specify which to use"))
      } else (stop(paste0("Both names from the ", names(columns[i]), 
                          " column exist in data. Please specify which to use")))
    }
  }
  return(columns_checked)
}

#' Establish connection to biomaRt database
#'
#' This function establishes a connection to a biomart database.
#'
#' @param species  The species of the protein identifiers in the term used by
#'   biomaRt (e.g. "hsapiens_gene_ensembl", "mmusculus_gene_ensembl",
#'   "drerio_gene_ensembl", etc.)
#' @param ensembl.path  Ensembl host to connect to. Default: www.ensembl.org
#' @param mart   The type of mart (e.g. "ENSEMBL_MART_ENSEMBL", etc.)
#' @param verbose print a summary of the ensembl connection.
#' @return Connection for performing biomart queries.
#' @author Peter Blattmann
#' @examples{
#'  data_table <- data.frame(Protein = c("Q01581", "P49327", "2/P63261/P60709"),
#'                           Abundance = c(100, 3390, 43423))
#'  mart <- convert_protein_ids(data_table)
#'  }
#' @importFrom biomaRt useMart listDatasets listMarts
#' @export
load_mart <- function(species, 
                      ensembl.path = "www.ensembl.org", 
                      mart, 
                      verbose = FALSE) {
    dataset.mart <- species
    ensembl.mart <- useMart(mart, dataset = dataset.mart, host = ensembl.path)

    list.datasets <- listDatasets(ensembl.mart)
    list.datasets[which(list.datasets == dataset.mart), "version"]
    list.marts <- listMarts(ensembl.mart, host = ensembl.path)
    list.marts[which(list.marts == mart), "version"]
    
    if (verbose) {
        sink(file = paste(Sys.Date(), species, "Ensembl_Version.txt", sep = "_"))
        writeLines(c(paste("Species:", species), 
                     paste("Host:", ensembl.path), 
                     paste("Date:", Sys.Date()), 
                     paste("Dataset:", list.datasets[which(list.datasets == dataset.mart), "version"]),
                     paste("Version:", list.marts[which(list.marts == mart), "version"])))
        sink()
        sink(type = "message")
    }
    return(ensembl.mart)
}

#' Adds gene symbols to a table
#' 
#' Gather gene symbols from biomart and add them to a data frame.
#'
#' @note Protein identifiers from shared peptides should be separated by a forward
#' slash. The host of archived ensembl databases can be introduced as well
#' (e.g. "dec2017.archive.ensembl.org")
#'
#' @param data_table A data frame or file name.
#' @param gene_ID_table A table to match gene identifiers against
#' @param column_name The column name where the original protein identifiers
#'   are present.
#' @param ID1 The type of the original protein identifiers
#'   (e.g. "uniprotswissprot", "ensembl_peptide_id").
#' @param ID2   The type of the converted protein identifiers
#'   (e.g. "hgnc_symbol", "mgi_symbol", "external_gene_name").
#' @param id.separator   Separator between protein identifiers of shared
#'   peptides.
#' @param copy_nonconverted  Option defining if the identifiers that cannot be
#'   converted should be copied.
#' @return Returns the data frame with an added column of the converted protein identifiers.
#' @author Peter Blattmann
#' @examples {
#'   gene_ID_table <- data.frame(uniprotswissprot = c("Q01581", "P49327", "P60709"), 
#'                               hgnc_symbol = c("HMGCS1", "FASN", "ACTB"))
#'   data_table <- data.frame(Protein = c("Q01581", "P49327", "2/P63261/P60709"), 
#'                            Abundance = c(100, 3390, 43423))
#'   add_genesymbol(data_table, gene_ID_table)
#'   }
#' @importFrom data.table fread
#' @importFrom biomaRt getBM
#' @importFrom utils write.table
#' @export
add_genesymbol <- function(data_table, 
                           gene_ID_table, 
                           column_name = "Protein", 
                           ID1 = "uniprotswissprot",
                           ID2 = "hgnc_symbol", 
                           id.separator = "/", 
                           copy_nonconverted = TRUE) {
    # remove column if it already exists
    if (ID2 %in% colnames(data_table)) {
        data_table[, ID2] <- NULL
    }
    gene_ID_table[, ID2] <- as.character(gene_ID_table[, ID2])
    data_table <- merge(data_table, gene_ID_table, by.x = column_name, 
                        by.y = ID1,all.x = TRUE, sort = FALSE)
    # annotate the shared peptides
    .ids <- which(is.na(data_table[, ID2]))
    .ids <- intersect(.ids, grep(id.separator, data_table[, column_name]))

    for (i in .ids) {
        .Protein <- as.character(data_table[i, column_name])
        .Protein.single <- unlist(strsplit(.Protein, id.separator))

        # remove number in front of shared peptides
        if (!is.na(suppressWarnings(as.numeric(.Protein.single[1]))) & 
            nchar(.Protein.single[1]) <= 3) {
            .Protein.single <- .Protein.single[-1]
        }
        .Protein.new <- .Protein
        for (k in .Protein.single[.Protein.single %in% gene_ID_table[, ID1]]) {
            .Protein.new <- gsub(k, gene_ID_table[gene_ID_table[, ID1] == k, ID2],
                .Protein.new)
        }
        data_table[data_table[, column_name] == .Protein, ID2] <- .Protein.new

        if (!copy_nonconverted) {
            for (k in .Protein.single[!(.Protein.single %in% gene_ID_table[, ID1])]) {
                .Protein.new <- gsub(paste(id.separator, k, sep = ""), "", .Protein.new)
                .Protein.new <- gsub(paste(k, id.separator, sep = ""), "", .Protein.new)
            }
            data_table[data_table[, column_name] == .Protein, ID2] <- .Protein.new
        }
    }

    # add non converted IDs
    non_converted <- (is.na(data_table[,ID2]) | data_table[,ID2] =="")
    if (sum(non_converted)) {
        non_converted_ids <- unique(data_table[non_converted,column_name])
        if (sum(non_converted) > 20) {
            message("The following ", sum(non_converted), 
                    " identifiers were not converted and will be copied (the first 20 are shown): ",
                    paste(non_converted_ids[seq_len(20)], collapse = ", "))
        } else {
            message("The following identifiers were not converted and will be copied: ",
                    paste(non_converted_ids, collapse = ", "))
        }
        if (copy_nonconverted) {
            for (i in which(non_converted)) {
                data_table[i, ID2] <- data_table[i, column_name]
            }
        }
    }

    # bring gene_symbol column in front
    data_table <- data_table[, c(ID2, colnames(data_table)[seq(length(colnames(data_table)) - 1)])]

    return(data_table)
}

#' Convert protein ids
#'
#' This function renames protein ids in a data frame or file
#'
#' @param data_table   A data frame or file name.
#' @param column_name  The column name where the original protein identifiers are present.
#' @param species  The species of the protein identifiers in the term used by
#'   biomaRt (e.g. "hsapiens_gene_ensembl", "mmusculus_gene_ensembl",
#'   "drerio_gene_ensembl", etc.)
#' @param host  Path of the biomaRt database (e.g. "www.ensembl.org",
#'   "dec2017.archive.ensembl.org").
#' @param mart  The type of mart (e.g. "ENSEMBL_MART_ENSEMBL", etc.)
#' @param ID1   The type of the original protein identifiers
#'   (e.g. "uniprotswissprot", "ensembl_peptide_id").
#' @param ID2   The type of the converted protein identifiers
#'   (e.g. "hgnc_symbol", "mgi_symbol", "external_gene_name").
#' @param id.separator   Separator between protein identifiers of shared
#'   peptides.
#' @param copy_nonconverted   Option defining if the identifiers that cannot be
#'   converted should be copied.
#' @param verbose   Option to write a file containing the version of the
#'   database used.
#' @return The data frame with an added column of the converted protein
#'   identifiers.
#' @note Protein identifiers from shared peptides should be separated by a
#'   forward slash. The host of archived ensembl databases can be introduced as
#'   well (e.g. "dec2017.archive.ensembl.org")
#' @author Peter Blattmann
#' @examples
#'  \dontrun{
#'   data_table <- data.frame(
#'        "Protein" = c("Q01581", "P49327", "2/P63261/P60709"),
#'        "Abundance" = c(100, 3390, 43423))
#'   convert_protein_ids(data_table)
#' }
#' @importFrom methods is
#' @export
convert_protein_ids <- function(data_table, 
                                column_name = "Protein", 
                                species = "hsapiens_gene_ensembl",
                                host = "https://www.ensembl.org", 
                                mart = "ENSEMBL_MART_ENSEMBL", 
                                ID1 = "uniprotswissprot",
                                ID2 = "hgnc_symbol", 
                                id.separator = "/", 
                                copy_nonconverted = TRUE, 
                                verbose = FALSE) {

    if (is(data_table, "data.frame")) {
        type <- "data.frame"
    } else {
        type <- "file"
        data_table <- data.frame(fread(file, sep = "\t", header = TRUE))
    }

    if (!(column_name %in% colnames(data_table))) {
        stop("Column name does not exist in data. ", paste(colnames(data_table),
            collapse = ", "))
    }

    # obtain Identifiers to map
    IDs <- unique(as.character(data_table[, column_name]))

    # separate non-proteotypic peptides
    IDs.np <- IDs[grep(id.separator, IDs)]
    IDs.np <- gsub("^[[:digit:]]+/", "", IDs.np)
    IDs.np <- unlist(strsplit(IDs.np, id.separator))

    # remove contaminant peptides
    IDs <- IDs[grep("^CONT_", IDs, invert = TRUE)]
    IDs.np <- IDs.np[grep("^CONT_", IDs.np, invert = TRUE)]

    IDs <- unique(c(IDs, IDs.np))
    length(IDs)

    organism.mart <- load_mart(species, host, mart)
    gene_ID_table <- getBM(attributes = c(ID2, ID1), filters = c(ID1), 
                           values = IDs, mart = organism.mart)

    n.ids <- table(gene_ID_table[, ID1])
    n.ids.multiple <- names(n.ids[n.ids > 1])
    for (i in n.ids.multiple) {
        new.id <- paste(gene_ID_table[gene_ID_table[, ID1] == i, ID2], 
                        collapse = id.separator)
        gene_ID_table <- gene_ID_table[gene_ID_table[, ID1] != i, ]
        new.id.df <- data.frame(uniprotswissprot = i, hgnc_symbol = new.id)
        colnames(new.id.df) <- c(ID1, ID2)
        gene_ID_table <- rbind(gene_ID_table, new.id.df)
    }

    data_table_output <- add_genesymbol(data_table, gene_ID_table, 
                                        column_name, ID1, ID2, 
                                        id.separator, copy_nonconverted)

    if (type == "file") {
        file.name <- gsub("\\..*", "", file)
        file.ext <- gsub(".*\\.", "", file)
        write.table(data_table_output, 
                    paste(file.name, "_annotated.", file.ext, sep = ""), 
                    quote = FALSE, row.names = FALSE, sep = "\t")
    }

    if (type == "data.frame") {
        data_table_output
    }
}

context("biomart")

test_that("convert_ids", {
  data_table <- data.frame(Protein = c("Q01581", "P49327", "2/P63261/P60709"), 
                           Abundance = c(100, 3390, 43423))
  result <- convert_protein_ids(data_table)
  expect_that(result[which(result$Protein == "Q01581"), "hgnc_symbol"], equals("HMGCS1"))
  expect_that(result[which(result$Protein == "P49327"), "hgnc_symbol"], equals("FASN"))
 })

#' Removes decoy proteins from the protein group label
#'
#' There exist peptides annotated as protein groups with
#' 2/ProteinA/DECOY_ProteinB. However these are in principal proteotypic
#' peptides and should be annoated 1/ProteinA. This function changes these
#' labels accordingly. The subfunction rmDecoyProt removes the Decoy protein,
#' calling removeDecoyProteins also changes the nubmer before the protein group
#' accordingly.
#'
#' @param data A data frame containing SWATH data.
#' @param column Column to query for decoy string
#' @param decoy_string String defining a decoy. Default: DECOY
#' @return Returns a data frame with changed protein labels
#' @author Moritz Heusel
#' @examples
#'  data("OpenSWATH_data", package="SWATH2stats")
#'  data("Study_design", package="SWATH2stats")
#'  data <- sample_annotation(OpenSWATH_data, Study_design)
#'  data.filtered.decoy <- filter_mscore(data, 0.01)
#'  data.2 <- removeDecoyProteins(data.filtered.decoy)
#' @export
removeDecoyProteins <- function(data,
                                column = "ProteinName",
                                decoy_string = "DECOY") {
    ids <- grep(decoy_string, data[,column])
    identifiers <- data[ids, column]
    identifiers_split <- strsplit(as.character(identifiers), "/")
    identifiers_split_removed <- lapply(identifiers_split, rmDecoyProt)
    identifiers_removed <- vapply(identifiers_split_removed, function(x) {
        paste(x, collapse = "/")
    }, "a")
    data[ids, "ProteinName"] <- identifiers_removed
    return(data)
}

#' Subfunction to remove decoys
#'
#' @param x proteinname string to query.
#' @param decoy_string String defining a decoy
#' @return returns string without elements containing the decoy string
rmDecoyProt <- function(x,
                        decoy_string = "DECOY") {
    ids <- grep(decoy_string, x)
    ids.sel <- grep(decoy_string, x, invert = TRUE)
    x[1] <- as.numeric(x[1]) - length(ids)
    x <- x[ids.sel]
    return(x)
}

#' Convert table into the format expected by aLFQ.
#'
#' This function selects the columns necessary for the aLFQ R package.
#'
#' @param data A data frame containing the SWATH data in transition-level
#'   format
#' @param annotation Option to indicate if the data has been annotated, i.e. if
#'   the columns Condition, Replicate, Run are present. If option
#'   is set to true it will write a new run_id as a string of the
#'   combination of these three columns.
#' @param check_transitions Option if number of transitions should be checked.
#'   As input only transition-level data should be used and
#'   therefore this is checked. However, this makes the function
#'   slow and herewith be omitted.
#' @return Returns a data frame in the appropriate format for aLFQ.
#' @author Peter Blattmann
#' @examples{
#'  data("OpenSWATH_data", package="SWATH2stats")
#'  data("Study_design", package="SWATH2stats")
#'  data <- SWATH2stats::sample_annotation(OpenSWATH_data, Study_design, verbose=TRUE)
#'  data.filtered.decoy <- filter_mscore(data, 0.01)
#'  raw <- disaggregate(data.filtered.decoy)
#'  data.aLFQ <- convert4aLFQ(raw)
#'  }
#' @importFrom stats aggregate median
#' @export

convert4aLFQ <- function(data, 
                         annotation = TRUE, 
                         check_transitions = TRUE) {

    if (annotation == TRUE) {
        data <- data[, c("ProteinName", "PeptideSequence", "FragmentIon", 
                         "NakedSequence", "PrecursorCharge", "Intensity", 
                         "Condition", "BioReplicate", "Run")]
        data$run_id <- paste(data$Condition, data$BioReplicate, data$Run, sep = "_")
    }

    colnames(data) <- gsub("ProteinName", "protein_id", colnames(data))
    colnames(data) <- gsub("PrecursorCharge", "precursor_charge", colnames(data))
    colnames(data) <- gsub("FragmentIon", "transition_id", colnames(data))
    colnames(data) <- gsub("Intensity", "transition_intensity", colnames(data))
    colnames(data) <- gsub("PeptideSequence", "peptide_id", colnames(data))
    colnames(data) <- gsub("NakedSequence", "peptide_sequence", colnames(data))

    data$transition_id <- gsub("_run[[:digit:]]*$", "", data$transition_id)
    data$transition_id <- paste(data$peptide_sequence, data$transition_id, sep = " ")

    data$concentration <- "?"

    data <- data[, c("run_id", "protein_id", "peptide_id", "transition_id", 
                     "peptide_sequence", "precursor_charge", 
                     "transition_intensity", "concentration")]
    # check transitions
    data$protein_id <- factor(data$protein_id)
    data$peptide_id <- factor(data$peptide_id)
    data$transition_id <- factor(data$transition_id)

    if (check_transitions) {
        message("Checking the integrity of the transitions takes a lot of time. 
                To speed up consider changing the option.")
        data.agg <- aggregate(data[, c("transition_id")], by = list(data$peptide_id,
            data$run_id), length)
        if (median(data.agg$x) == 1) {
            warning("The aLFQ package should only be used with transition-level data.
              The data only contains one transition per peptide.")
        }
    }

    # convert back to character vector
    data$protein_id <- as.character(data$protein_id)
    data$peptide_id <- as.character(data$peptide_id)
    data$transition_id <- as.character(data$transition_id)

    return(data)
}

#'  Transforms column names to OpenSWATH column names
#'
#' This functions transforms the column names from a data frame in MSstats
#' format to a data frame with column names used by the OpenSWATH output. The
#' original table needs to contain at least the 10 columns defined by MSstats:
#' ProteinName, PeptideSequence, PrecursorCharge, Fragmentation, ProductCharge,
#' IsotopeLabelType, Condition, BioReplicate, Run, Intensity.)
#'
#' @param data A data frame containing the SWATH data in the MSstats format
#' @return The data frame in the appropriate format.
#' @references Choi M, Chang CY, Clough T, Broudy D, Killeen T, MacLean B, Vitek
#'   O. MSstats: an R package for statistical analysis of quantitative mass
#'   spectrometry-based proteomic experiments.Bioinformatics. 2014 Sep
#'   1;30(17):2524-6. doi: 10.1093/bioinformatics/btu305.
#' @author Peter Blattmann
#' @examples
#'  data("MSstats_data", package="SWATH2stats")
#'  transformed <- transform_MSstats_OpenSWATH(MSstats_data)
#' @export
transform_MSstats_OpenSWATH <- function(data) {
    colnames(data) <- gsub("\\.", "", colnames(data))

    missing_columns <- setdiff(c("ProteinName", "PeptideSequence", "PrecursorCharge",
        "FragmentIon", "ProductCharge", "IsotopeLabelType", "Condition", "BioReplicate",
        "Run", "Intensity"), colnames(data))

    if (length(missing_columns > 0)) {
        stop(paste("The data frame doesn't contain all required columns. The following columns are missing:\n",
            paste(missing_columns, collapse = ", ")))

    }

    add.colnames <- colnames(data)[!(colnames(data) %in% c("ProteinName", "PeptideSequence",
        "PrecursorCharge", "FragmentIon", "ProductCharge", "IsotopeLabelType", "Condition",
        "BioReplicate", "Run", "Intensity"))]

    data <- data[, c("ProteinName", "PeptideSequence", "PrecursorCharge", "FragmentIon",
        "ProductCharge", "IsotopeLabelType", "Condition", "BioReplicate", "Run",
        "Intensity", add.colnames)]
    colnames(data)[seq_len(10)] <- c("ProteinName", "FullPeptideName", "Charge",
        "FragmentIon", "ProductCharge", "IsotopeLabelType", "Condition", "BioReplicate",
        "Run", "Intensity")

    if (length(add.colnames) > 0) {
        message("Additional columns present in the data: ", paste(add.colnames, collapse = ", "))
    }

    if (!("mscore" %in% colnames(data))) {
        message("No column 'mscore' present in the data. This column is required for functions estimating FDR or filtering the data.")
    }
    return(data)

}

context("load data and annotate")

test_that("load data and annotate", {
  data(OpenSWATH_data, package="SWATH2stats")
  data <- OpenSWATH_data
  data(Study_design, package="SWATH2stats")

  # filter_openSWATH_output
  expect_that(dim(data)[2], equals(58))
  data <- reduce_OpenSWATH_output(data)
  expect_that(dim(data)[2], equals(13))

  # annotate data
  expect_that(dim(data)[2], equals(13))
  data <- sample_annotation(data, Study_design)

  filename.data <- unique(subset(data, Condition == "Hela_Treatment" & BioReplicate == 2 & Run == 4)[,"filename"])
  filename.design <- subset(Study_design, Condition == "Hela_Treatment" & BioReplicate == 2 & Run == 4)[,"Filename"]
  filename.design2 <- subset(Study_design, Condition == "Hela_Treatment" & BioReplicate == 1 & Run == 4)[,"Filename"]

  expect_true(length(grep(filename.design, filename.data))>0)
  expect_error(length(grep(filename.design2, filename.data))>0)
  
  Study_design2 <- Study_design
  Study_design2$Filename <- gsub("(peterb_L[[:digit:]]{6}).*", "\\1", Study_design2$Filename)
  
  # error that it has different number of file names
  expect_error(sample_annotation(data, Study_design2))
  
  # error that the strings in Filename are not unique
  Study_design3 <- Study_design
  Study_design3$Filename <- gsub("peterb_L150514_001_SW", "peterb_L150514", Study_design3$Filename)

  expect_error(sample_annotation(data, Study_design3))
  
  data <- sample_annotation(data, Study_design, verbose = TRUE)
  expect_message(sample_annotation(data, Study_design, verbose = TRUE), "peterb_J131223_043")
  expect_error(sample_annotation(data, Study_design, column_file = "Files"), "Column for filename is not present in data file")
  
  
})

test_that("load MSstats data and annotate", {
  data(MSstats_data, package="SWATH2stats")
  data(Study_design, package="SWATH2stats")
  
  MSstats_data2 <- MSstats_data
  MSstats_data2$mscore <- 0.01
  expect_message(transform_MSstats_OpenSWATH(MSstats_data), "No column 'mscore' present")
  expect_message(transform_MSstats_OpenSWATH(MSstats_data2), "Additional columns present in the data: FileName, mscore")
  
  expect_warning(sample_annotation(MSstats_data[,c(1:6,9:11)], Study_design, data_type = "MSstats", column_file = "FileName"),
                 "The number of sample annotation condition and filenames in data are not balanced")
  
  MSstats_data2$FragmentIon <- NULL
  expect_error(transform_MSstats_OpenSWATH(MSstats_data2), "The data frame doesn't contain all required columns.")
})

#' Filter OpenSWATH output table according to mscore and conditions.
#'
#' This function filters the SWATH data according to the m_score value, as well
#' as to the number of occurence in the data (requant) and within a condition
#' (condition).
#'
#' @param data A data frame containing SWATH data.
#' @param mscore Value that defines the mscore threshold according to which the
#'   data will be filtered.
#' @param n_replica Number of measurements within at least one condition that
#'   have to pass the mscore threshold for this transition.
#' @param peptide_col Column with peptide identifiers. Default: Peptide.Sequence or FullPeptideName
#' @param charge_col Column with peptide charge. Default: Charge
#' @param condition_col Column with conditions. Default: Condition
#' @param rm.decoy  Option to drop decoys from the data
#' @param mscore.col Defines the column from which to retrieve the m_score. 
#'    If you use JPP (Rosenberger, Bludau et al. 2017) this can be used to 
#'    select between Protein and transition_group m_score.
#' @return Data which has been filtered.
#' @author Peter Blattmann
#' @examples{
#'  data("OpenSWATH_data", package="SWATH2stats")
#'  data("Study_design", package="SWATH2stats")
#'  data <- sample_annotation(OpenSWATH_data, Study_design)
#'  data.filtered <- filter_mscore(data, 0.01)
#'  data.filtered <- filter_mscore_freqobs(data, 0.01, 0.8)
#'  data.filtered <- filter_mscore_condition(data, 0.01, 3)
#'  }
#' @importFrom data.table setkeyv data.table
#' @export
filter_mscore_condition <- function(data, 
                                    mscore, 
                                    n_replica, 
                                    peptide_col = c("Peptide.Sequence", "FullPeptideName"),
                                    charge_col = "Charge",
                                    condition_col = "Condition",
                                    rm.decoy = TRUE, 
                                    mscore.col = "m_score") {
    .N <- NULL  ## Avoid R CMD check shenanigans

    mscore.col <- JPP_update(data, mscore.col)
   
     ## decoys are removed if present
    if (sum(colnames(data) == "decoy") == 1 & isTRUE(rm.decoy)) {
      data <- data[data$decoy == 0, ]
    }
    
    #Select valid columns
    columns <- validate_columns(data, list(Peptide = peptide_col,
                                        Charge = charge_col,
                                        Condition = condition_col,
                                        Score = mscore.col))
    
    
    # a unique identifiers for each Precursor (e.g. ADF_3) is generated
    data$Peptide_Charge <- paste(data[,columns[["Peptide"]]], data[,columns[["Charge"]]])
    ## a unique identifiers for each Precursor and Condition is generated 
    data$Peptide_Charge_Condition <- paste(data$Peptide_Charge, data[,columns[["Condition"]]])

    ## only data that is below the indicated mscore is selected and then only 
    ## the unique data selected data.filtered <- subset(data, m_score <= mscore)
    data.filtered <- data[data[, columns[["Score"]]] <= mscore, ]
    data.filtered <- unique(data.filtered[, c("Peptide_Charge", 
                                              "Peptide_Charge_Condition",
                                              "aggr_Peak_Area")])
    data.filtered <- data.table::data.table(data.filtered)

    # aggr_Peak area is only used to have a unique column
    data.table::setkeyv(data.filtered,
                        cols = c("Peptide_Charge", "Peptide_Charge_Condition", 
                                 "aggr_Peak_Area"))
    
    ## number of occurrences of Precursor per Condition is calculated
    data.n <- data.filtered[, .N, by = "Peptide_Charge,Peptide_Charge_Condition"]

    ## only precursors that are present in more that n_replica are selected
    precursor.filtered <- data.n[data.n$N >= n_replica]
    precursor.filtered <- data.frame(
      "Peptide_Charge" = unique(precursor.filtered$Peptide_Charge))

    message("Fraction of peptides selected: ",
            signif(length(unique(precursor.filtered$Peptide_Charge)) /
                   length(unique(data$Peptide_Charge)),
                   digits = 2))

    # only data that is present in the precursor.filtered list is selected
    data.filtered <- merge(data, precursor.filtered,
                           by.x = "Peptide_Charge", by.y = "Peptide_Charge")

    message("Dimension difference: ", paste(dim(data) - dim(data.filtered), 
                                            collapse = ", "))
    return(data.filtered)
}

context("data processing")

test_that("filtering", {
  data(OpenSWATH_data, package="SWATH2stats")
  data <- OpenSWATH_data
  data(Study_design, package="SWATH2stats")

  data <- reduce_OpenSWATH_output(data)
  
  # test if checking for missing columns works
  data2 <- data
  data2$ProteinName <- NULL
  expect_warning(reduce_OpenSWATH_output(data2),"These columns are missing from the data:ProteinName")
  
  # sample_annotation
  data <- sample_annotation(data, Study_design)
  
  ## assess_decoy_rate
  assess_decoy_rate(data)
  expect_that(assess_decoy_rate(data), shows_message("Number of non-decoy identifiers: 273"))
  expect_that(assess_decoy_rate(data), shows_message("Number of decoy identifiers: 11"))
  expect_that(assess_decoy_rate(data), shows_message("Decoy rate: 0.0403"))

  # test if no decoys present
  data2 <- data
  data2$decoy <- NULL
  expect_error(assess_decoy_rate(data2), "There is no decoy column in the table")
  # test if no decoys present
  data2 <- data
  data2$FullPeptideName <- NULL
  expect_error(assess_decoy_rate(data2))
  
  ## filter_mscore
  data.filtered.mscore <- filter_mscore(data, 0.01)
  data.filtered.mscore <- filter_mscore(data, 0.001)
  data.filtered.mscore <- filter_mscore(data, 0.0001)

  expect_that(filter_mscore(data, 0.01), shows_message("Dimension difference: 931, 0"))
  expect_that(filter_mscore(data, 0.001), shows_message("Dimension difference: 1068, 0"))
  expect_that(filter_mscore(data, 0.0001), shows_message("Dimension difference: 1166, 0"))

  # filter mscore_requant
  data.filtered.mscore <- filter_mscore_freqobs(data, 0.01, 0.8)

  expect_that(filter_mscore_freqobs(data, 0.01, 0.8), shows_message("Treshold, peptides need to have been quantified in more conditions than: 4.8"))
  expect_that(filter_mscore_freqobs(data, 0.01, 0.8), shows_message("Fraction of peptides selected: 0.42"))
  expect_that(filter_mscore_freqobs(data, 0.01, 0.8), shows_message("Dimension difference: 1323, 0"))

  expect_message(filter_mscore_freqobs(data, 0.01), "Treshold, peptides need to have been quantified in more conditions than: 0")
  
  # filter mscore_conditions
  data.filtered.mscore <- filter_mscore_condition(data, 0.01, 3)
  expect_that(filter_mscore_condition(data, 0.01, 3), shows_message("Fraction of peptides selected: 0.47"))
  expect_that(filter_mscore_condition(data, 0.01, 3), shows_message("Dimension difference: 1209, 0"))

  # filter on_fdr
  expect_that(filter_mscore_fdr(data, FFT = 0.7, overall_protein_fdr_target=0.1, upper_overall_peptide_fdr_limit=0.05),
  shows_message("been removed from the returned data"))
  
  # test if remove decoy works
  data2 <- filter_mscore_fdr(data, FFT = 0.7, overall_protein_fdr_target=0.1, upper_overall_peptide_fdr_limit=0.05, rm_decoy=FALSE)
  data3 <- filter_mscore_fdr(data, FFT = 0.7, overall_protein_fdr_target=0.1, upper_overall_peptide_fdr_limit=0.05, rm_decoy=TRUE)
  
  expect_message(filter_mscore_fdr(data, FFT = 0.7, overall_protein_fdr_target=0.1, upper_overall_peptide_fdr_limit=0.05, rm_decoy=FALSE),
                 "The decoys have NOT been removed from the returned data")
  expect_true(length(grep("DECOY", data2$ProteinName)) > 1)
  expect_false(length(grep("DECOY", data3$ProteinName)) > 1)
  
  
  # test that error is returned if lowest m_score is from decoy
  data2 <- data
  data2 <- data2[data2$m_score != 0,]
  data2[data2$decoy == 1,"m_score"] <- data2[data2$decoy == 1,"m_score"]*1e-20
  expect_true(is.na(mscore4protfdr(data2, FFT = 0.7, fdr_target = 0.001)))
  expect_error(filter_mscore_fdr(data2, FFT = 0.7, overall_protein_fdr_target=0.1, upper_overall_peptide_fdr_limit=0.05), 
               "The overall_protein_fdr_target cannot be reached in this dataset")
  
  # test that local FDR is only calculated if enought decoys for each run
  data2 <- data
  data2 <- data2[data2$m_score != 0,]
  data2[data2$decoy == 1 & data2$m_score == 2,"m_score"] <- 0.01

  expect_message(filter_mscore_fdr(data, FFT = 0.7, overall_protein_fdr_target=0.1, upper_overall_peptide_fdr_limit=0.05), 
               "Individual run FDR quality of the peptides was not calculated\nas not every run contains a decoy")
  expect_message(filter_mscore_fdr(data2, FFT = 0.7, overall_protein_fdr_target=0.1, upper_overall_peptide_fdr_limit=0.05), 
                 "Individual run FDR quality of the peptides selected from")
  
              
})

test_that("count_analytes", {
  data(OpenSWATH_data, package="SWATH2stats")
  data <- OpenSWATH_data
  
  analysis <- count_analytes(data)
  expect_that(analysis[analysis$run_id == "0_210","FullPeptideName"], equals(261))
  expect_that(analysis[analysis$run_id == "0_221","FullPeptideName"], equals(271))
  
  analysis2 <- count_analytes(data[data$m_score < 1e-4,])
  expect_that(analysis[analysis$run_id == "0_177","transition_group_id"], equals(388))
  expect_that(analysis[analysis$run_id == "0_29","ProteinName"], equals(10))
})

#' Counts the analytes across the different injections
#'
#' This functions counts the number of different peakgroups, peptides and
#' proteins in different injections.
#'
#' @param data A data frame containing SWATH data.
#' @param column_levels  Columns in which the number of unique identifiers 
#'   should be counted.
#' @param column_by Column for which the different identifiers should be counted
#'   for, e.g. for the different injections.
#' @param rm_decoy Option to not remove decoy before counting.
#' @return Returns a data frame with the count of the different identifiers per
#'   e.g. injection.
#' @author Peter Blattmann
#' @examples{
#'  data("OpenSWATH_data", package="SWATH2stats")
#'  data("Study_design", package="SWATH2stats")
#'  data <- sample_annotation(OpenSWATH_data, Study_design)
#'  counts <- count_analytes(data)
#'  }
#' @importFrom stats aggregate
#' @export
count_analytes <- function(data, 
                           column_levels = c("transition_group_id", 
                                             "FullPeptideName", 
                                             "ProteinName"), 
                           column_by = "run_id", 
                           rm_decoy = TRUE) {
    if (sum(colnames(data) == "decoy") == 1 & isTRUE(rm_decoy)) {
        # data <- subset(data, decoy == 0)
        data <- data[data$decoy == 0, ]
    }

    data.n <- aggregate(data[, column_levels], 
                        by = list(data[, column_by]), 
                        function(x) length(unique(x)))
    colnames(data.n)[1] <- column_by
    return(data.n)
}

#' Find m_score cutoff to reach a desired FDR on assay level (over the entire
#' OpenSWATH/pyProphet output table)
#'
#' This function estimates the m_score cutoff required in a dataset to reach a
#' given overall assay level FDR.
#' It counts target and decoy assays at high resolution across the m_score
#' cutoffs and reports a useful m_score cutoff - assay FDR pair close to the
#' supplied fdr_target level over the entire dataset. The m_score cutoff is
#' returned by the function and can be used in the context of the filtering
#' functions, e.g.:
#' data.assayFDR1pc<-filter_mscore(data, mscore4assayfdr(data, fdr_target=0.01))
#' To arrive from decoy counts at an estimation of the false discovery rate
#' (false positives among the targets remaining at a given mscore cutoff) the
#' ratio of false positives to true negatives (decoys) (FFT) must be
#' supplied. It is estimated for each run individually by pyProphet and
#' contained in the pyProphet statistics [Injection_name]_full_stat.csv. As an
#' approximation, the FFTs of multiple runs are averaged and supplied as
#' argument FFT. For further details see the Vignette Section 1.3 and 4.1.
#' For FDR evaluations on peptide and protein level, please refer to functions
#' mscore4pepfdr and mscore4protfdr.
#'
#' @param data Annotated OpenSWATH/pyProphet data table. See function
#'   sample_annotation from this package.
#' @param FFT Ratio of false positives to true negatives, q-values from
#'   [Injection_name]_full_stat.csv in pyProphet stats output. As an
#'   approximation, the q-values of multiple runs are averaged and supplied as
#'   argument FFT. Numeric from 0 to 1. Defaults to 1, the most conservative
#'   value (1 Decoy indicates 1 False target).
#' @param fdr_target  Assay FDR target, numeric, defaults to 0.01. An m_score
#'   cutoff achieving an FDR < fdr_target will be selected.
#' Calculated as FDR = (TN*FFT/T); TN=decoys, T=targets, FFT=see above.
#' @param mscore.col Column name containing the computed m scores.
#' @return Returns the m_score cutoff selected to arrive at the desired FDR
#' @author Moritz Heusel
#' @examples
#'  data("OpenSWATH_data", package="SWATH2stats")
#'  data("Study_design", package="SWATH2stats")
#'  data <- sample_annotation(OpenSWATH_data, Study_design)
#'  chosen <- mscore4assayfdr(data, FFT=0.7, fdr_target=0.01)
#' @export
mscore4assayfdr <- function(data, 
                            FFT = 1, 
                            fdr_target = 0.01, 
                            mscore.col = "m_score") {
    mscore.col <- JPP_update(data, mscore.col)

    # generate high resolution mscore levels to assess mscore cutoff for a given
    # fdr_target
    mscore_levels_highres = 10^-(c(seq(2, 20, 0.05)))
    target.assays.highres <- NULL
    decoy.assays.highres <- NULL
    for (i in seq_len(length(mscore_levels_highres))) {
        target.assays.highres[i] <- length(unique(data[data$decoy == FALSE & data[,
            mscore.col] <= mscore_levels_highres[i], c("transition_group_id")]))
        decoy.assays.highres[i] <- length(unique(data[data$decoy == TRUE & data[,
            mscore.col] <= mscore_levels_highres[i], c("transition_group_id")]))
    }
    assay.fdr.highres <- (decoy.assays.highres/target.assays.highres) * FFT

    # pick mscore cutoff closest to (<=) fdr_target % peptide FDR & report
    mscore_chosen <- mscore_levels_highres[assay.fdr.highres <= fdr_target][1]
    assay_fdr_chosen <- assay.fdr.highres[assay.fdr.highres <= fdr_target][1]
    message("Target assay FDR: ", fdr_target, "\n")
    message("Required overall m-score cutoff:", signif(mscore_chosen, digits = 5),
        "\n", "achieving assay FDR =", signif(assay_fdr_chosen, digits = 3), "\n")
    return(mscore_chosen)
}

context("FDR assessment on annotated data")

test_that("FDR assessment on annotated data", {
  data(OpenSWATH_data, package="SWATH2stats")
  data(Study_design, package="SWATH2stats")
  data.FDR<-sample_annotation(OpenSWATH_data, Study_design)

  # assess_fdr_overall output test
  expect_message(assess_fdr_overall(data.FDR), ".csv")
  expect_message(assess_fdr_overall(data.FDR), ".pdf")
  expect_that(length(assess_fdr_overall(data.FDR)), equals(0))
  expect_that(length(assess_fdr_overall(data.FDR, output = "Rconsole")), equals(13))
  
  # assess_fdr_byrun output test
  expect_message(assess_fdr_byrun(data.FDR), ".csv")
  expect_message(assess_fdr_byrun(data.FDR), ".pdf")
  expect_message(assess_fdr_byrun(data.FDR), "assay")
  expect_message(assess_fdr_byrun(data.FDR), "peptide")
  expect_message(assess_fdr_byrun(data.FDR), "protein")

  # assess_fdr_byrun return test
  expect_that(round(mean(assess_fdr_byrun(data.FDR, output = "Rconsole")[1,,]),digits = 4), equals(229.3333))

  # mscore4(assay,pep,prot)fdr return tests
  expect_that(signif(mscore4assayfdr(data.FDR, fdr_target = 0.001), digits=5), equals(0.00017783))
  expect_that(signif(mscore4pepfdr(data.FDR, fdr_target = 0.002), digits=5), equals(0.00017783))
  expect_that(signif(mscore4protfdr(data.FDR, fdr_target = 0.05), digits=5), equals(0.00017783))

})

#' Assess assay, peptide and protein level FDR by run (for each MS_injection
#' separately) in OpenSWATH output table
#'
#' This function estimates the assay, peptide and protein FDR by run in an
#' OpenSWATH result table in dependence of a range of m_score cutoffs. The
#' results can be visualized and summarized by the associated method
#' plot.fdr_table().
#' It counts target and decoy assays (unique transition_group_id), peptides
#' (unique FullPeptideName) and proteins (unique ProteinName) in the OpenSWATH
#' output table in dependence of m-score cutoff, the useful m_score cutoff range
#' is evaluated for each dataset individually on the fly.
#' To arrive from decoy counts at an estimation of the false discovery rate
#' (false positives among the targets remaining at a given mscore cutoff) the
#' ratio of false positives to true negatives (decoys) (FFT) must be
#' supplied. It is estimated for each run individually by pyProphet and
#' contained in the pyProphet statistics [Injection_name]_full_stat.csv. As an
#' approximation, the FFTs of multiple runs are averaged and supplied as
#' argument FFT. For further details see the Vignette Section 1.3 and 4.1.
#' To assess fdr over the entire dataset, please refer to function
#' assess_fdr_overall. FDR is calculated as FDR = (TN*FFT/T); TN=decoys,
#' T=targets, FFT=see above.
#'
#' @param data Annotated OpenSWATH/pyProphet output table. Refer to function
#'   sample_annotation from this package for further information.
#' @param FFT Ratio of false positives to true negatives, q-values from
#'   [Injection_name]_full_stat.csv in pyProphet stats output. As an
#'   approximation, the q-values of multiple runs are averaged and supplied as
#'   argument FFT. Numeric from 0 to 1. Defaults to 1, the most conservative
#'   value (1 Decoy indicates 1 False target).
#' @param n_range  Option to set the number of magnitude for which the m_score
#'   threshold is decreased (e.g. n_range = 10, m-score from 0.1 until
#'   10^-10)^.
#' @param output Choose output type. "pdf_csv" creates the output as files in
#'   the working directory, "Rconsole" triggers delivery of the output to the
#'   console enabling further computation or custom plotting / output.
#' @param plot Logical, whether or not to create plots from the results (using
#'   the associated method plot.fdr_cube()
#' @param filename  Modify the basename of the result files if set.
#' @param output_mscore_levels Define m-score levels to plot and write the
#'   estimated FDR results.
#' @param score_col Column that contains the score. Default. m_score 
#' @return Returns an array of target/decoy identification numbers and
#'   calculated FDR values at different m-score cutoffs.
#' @author Moritz Heusel
#' @examples{
#'  data("OpenSWATH_data", package="SWATH2stats")
#'  data("Study_design", package="SWATH2stats")
#'  data <- sample_annotation(OpenSWATH_data, Study_design)
#'  assessed <- assess_fdr_byrun(data, FFT=0.7, output="Rconsole", plot=TRUE,
#'                               filename="Testoutput_assess_fdr_byrun")
#'  summary(assessed)
#'  }
#' @importFrom utils write.csv
#' @export

assess_fdr_byrun <- function(data, 
                             FFT = 1, 
                             n_range = 20, 
                             output = "pdf_csv", 
                             plot = TRUE,
                             filename = "FDR_report_byrun", 
                             output_mscore_levels = c(0.01, 0.001), 
                             score_col = "m_score") {
    score_col <- JPP_update(data, score_col)

    # create m_score intervals to be tested
    test_levels <- 10^-seq_len(n_range)

    ## Identify the minimal m-score cutoff at which all runs still contain decoys
    decoy_count_lengths <- NULL
    for (i in seq_len(n_range)) {
    decoy_count_lengths[i] <- length(by(data[data$decoy == TRUE & data[,score_col] <= test_levels[i],
        c("transition_group_id")], data[data$decoy == TRUE & data[,score_col] <= test_levels[i], c("run_id")], length))
    }

    mscore_limit <- length(decoy_count_lengths[decoy_count_lengths == length(unique(data$run_id))])
    if (mscore_limit < 2) {
        mscore_limit <- 2
    }

    mscore_levels <- 10^-c(seq(2, mscore_limit))
    output_mscore_levels <- output_mscore_levels[output_mscore_levels %in% mscore_levels]

    fdr_cube <- array(NA, dim = c(12, length(unique(data$run_id)), length(mscore_levels)))

    dimnames(fdr_cube) <- list(c("target_assays", "decoy_assays", "false_target_assays",
        "assay_fdr", "target_peptides", "decoy_peptides", "false_target_peptides",
        "peptide_fdr", "target_proteins", "decoy_proteins", "false_target_proteins",
        "protein_fdr"), sort(unique(data$run_id)), mscore_levels)

    length_unique <- function(X) {
        length(unique(X))
    }

    data.t <- data[data$decoy == 0, ]
    data.d <- data[data$decoy == 1, ]

    # for each m_score cutoff in mscore_levels, count targets and decoys and
    # calculate false targets and fdr
    for (i in seq_len(length(mscore_levels))) {
        # for each run_id, count target (and decoy) assays identified ('id' column
        # entries) and store in pane i /row 1 (targets) /row 2 (decoys) & calculate false
        # targets & fdr based on FFT
        fdr_cube[1, , i] <- by(data.t[data.t[, score_col] <= mscore_levels[i], c("transition_group_id")],
            data.t[data.t[, score_col] <= mscore_levels[i], c("run_id")], length)
        fdr_cube[2, , i] <- by(data.d[data.d[, score_col] <= mscore_levels[i], c("transition_group_id")],
            data.d[data.d[, score_col] <= mscore_levels[i], c("run_id")], length)
        fdr_cube[3, , i] <- fdr_cube[2, , i] * FFT
        fdr_cube[4, , i] <- fdr_cube[3, , i]/fdr_cube[1, , i]

        # for each run_id, count target (and decoy) peptides identified (unique
        # 'FullPeptideName' column entries) and store in pane i /row 1 (targets) /row 2
        # (decoys) & calculate false targets & fdr based on FFT
        fdr_cube[5, , i] <- by(data.t[data.t[, score_col] <= mscore_levels[i], c("FullPeptideName")],
            data.t[data.t[, score_col] <= mscore_levels[i], c("run_id")], length_unique)
        fdr_cube[6, , i] <- by(data.d[data.d[, score_col] <= mscore_levels[i], c("FullPeptideName")],
            data.d[data.d[, score_col] <= mscore_levels[i], c("run_id")], length_unique)
        fdr_cube[7, , i] <- fdr_cube[6, , i] * FFT
        fdr_cube[8, , i] <- fdr_cube[7, , i]/fdr_cube[5, , i]

        # for each run_id, count target (and decoy) proteins identified (unique
        # 'ProteinName' column entries) and store in pane i /row 1 (targets) /row 2
        # (decoys) & calculate false targets & fdr based on FFT
        fdr_cube[9, , i] <- by(data.t[data.t[, score_col] <= mscore_levels[i], c("ProteinName")],
            data.t[data.t[, score_col] <= mscore_levels[i], c("run_id")], length_unique)
    fdr_cube[10, , i] <- by(data.d[data.d[,score_col] <= mscore_levels[i], c("ProteinName")],
                            data.d[data.d[,score_col] <= mscore_levels[i], c("run_id")], length_unique)
        fdr_cube[11, , i] <- fdr_cube[10, , i] * FFT
        fdr_cube[12, , i] <- fdr_cube[11, , i]/fdr_cube[9, , i]
    }

    # print fdr values to console (m_score cutoff 1e-2)
  message("The average FDR by run on assay level is ", 
          round(mean(fdr_cube[4, , 1], na.rm=TRUE), digits=3), "\n")
  message("The average FDR by run on peptide level is ", 
          round(mean(fdr_cube[8, , 1], na.rm=TRUE), digits=3), "\n")
  message("The average FDR by run on protein level is ", 
          round(mean(fdr_cube[12, , 1], na.rm=TRUE), digits=3), "\n")

    if (output == "pdf_csv") {
    message("Individual run FDR qualities can be retrieved from ", 
            paste(filename, ".csv"), "\n", sep="")
        # Write csv reports for mscore 1e-2 and 1e-3

        for (i in output_mscore_levels) {
            k.mscore <- which(dimnames(fdr_cube)[[3]] == i)
            k.mscore.label <- as.numeric(dimnames(fdr_cube)[[3]][k.mscore])
            write.csv(fdr_cube[, , k.mscore], file = paste(filename, "table_mscore_",
                format(k.mscore.label, scientific = TRUE), ".csv", sep = ""))
        }
        message(filename, ".csv reports written to working folder", "\n")
    }

    fdr_cube2 <- fdr_cube
    class(fdr_cube2) <- "fdr_cube"

    if (isTRUE(plot)) {
    plot.fdr_cube(fdr_cube2, output = output, filename = filename, 
                  plot_mscore_levels = output_mscore_levels)
    }

    if (output == "Rconsole") {
        return(fdr_cube2)
    }

}

#' Filter OpenSWATH output table based on mscore.
#'
#' This function filters the SWATH/DIA data according to a m_score value, as well
#' as to the number of occurence in the data (requant) and within a condition
#' (condition).
#'
#' @param data A data frame containing SWATH data.
#' @param mscore Value that defines the mscore threshold according to which the
#'   data will be filtered.
#' @param rm.decoy  Option to drop decoys from the data
#' @param mscore.col Defines the column from which to retrieve the m_score. 
#'    If you use JPP (Rosenberger, Bludau et al. 2017) this can be used to 
#'    select between Protein and transition_group m_score.
#' @return Returns a data frame with the filtered data
#' @author Peter Blattmann
#' @examples
#'  data("OpenSWATH_data", package="SWATH2stats")
#'  data("Study_design", package="SWATH2stats")
#'  data <- sample_annotation(OpenSWATH_data, Study_design)
#'  data.filtered <- filter_mscore(data, 0.01)
#'  data.filtered <- filter_mscore_freqobs(data, 0.01, 0.8)
#'  data.filtered <- filter_mscore_condition(data, 0.01, 3)
#' @export
filter_mscore <- function(data, 
                          mscore, 
                          rm.decoy = TRUE, 
                          mscore.col = "m_score") {
    if (sum(colnames(data) == "decoy") == 1 & rm.decoy == TRUE) {
        data <- data[data$decoy == 0, ]
        # subset(data, decoy == 0)
    }
    
    mscore.col <- JPP_update(data, mscore.col)
    data.filtered <- data[data[, mscore.col] <= mscore, ]
    message("Dimension difference: ", paste(dim(data) - dim(data.filtered), collapse = ", "))
    return(data.filtered)
}

#' Plots the coefficient of variation for different replicates.
#'
#' This function plots the coefficient of variation within replicates for a
#' given value. If decoys are present these are removed before plotting.
#'
#' @param data Data frame that is produced by the OpenSWATH/pyProphet workflow.
#' @param column.values  Indicates the columns for which the variation is
#'   assessed. This can be the Intensity or Signal, but also the retention
#'   time.
#' @param comparison The comparison for assessing the variability. Default is to
#'   assess the variability per transition_group_id and Condition over the
#'   different Replicates. Comparison is performed using the dcast() function of
#'   the reshape2 package.
#' @param fun_aggregate If for the comparison values have to be aggregated one
#'   needs to provide the function here.
#' @param label  Option to print value of median cv.
#' @param title Title of plot. Default: "cv across conditions"
#' @param boxplot   Logical. If boxplot should be plotted. Default: TRUE
#' @param ... further arguments passed to method.
#' @return Returns a list with the data and calculated cv and a table that
#'   summarizes the mean, median and mode cv per Condition (if Condition is
#'   contained in the comparison). In addition it plots in Rconsole a violin
#'   plot with the observed coefficient of variations.
#' @author Peter Blattmann
#' @examples{
#'  data("OpenSWATH_data", package="SWATH2stats")
#'  data("Study_design", package="SWATH2stats")
#'  data <- sample_annotation(OpenSWATH_data, Study_design)
#'  var_summary <- plot_variation(data)
#'  }
#' @importFrom reshape2 dcast
#' @importFrom stats sd na.omit aggregate density
#' @importFrom ggplot2 stat_summary aes_string geom_violin theme labs
#' @export
plot_variation <- function(data, 
                           column.values = "Intensity", 
                           comparison = transition_group_id + Condition ~ BioReplicate, 
                           fun_aggregate = NULL, 
                           label = FALSE, 
                           title = "cv across conditions",
                           boxplot = TRUE, ...) {
    if (sum(colnames(data) == "decoy") == 1) {
        data <- data[data$decoy == 0, ]
    }
    data.c <- dcast(data, comparison, value.var = column.values, 
                    fun.aggregate = fun_aggregate)
    data.c[data.c == 0] <- NA

    n_vars <- length(all.vars(comparison))

    data.sd <- apply(data.c[, n_vars:dim(data.c)[2]], 1, 
                     function(x) sd(x, na.rm = TRUE))
    data.mean <- apply(data.c[, n_vars:dim(data.c)[2]], 1, 
                       function(x) mean(x, na.rm = TRUE))
    data.c$cv <- data.sd/data.mean
    mean.cv <- mean(data.c$cv, na.rm = TRUE)
    median.cv <- median(data.c$cv, na.rm = TRUE)

    data.cv <- data.c[, c(colnames(data.c)[2], "cv")]

    p <- ggplot(na.omit(data.cv), aes_string(x = colnames(data.cv)[1], y = "cv")) +
        geom_violin(scale = "area") + 
        theme(axis.text.x = element_text(size = 8, angle = 90, hjust = 1, vjust = 0.5)) + 
        labs(title = title) +
        theme_bw()
    
    if (isTRUE(label)) {
        p <- p + stat_summary(fun.data = function(x) data.frame(y = max(x) * 0.75,
            label = paste("median cv:\n", signif(median(x, na.rm = TRUE), digits = 2))),
            geom = "text")
    }

    if (isTRUE(boxplot)) {
        p <- p + geom_boxplot(width = 0.1, outlier.shape = NA)
    }

    print(p)
    if ("Condition" %in% colnames(data.c)) {
        median <- aggregate(data.c[, "cv"], by = list(data.c$Condition), FUN = function(x) median(x,
            na.rm = TRUE))
        colnames(median) <- c("Condition", "median_cv")
        mean <- aggregate(data.c[, "cv"], by = list(data.c$Condition), FUN = function(x) mean(x,
            na.rm = TRUE))
        colnames(mean) <- c("Condition", "mean_cv")
        mode <- aggregate(data.c[, "cv"], by = list(data.c$Condition), FUN = function(x) {
            d <- density(x, na.rm = TRUE)
            i <- which.max(d$y)
            return(d$x[i])
        })
        colnames(mode) <- c("Condition", "mode_cv")

        cv_table <- merge(mode, merge(mean, median, by = "Condition"), by = "Condition")

        return(list(data.c, cv_table))
    }
    return(data.c)
}

context("plotting")

test_that("correlation plot", {
  data(OpenSWATH_data, package="SWATH2stats")
  data(Study_design, package="SWATH2stats")
  data.FDR<-sample_annotation(OpenSWATH_data, Study_design)
  
  
  correlations <- plot_correlation_between_samples(data.FDR)
  expect_that(correlations[correlations$Var1 == "Hela_Control_1" & correlations$Var2 == "Hela_Control_1","value"], equals(1))
  expect_that(correlations[correlations$Var1 == "Hela_Treatment_1" & correlations$Var2 == "Hela_Treatment_1","value"], equals(1))
  
  correlations <- plot_correlation_between_samples(data.FDR, label = FALSE)
  expect_that(correlations[correlations$Var1 == "Hela_Control_1" & correlations$Var2 == "Hela_Control_1","value"], equals(1))
  
  
  data1 <- data.FDR[data.FDR$Condition == "Hela_Control" & data.FDR$BioReplicate == 1 & data.FDR$decoy == 0, c("transition_group_id", "Intensity")]
  data2 <- data.FDR[data.FDR$Condition == "Hela_Treatment" & data.FDR$BioReplicate == 1 & data.FDR$decoy == 0, c("transition_group_id", "Intensity")]
  data3 <- merge(data1, data2, by="transition_group_id")
  
  cor.p <- cor(data3$Intensity.x, data3$Intensity.y, method="pearson")
  cor.s <- cor(data3$Intensity.x, data3$Intensity.y, method="spearman")
  
  expect_that(correlations[correlations$Var1 == "Hela_Control_1" & correlations$Var2 == "Hela_Treatment_1" & correlations$method == "pearson","value"], equals(cor.p))
  expect_that(correlations[correlations$Var1 == "Hela_Treatment_1" & correlations$Var2 == "Hela_Control_1" & correlations$method == "spearman","value"], equals(cor.s))
  
 })

test_that("variation plot", {
  data(OpenSWATH_data, package="SWATH2stats")
  data(Study_design, package="SWATH2stats")
  data.FDR<-sample_annotation(OpenSWATH_data, Study_design)
  
  variation <- plot_variation(data.FDR, label=FALSE)
  variation <- plot_variation(data.FDR)
  variation.data <- variation[[1]]
  
  # test extraction
  test1 <- variation.data[variation.data$transition_group_id == "124947_SGWVKPIIIGVLR_2_run0" & variation.data$Condition == "Hela_Control",]
  test2 <- variation.data[variation.data$transition_group_id == "214321_GTLNLDSYR_2_run0" & variation.data$Condition == "Hela_Treatment",]
  
  val1 <- data.FDR[data.FDR$transition_group_id == "124947_SGWVKPIIIGVLR_2_run0" & data.FDR$Condition == "Hela_Control", "Intensity"]
  val2 <- data.FDR[data.FDR$transition_group_id == "214321_GTLNLDSYR_2_run0" & data.FDR$Condition == "Hela_Treatment", "Intensity"]
  
  expect_true(sum(test1[,c("1","2","3")] %in% val1) == 3)
  expect_true(sum(test2[,c("1","2","3")] %in% val2) == 3)
  
  # test cv calculation
  
  mean.test1 <- mean(as.numeric(test1[,c("1","2","3")]))
  mean.test2 <- mean(as.numeric(test2[,c("1","2","3")]))
  
  expect_true(all.equal(sd(as.numeric(test1[,c("1","2","3")]))/mean.test1, as.numeric(test1[,"cv"])))
  expect_true(all.equal(sd(as.numeric(test2[,c("1","2","3")]))/mean.test2, as.numeric(test2[,"cv"])))
})


test_that("variation plot vs total", {
  data(OpenSWATH_data, package="SWATH2stats")
  data(Study_design, package="SWATH2stats")
  data.FDR<-sample_annotation(OpenSWATH_data, Study_design)
  
  
  variation <- plot_variation_vs_total(data.FDR)
  variation.data <- variation[[1]]
  
  # test extraction
  test1.total <- variation.data[variation.data$rep == "124947_SGWVKPIIIGVLR_2_run0",]
  test1.rep <- variation.data[variation.data$rep == "124947_SGWVKPIIIGVLR_2_run0 Hela_Control",]
  
  test2.total <- variation.data[variation.data$rep == "214321_GTLNLDSYR_2_run0",]
  test2.rep <- variation.data[variation.data$rep == "214321_GTLNLDSYR_2_run0 Hela_Treatment",]
  
  val1.total <- data.FDR[data.FDR$transition_group_id == "124947_SGWVKPIIIGVLR_2_run0", "Intensity"]
  val2.total <- data.FDR[data.FDR$transition_group_id == "214321_GTLNLDSYR_2_run0", "Intensity"]
  
  val1.rep <- data.FDR[data.FDR$transition_group_id == "124947_SGWVKPIIIGVLR_2_run0" & data.FDR$Condition == "Hela_Control", "Intensity"]
  val2.rep <- data.FDR[data.FDR$transition_group_id == "214321_GTLNLDSYR_2_run0" & data.FDR$Condition == "Hela_Treatment", "Intensity"]

  cv.val1.total <- sd(val1.total)/mean(val1.total)
  cv.val2.total <- sd(val2.total)/mean(val2.total)
  cv.val1.rep <- sd(val1.rep)/mean(val1.rep)
  cv.val2.rep <- sd(val2.rep)/mean(val2.rep)
  
  expect_true(all.equal(cv.val1.total, as.numeric(test1.total[,"cv"])))
  expect_true(all.equal(cv.val2.total, as.numeric(test2.total[,"cv"])))
  expect_true(all.equal(cv.val1.rep, as.numeric(test1.rep[,"cv"])))
  expect_true(all.equal(cv.val2.rep, as.numeric(test2.rep[,"cv"])))
})

#' Unify the protein group labels.
#'
#' Unify the protein group labels (2/ProteinA/ProteinB and 2/ProteinB/ProteinA)
#' to one common label (e.g. 2/ProteinA/ProteinB)
#'
#' @param data  A data frame containing SWATH data.
#' @param column Which column to use for unifying the protein group identifiers.
#' @return Returns a data frame with the unififed protein labels.
#' @author Moritz Heusel
#' @examples
#'  data("OpenSWATH_data", package="SWATH2stats")
#'  data("Study_design", package="SWATH2stats")
#'  data <- sample_annotation(OpenSWATH_data, Study_design)
#'  data.filtered.decoy <- filter_mscore(data, 0.01)
#'  data.unified <- unifyProteinGroupLabels(data.filtered.decoy)
#' @export
unifyProteinGroupLabels <- function(data,
                                    column = "ProteinName"){
    data[,column] <- as.character(data[,column])
    ids <- grep("^([2-9])|([1-9][0-9][0-9]*)/", data[,column])
    identifiers <- data[ids, column]
    identifiers_split <- strsplit(as.character(identifiers), "/")
    identifiers_split_sorted <- lapply(identifiers_split, function(x) {
        sort(x)
    })
    identifiers_sorted <- vapply(identifiers_split_sorted, function(x) {
        paste(x, collapse = "/")
    }, "a")
    data[ids, column] <- identifiers_sorted
    return(data)
}

#' Filter for proteins that are supported by proteotypic peptides.
#'
#' Peptides can match to several proteins. With this function proteotypic
#' peptides, peptides that are only contained in one protein are
#' selected. Additionally the number of proteins are counted and printed.
#'
#' @param data  A data frame containing SWATH data.
#' @param rm.decoy  Option to remove the decoys during filtering.
#' @return  Returns a data frame with only the data supported by proteotypic
#'   peptides.
#' @author  Peter Blattmann
#' @examples
#'  data("OpenSWATH_data", package="SWATH2stats")
#'  data("Study_design", package="SWATH2stats")
#'  data <- sample_annotation(OpenSWATH_data, Study_design)
#'  data.filtered.decoy <- filter_mscore(data, 0.01)
#'  data.all <- filter_proteotypic_peptides(data.filtered.decoy)
#' @export
filter_proteotypic_peptides <- function(data, 
                                        rm.decoy = TRUE) {
    data <- unifyProteinGroupLabels(data)
    if (isTRUE(rm.decoy)) {
        data <- removeDecoyProteins(data)
    }

    data.proteins <- gsub("^[[:digit:]]\\/", "", data$ProteinName)
    data.proteins <- unlist(strsplit(as.character(data.proteins), "\\/"))
    data.proteins <- unique(data.proteins)

    message("Number of proteins detected: ", length(data.proteins), "\n",
            "Protein identifiers: ", paste(head(data.proteins), collapse = ", "))
    
    ## Delete non-unique features (unique features have '1/' in front of protein name)
    data.unique <- data[grep("^1/", data$ProteinName), ]
    data.unique$ProteinName <- gsub("^1\\/", "", data.unique$ProteinName)
    
    message("Number of proteins detected that are supported by a proteotypic peptide: ",
            length(unique(data.unique$ProteinName)),
            "\n", "Number of proteotypic peptides detected: ",
            length(unique(data.unique$FullPeptideName)))
    
    return(data.unique)
}

#' Transforms the column names from a data frame to the required format.
#'
#' This functions transforms the column names from a data frame from another
#' format to a data frame with column names used by the OpenSWATH output and
#' required for these functions. During executing of the function the
#' corresponding columns for each column in the data need to be selected. For
#' columns that do not corresond to a certain column 'not applicable' needs to
#' be selected and the column names are not changed.
#'
#' @note List of column names of the OpenSWATH data:
#' ProteinName: Unique identifier for protein or proteingroup that the peptide
#'   maps to. Proteotypic peptides should be indicated by 1/ in order to be
#'   recognized as such by the function filter_proteotypic_peptides.
#' FullPeptideName: Unique identifier for the peptide.
#' Charge: Charge of the peptide precursor ion quantified.
#' Sequence: Naked peptide sequence without modifications.
#' aggr_Fragment_Annotation: aggregated annotation for the different Fragments
#'   quantified for this peptide. In the OpenSWATH results the different
#'   annotation in OpenSWATH are concatenated by a semicolon.
#' aggr_Peak_Area: aggregated Intensity values for the different Fragments
#'   quantified for this peptide. In the OpenSWATH results the aggregated Peak
#'   Area intensities are concatenated by a semicolon.
#' transition_group_id: A unique identifier for each transition group used.
#' decoy: Indicating with 1 or 0 if this transition group is a decoy.
#' m_score: Column containing the score that is used to estimate FDR or
#'   filter. M-score values of identified peak groups are equivalent to a
#'   q-value and thus typically are smaller than 0.01, depending on the
#'   confidence of identification (the lower the m-score, the higher the
#'   confidence).
#' Column containing the score that is used to estimate FDR or filter.
#' RT: Column containing the retention time of the quantified peak.
#' filename: Column containing the filename or a unique identifier for each
#'   injection.
#' Intensity: column containing the intensity value for each quantified peptide.
#' Columns needed for FDR estimation and filtering functions: ProteinName,
#'   FullPeptideName, transition_group_id, decoy, m_score
#' Columns needed for conversion to transition-level format (needed for MSStats
#'   and mapDIA input): aggr_Fragment_Annotation, aggr_Peak_Are
#'
#' @param data  A data frame containing the SWATH-MS data (one line per peptide
#'   precursor quantified) but with different column names.
#' @return Returns the data frame in the appropriate format.
#' @author Peter Blattmann
#' @examples
#'  data('Spyogenes', package = 'SWATH2stats')
#'  head(data)
#'  str(data)
#' @export
import_data <- function(data) {
  colnames_OpenSWATH <- c("ProteinName", "FullPeptideName", "Charge","Sequence",
                          "aggr_Fragment_Annotation", "aggr_Peak_Area", 
                          "transition_group_id", "decoy", "m_score", "RT", 
                          "filename", "Intensity", "not applicable")

  message("When reading in data, please specify to which column in the 
  OpenSWATH data they correspond. For columns that do not correspond to any 
  OpenSWATH column, choose \"not applicable\". Explanation of the OpenSWATH 
          columns can be found in the manual page.")
  
  # Dialogue to map columns
  for (i in colnames(data)) {
    value <- utils::select.list(
      colnames_OpenSWATH,
      title = paste0("Select a column from the OpenSWATH output that the column ",
                     i, "corresponds to."))
    if (value != "not applicable") {
      colnames(data) <- gsub(i, value, colnames(data))
      message("Column name ", i, " was substituted by ", value, "\n")
    }
  }

  ## add NA to colnames that were not mapped
  add.colnames <- colnames(data)[!(colnames(data) %in% colnames_OpenSWATH)]
  if (length(add.colnames > 0)) {
    for (i in add.colnames) data[, i] <- NA
    warning("Not all columns required within SWATH2stats were mapped: The columns ",
            paste(add.colnames, collapse = ", "), " were added with NA as value.")
  }

  return(data)
}

utils::globalVariables(c("PROTEIN", "PEPTIDE"))

#' Filter only for the highest intense peptides
#'
#' In order to reduce the data, the data is filtered only for the proteins with
#' the highest intensity peptides.
#'
#' @param data A data frame containing SWATH data with the column names:
#'   ProteinNames, PeptideSequence, PrecursorCharge, Intensity.
#' @param n_peptides Maximum number of highest intense peptides to filter the
#'   data on.
#' @param protein_col Column with protein identifiers. Default: ProteinName
#' @param peptide_col Column with peptide identifiers. Default: Peptide.Sequence or FullPeptideName
#' @param rm.decoy  Option to remove the decoys during filtering.
#' @return  Returns a data frame of the filtered data.
#' @author Peter Blattmann
#' @examples{
#'  data("OpenSWATH_data", package="SWATH2stats")
#'  data("Study_design", package="SWATH2stats")
#'  data <- sample_annotation(OpenSWATH_data, Study_design)
#'  data.filtered <- filter_mscore_freqobs(data, 0.01,0.8)
#'  data.max <- filter_on_max_peptides(data.filtered, 5)
#'  }
#' @importFrom data.table setkey setkeyv setnames
#' @importFrom utils head
#' @export
filter_on_max_peptides <- function(data, 
                                   n_peptides, 
                                   protein_col = "ProteinName",
                                   peptide_col = c("Peptide.Sequence", "FullPeptideName"),
                                   rm.decoy = TRUE) {
    data <- unifyProteinGroupLabels(data)
    if (isTRUE(rm.decoy)) {
        data <- removeDecoyProteins(data)
    }
    
    # select valid columns
    columns <- validate_columns(data, list(Protein = protein_col,
                                        Peptide = peptide_col))

    data <- data.table::data.table(data)
    setnames(data, columns[["Protein"]], "PROTEIN")
    setnames(data, columns[["Peptide"]], "PEPTIDE")
  
    data.peptides <- data[, c("PROTEIN", "PEPTIDE", "Intensity"), with = FALSE]
    data.table::setkeyv(data, cols = c("PROTEIN", "PEPTIDE"))

    Intensity <- NULL
    data.peptides.int <- data.peptides[, sum(Intensity), by = "PROTEIN,PEPTIDE"]
    setnames(data.peptides.int, "V1", "SUM.INTENSITY")

    setkey(data.peptides.int, PROTEIN)
    data.peptides.int <- data.peptides.int[order(data.peptides.int$SUM.INTENSITY,
        decreasing = TRUE), ]

    .SD <- NULL
    peptides.sel <- unique(data.peptides.int[, head(.SD, n_peptides), by = PROTEIN])

    data.filtered <- data.frame(data[PEPTIDE %in% peptides.sel$PEPTIDE, ])


    message("Before filtering: ", "\n", 
            "  Number of proteins: ", length(unique(data$PROTEIN)), "\n", 
            "  Number of peptides: ", length(unique(data$PEPTIDE)), "\n\n", 
            "Percentage of peptides removed: ", 
            round((length(unique(data$PEPTIDE)) - length(unique(data.filtered$PEPTIDE)))/length(unique(data$PEPTIDE)) *
            100, digits = 2), "%", "\n\n", 
            "After filtering: ", "\n", "  Number of proteins: ", length(unique(data.filtered$PROTEIN)), "\n", 
            "  Number of peptides: ", length(unique(data.filtered$PEPTIDE)),"\n")

    colnames(data.filtered) <- gsub("PROTEIN", columns[["Protein"]], colnames(data.filtered))
    colnames(data.filtered) <- gsub("PEPTIDE", columns[["Peptide"]], colnames(data.filtered))

    return(data.filtered)
}

#' Filter OpenSWATH output table according to mscore.
#'
#' This function filters the SWATH data according to the m_score value, as well
#' as to the number of occurence in the data.
#'
#' @param data  A data frame containing SWATH data.
#' @param mscore  Value that defines the mscore threshold according to which the
#'   data will be filtered.
#' @param percentage  Percentage in which replicas the transition has to reach
#'   the mscore threshold.
#' @param rm.decoy  Option to remove the decoys during filtering.
#' @param mscore.col Defines the column from which to retrieve the m_score. 
#'    If you use JPP (Rosenberger, Bludau et al. 2017) this can be used to 
#'    select between Protein and transition_group m_score.
#' @return Returns a data frame with the filtered data.
#' @author Peter Blattmann
#' @examples
#'  data("OpenSWATH_data", package="SWATH2stats")
#'  data("Study_design", package="SWATH2stats")
#'  data <- sample_annotation(OpenSWATH_data, Study_design)
#'  data.filtered <- filter_mscore(data, 0.01)
#'  data.filtered <- filter_mscore_freqobs(data, 0.01, 0.8)
#'  data.filtered <- filter_mscore_condition(data, 0.01, 3)
#' @export

filter_mscore_freqobs <- function(data, 
                                  mscore, 
                                  percentage = NULL, 
                                  rm.decoy = TRUE,
                                  mscore.col = "m_score") {
    
    mscore.col <- JPP_update(data, mscore.col)

    data$Peptide_Charge <- paste(data$FullPeptideName, data$Charge)
    if (sum(colnames(data) == "decoy") == 1 & isTRUE(rm.decoy)) {
        # data <- subset(data, decoy == 0)
        data <- data[data$decoy == 0, ]
    }

    # data.filtered <- subset(data, m_score <= mscore)
    data.filtered <- data[data[, mscore.col] <= mscore, ]
    data.filtered <- data.table::data.table(data.filtered)

    data.filtered <- data.filtered[, c("Peptide_Charge", "aggr_Peak_Area"), with = FALSE]
    data.table::setkeyv(data.filtered, cols = "Peptide_Charge")
    .N <- NULL
    data.n <- data.filtered[, .N, by = "Peptide_Charge"]

    if (is.null(percentage)) {
        percentage <- 0
    }

    threshold <- nlevels(factor(data$filename)) * percentage
    message("Treshold, peptides need to have been quantified in more conditions than: ",
        threshold)

    peptides.filtered <- data.n[data.n$N >= threshold]
    peptides.filtered <- data.frame(Peptides_Charge = peptides.filtered$Peptide_Charge)

    message("Fraction of peptides selected: ",
            signif(length(unique(peptides.filtered$Peptides_Charge)) /
                   length(unique(data$Peptide_Charge)),
                   digits = 2))

    data.filtered <- merge(data, peptides.filtered,
                           by.x = "Peptide_Charge", by.y = "Peptides_Charge")
    message("Dimension difference: ", paste(dim(data) - dim(data.filtered), collapse = ", "))
    return(data.filtered)
}

#' Writes out an overview matrix of peptides mapping to a FDR quality controlled
#' protein master list at controlled global peptide FDR quality.
#'
#' Writes out an overview matrix on peptide level of a supplied (unfiltered or
#' prefiltered) OpenSWATH results data frame.
#' The peptide quantification is achieved by summing the areas under all 6
#' transitions per precursor and summing all precursors per FullPeptideName.
#' In order to keep the peptide-to-protein association, the FullPeptideName is
#' joined with the ProteinName.
#'
#' @param data A data frame containing annotated OpenSWATH/pyProphet data.
#' @param write_csv Option to determine if table should be written automatically
#'   into csv file.
#' @param fun_aggregate  What function to use when aggregating the set of 
#'   intensities (sum or mean)?. Default: sum.
#' @param filename File base name of the .csv matrix written out to the working
#'   folder.
#' @param rm_decoy Logical whether decoys will be removed from the data
#'   matrix. Defaults to FALSE. It's sometimes useful to know how decoys behave
#'   across a dataset and how many you allow into your final table with the
#'   current filtering strategy.
#' @return the peptides as a matrix! also output .csv matrix is written to the
#'   working folder.
#' @author Moritz Heusel
#' @examples{
#'  data("OpenSWATH_data", package="SWATH2stats")
#'  data("Study_design", package="SWATH2stats")
#'  data <- sample_annotation(OpenSWATH_data, Study_design)
#'  written <- write_matrix_peptides(data)
#'  }
#' @importFrom reshape2 dcast
#' @export
write_matrix_peptides <- function(data, 
                                  write_csv = FALSE, 
                                  fun_aggregate = "sum",
                                  filename = "SWATH2stats_overview_matrix_peptidelevel.csv",
                                  rm_decoy = FALSE) {
    if (rm_decoy == TRUE) {
        data <- subset(data, data$decoy == 0)
    }
    data.peptide <- data[, c("ProteinName", "run_id", "FullPeptideName", "Intensity")]
    ProteinName_FullPeptideName <- paste(data.peptide$ProteinName, 
                                         data.peptide$FullPeptideName,
                                         sep = "_")
    data.peptide <- cbind(ProteinName_FullPeptideName, data.peptide)
    if(fun_aggregate == "sum"){
        data.peptide.table <- dcast(data.peptide, 
                                ProteinName_FullPeptideName ~ run_id,
                                value.var = "Intensity", fun.aggregate = sum)
    }
    if(fun_aggregate == "mean"){
        data.peptide.table <- dcast(data.peptide, 
                                    ProteinName_FullPeptideName ~ run_id,
                                    value.var = "Intensity", fun.aggregate = mean)
    }
    if(write_csv) {
        write.csv(data.peptide.table, file = filename, row.names = FALSE, quote = FALSE)
        message("Peptide overview matrix ", filename, " written to working folder.",
            "\n")
    }
    return(data.peptide.table)
}

context("Overview matrix writing")

test_that("Overview matrix writing", {
  data(OpenSWATH_data, package="SWATH2stats")
  data(Study_design, package="SWATH2stats")
  data<-sample_annotation(OpenSWATH_data, Study_design)
  
  # peptide_matrix_test
  peptide_matrix <- write_matrix_peptides(data)
  
  expect_that(peptide_matrix[which(peptide_matrix$ProteinName_FullPeptideName == "1/iRT_protein_ATFGVDESNAEVK"),
                             "Hela_Control_1_1"],
              equals(86041))

  # test rm.decoy option
  peptide_matrix2 <- write_matrix_peptides(data, rm_decoy=TRUE)
  expect_true(length(grep("DECOY", peptide_matrix$ProteinName_FullPeptideName)) > 0)
  expect_false(length(grep("DECOY", peptide_matrix2$ProteinName_FullPeptideName)) > 0)
  
  # test write.csv option
  expect_message(write_matrix_peptides(data, write_csv = TRUE),
                 "Peptide overview matrix SWATH2stats_overview_matrix_peptidelevel.csv written to working folder.")
  
  
  # protein_matrix_test
  protein_matrix <- write_matrix_proteins(data)

  expect_that(protein_matrix[which(protein_matrix$ProteinName == "1/iRT_protein"),
                                "Hela_Control_1_1"], equals(1538879))

  # test rm.decoy option
  protein_matrix2 <- write_matrix_proteins(data, rm_decoy=TRUE)
  expect_true(length(grep("DECOY", protein_matrix$ProteinName)) > 0)
  expect_false(length(grep("DECOY", protein_matrix2$ProteinName)) > 0)
  
  # test write.csv option
  expect_message(write_matrix_proteins(data, write_csv = TRUE),
                 "Protein overview matrix SWATH2stats_overview_matrix_proteinlevel.csv written to working folder")
})

#' Assess decoy rate in data
#'
#' This function assesses the number of quantifications (typically peptides)
#' that are decoys (false-positive) versus true identifications. 
#'
#' A printout is generated to indicate the number of non-decoy, decoy peptides
#' and the rate of decoy vs non-decoy peptides. Unique peptides are counted, so
#' a precursor with different charge states is counted as one peptide. In the
#' column "decoy" the values need to be 1,0 or TRUE and FALSE.
#'
#' @param data A data frame that contains at least a column named 
#'     "FullPeptideName" and "decoy".
#' @param column The column name of the Peptide identifier. 
#'     Default: FullPeptideName.
#' @param column_decoy The column name of the decoy column. Default: decoy.
#' @return Message detailing the number of decoys, non-decoys, and the ratio.
#' @author Peter Blattmann
#' @examples
#'  data("OpenSWATH_data", package="SWATH2stats")
#'  data <- OpenSWATH_data
#'  assess_decoy_rate(data)
#' @export

assess_decoy_rate <- function(data, 
                              column = "FullPeptideName",
                              column_decoy = "decoy") {
    if (sum(colnames(data) == column_decoy) < 1) {
        stop("There is no decoy column in the table")
    }
    if (sum(colnames(data) == column) < 1) {
        stop("There is no column in the table for identifiers (e.g. peptides).")
    }

    .non_decoy_peptides <- unique(data[data$decoy == FALSE, c(column)])
    .decoy_peptides <- unique(data[data$decoy == TRUE, c(column)])

    message("Number of non-decoy identifiers: ",length(.non_decoy_peptides),"\n",
        "Number of decoy identifiers: ", length(.decoy_peptides), "\n", 
        "Decoy rate: ", 
        sprintf("%.4f", (length(.decoy_peptides)/length(.non_decoy_peptides))))
}

#' Convert data into the format for running a python script.
#'
#' This functions selects the columns suggested to run a python script to change
#' the data from peptide-level to transition-level.
#'
#' The necessary columns are selected and the run column is renamed to filename
#' for the script. The intensities are taken from the column aggr_Peak_Area and
#' therefore the Intensity column is not exported.
#'
#' @param data A data frame containing SWATH data.
#' @param replace.Unimod Option to indicate if Unimod Identifier should be
#'   replaced form ":"" to "_".
#' @return Returns a data frame in the appropriate format to be used by a custom
#'   python script stored in the scripts folder.
#' @author Peter Blattmann
#' @examples
#'  data("OpenSWATH_data", package="SWATH2stats")
#'  data("Study_design", package="SWATH2stats")
#'  data <- sample_annotation(OpenSWATH_data, Study_design)
#'  data.filtered.decoy <- filter_mscore(data,0.01)
#'  data.pythonscript <- convert4pythonscript(data.filtered.decoy)
#' @export

convert4pythonscript <- function(data, replace.Unimod = TRUE) {

    data <- data[, c("ProteinName", "FullPeptideName", "Charge", 
                     "aggr_Fragment_Annotation","aggr_Peak_Area", "RT", 
                     "BioReplicate", "Condition", "Run")]

    colnames(data) <- gsub("Run", "filename", colnames(data))

    if (isTRUE(replace.Unimod)) {
        # replace UniMod: to UniMod_
        data$FullPeptideName <- gsub("UniMod:", "UniMod_", data$FullPeptideName)
        data$aggr_Fragment_Annotation <- gsub("UniMod:", "UniMod_", data$aggr_Fragment_Annotation)
    }

    return(data)
}

#' Transforms the SWATH data from a peptide- to a transition-level table.
#'
#' If the SWATH data should be analyzed on transition-level the data needs to be
#' tranformed from peptide-level table to a transition-level table (one row per
#' transition instead of one row per peptide). The columns
#' "aggr_Fragment_Annotation" and "aggr_Peak_Area" are disaggregated into the
#' new columns "Fragmentation" and "Intensity".
#' The following columns are renamed if they exist: FullPeptideName ->
#' PeptideSequence, Charge -> PrecursorCharge, Area -> Intensity, Fragment ->
#' Fragmentation, Sequence -> NakedSequence.
#'
#' @param data A data frame containing SWATH data.
#' @param all.columns Option that all columns are processed. Otherwise only the
#'   columns typically needed for downstream analysis are processed.
#' @return Returns a data frame containing the SWATH data in a transition-level
#'   table.
#' @author Peter Blattmann
#' @examples{
#'  data("OpenSWATH_data", package="SWATH2stats")
#'  data("Study_design", package="SWATH2stats")
#'  data <- sample_annotation(OpenSWATH_data, Study_design)
#'  data.filtered.decoy <- filter_mscore(data, 0.01)
#'  raw <- disaggregate(data.filtered.decoy)
#'  }
#' @importFrom reshape2 colsplit
#' @export
disaggregate <- function(data, all.columns = FALSE) {

    # sanity test on the number of transitions per precursor
    n.transitions <- lapply(as.character(data$aggr_Fragment_Annotation), 
                            function(x) strsplit(x, ";"))
    n.transitions2 <- unlist(lapply(n.transitions, 
                                    function(x) length(unlist(x))))

    n.transitions3 <- lapply(as.character(data$aggr_Peak_Area), 
                             function(x) strsplit(x, ";"))
    n.transitions4 <- unlist(lapply(n.transitions3, 
                                    function(x) length(unlist(x))))

    if (sum(n.transitions2 != n.transitions4) > 0) {
        stop(paste("The number of transitions annotated and measured do not match in the following transitions:\n",
            paste(unlist(n.transitions[n.transitions2 != n.transitions4]), collapse = ", ")))
    }

    # test if always the same number of transitions per precursor were used
    if (min(n.transitions2) == max(n.transitions2)) {
        message(paste("The library contains", max(n.transitions2), "transitions per precursor.
                  \nThe data table was transformed into a table containing one row per transition."))
    }
    if (min(n.transitions2) != max(n.transitions2)) {
        message(paste("The library contains between", min(n.transitions2), "and",
            max(n.transitions2), "transitions per precursor.\nThe data table was transformed into a table containing one row per transition."))
    }

    data.new <- cbind(data, 
                      colsplit(data$aggr_Fragment_Annotation, ";", paste("Split_FragAnnot_", seq_len(max(n.transitions2)), sep = "")), 
                      colsplit(data$aggr_Peak_Area, ";", paste("Split_PeakArea_", seq_len(max(n.transitions2)), sep = "")), stringsAsFactors = FALSE)

    data.new.m <- reshape2::melt(data.new, 
                                 id.vars = grep("Split_FragAnnot", colnames(data.new),invert = TRUE), 
                                 measure.vars = grep("Split_FragAnnot", colnames(data.new)),
                                 variable.name = "FragAnnot_N", value.name = "Fragment")
    
    data.new.m2 <- reshape2::melt(data.new.m, 
                                  id.vars = grep("Split_PeakArea", colnames(data.new.m), invert = TRUE),
                                  measure.vars = grep("Split_PeakArea", colnames(data.new.m)),
                                  variable.name = "Area_N", value.name = "Area")

    # added because it didn't name the variables in a later trial
    if (sum(colnames(data.new.m2) %in% c("FragAnnot_N")) == 0) {
        l <- length(colnames(data.new.m2))
        colnames(data.new.m2)[l - 3] <- "FragAnnot_N"
        colnames(data.new.m2)[l - 2] <- "Fragment"
        colnames(data.new.m2)[l - 1] <- "Area_N"
        colnames(data.new.m2)[l] <- "Area"
    }


    data.new.m3 <- data.new.m2[gsub("Split_FragAnnot_", "", 
                                    data.new.m2[, "FragAnnot_N"]) == gsub("Split_PeakArea_", "", data.new.m2[, "Area_N"]), ]

    if (!isTRUE(all.columns)) {
        cols <- colnames(data.new.m3)[colnames(data.new.m3) %in% c("ProteinName",
            "FullPeptideName", "PeptideSequence", "Sequence", "Charge", 
            "PrecursorCharge", "Fragment", "FragmentIon", "Area", "Condition", 
            "BioReplicate", "Run", "RT")]
        data.new.merged <- data.new.m3[, cols]
    }
    if (isTRUE(all.columns)) {
        data.new.merged <- data.new.m3
    }

    colnames(data.new.merged) <- gsub("FullPeptideName", "PeptideSequence", 
                                      colnames(data.new.merged))
    colnames(data.new.merged) <- gsub("^Charge$", "PrecursorCharge", 
                                      colnames(data.new.merged))

    colnames(data.new.merged) <- gsub("Area", "Intensity", 
                                      colnames(data.new.merged))
    colnames(data.new.merged) <- gsub("Fragment", "FragmentIon", 
                                      colnames(data.new.merged))

    if ("Sequence" %in% cols) {
        colnames(data.new.merged) <- gsub("^Sequence$", "NakedSequence", 
                                          colnames(data.new.merged))
    }
    if (sum(is.na(data.new.merged$Intensity)) > 0) {
        .ids <- !is.na(data.new.merged$Intensity)
        message(paste((length(data.new.merged$Intensity) - sum(.ids)), "row(s) was/were removed because they did not contain data due to different number of transitions per precursor"))
        data.new.merged <- data.new.merged[.ids, ]
    }

    return(data.new.merged)
}

#' Writes out an overview matrix of summed signals per protein identifier
#' (lines) over run_id(columns).
#'
#' Writes out an overview matrix on protein level of a supplied (unfiltered or
#' filtered) OpenSWATH results data frame. The protein quantification is achieved
#' by summing the areas under all 6 transitions per precursor, summing all
#' precursors per FullPeptideName and all FullPeptideName signals per
#' ProteinName entry.
#' This function does not select consistently quantified or top peptides but
#' sums all signals availabe that may or may not originate from the same set of
#' peptides across different runs. A more detailed overview can be generated
#' using the function write_matrix_peptides().
#' Peptide selection can be achieved upstream using e.g. the functions
#' filter_mscore_requant(), filter_on_max_peptides() and
#' filter_on_min_peptides().
#'
#' @param data A data frame containing annotated OpenSWATH/pyProphet data.
#' @param write_csv  Option to determine if table should be written 
#'   automatically into csv file.
#' @param fun_aggregate  What function to use when aggregating the set of 
#'   intensities (sum or mean)?. Default: sum.
#' @param filename  File base name of the .csv matrix written out to the working
#'   folder
#' @param rm_decoy  Logical whether decoys will be removed from the data
#'   matrix. Defaults to FALSE. It's sometimes useful to know how decoys behave
#'   across a dataset and how many you allow into your final table with the
#'   current filtering strategy.
#' @return the peptides as a matrix, also output .csv matrix is written to the
#'   working folder
#' @author Moritz Heusel
#' @examples{
#'  data("OpenSWATH_data", package="SWATH2stats")
#'  data("Study_design", package="SWATH2stats")
#'  data <- sample_annotation(OpenSWATH_data, Study_design)
#'  written <- write_matrix_proteins(data)
#'  }
#' @importFrom reshape2 dcast
#' @importFrom stats aggregate
#' @export
write_matrix_proteins <- function(data, 
                                  write_csv = FALSE, 
                                  fun_aggregate = "sum",
                                  filename = "SWATH2stats_overview_matrix_proteinlevel.csv",
                                  rm_decoy = FALSE) {
    if (rm_decoy == TRUE) {
        data <- subset(data, data$decoy == 0)
    }
    if(fun_aggregate == "sum"){
        data.protein.sum.all <- aggregate(data[, "Intensity"], 
                                         by = list(data$ProteinName, data$run_id), sum)
        colnames(data.protein.sum.all) <- c("ProteinName", "run_id", "Intensity.all.sum")
        data.protein.aggr.table <- dcast(data.protein.sum.all, ProteinName ~ run_id, 
                                        value.var = "Intensity.all.sum", 
                                        fun.aggregate = sum)
    }
    if(fun_aggregate == "mean"){
        data.protein.mean.all <- aggregate(data[, "Intensity"], 
                                         by = list(data$ProteinName, data$run_id), mean)
        colnames(data.protein.mean.all) <- c("ProteinName", "run_id", "Intensity.all.mean")
        data.protein.aggr.table <- dcast(data.protein.mean.all, ProteinName ~ run_id, 
                                        value.var = "Intensity.all.mean", 
                                        fun.aggregate = mean)
        
    }
      if (write_csv) {
        write.csv(data.protein.aggr.table, file = filename, 
                  row.names = FALSE, quote = FALSE)
        message("Protein overview matrix ", filename, " written to working folder.",
            "\n")
    }
    return(data.protein.aggr.table)
}

#' Plots the total variation versus variation within replicates
#'
#' This function plots the total variation and the variation within replicates
#' for a given value. If decoys are present these are removed before plotting.
#'
#' @param data Data table that is produced by the OpenSWATH/pyProphet workflow.
#' @param column.values  Indicates the columns for which the variation is
#'   assessed. This can be the Intensity or Signal, but also the retention time.
#' @param comparison1 The comparison for assessing the total
#'   variability. Default is to assess the variability per transition_group_id
#'   over the combination of Replicates and different Conditions.
#' @param comparison2  The comparison for assessing the variability within the
#'   replicates. Default is to assess the variability per transition_group_id
#'   and Condition over the different Replicates.
#' @param fun_aggregate If depending on the comparison values have to be
#'   aggregated one needs to provide the function here. (I think this should be
#'   sum, yesno?)
#' @param label  Option to print value of median cv.
#' @param title Title of plot. Default: "cv across conditions"
#' @param boxplot   Logical. If boxplot should be plotted. Default: TRUE
#' @param ...  Arguments passed through, currently unused.
#' @return Plots in Rconsole a violin plot comparing the total variation with
#'   the variation within replicates. In addition it returns the data frame from
#'   which the plotting is done and a table with the calculated mean, median and
#'   mode of the cv for the total or replicate data.
#' @author Peter Blattmann
#' @examples {
#'  data("OpenSWATH_data", package="SWATH2stats")
#'  data("Study_design", package="SWATH2stats")
#'  data <- sample_annotation(OpenSWATH_data, Study_design)
#'  var_summary <- plot_variation_vs_total(data)
#'  }
#' @importFrom ggplot2 ggplot geom_violin aes_string geom_tile 
#'   scale_fill_gradient xlab ylab labs geom_text geom_boxplot scale_x_discrete 
#'   scale_y_discrete theme element_text stat_summary
#' @importFrom stats sd na.omit aggregate density
#' @importFrom reshape2 dcast
#' @export
plot_variation_vs_total <- function(data, 
                                    column.values = "Intensity", 
                                    comparison1 = transition_group_id ~ BioReplicate + Condition, 
                                    comparison2 = transition_group_id + Condition ~ BioReplicate,
                                    fun_aggregate = NULL, 
                                    label = FALSE, 
                                    title = "coefficient of variation - total versus within replicates",
                                    boxplot = TRUE, ...){
    if (sum(colnames(data) == "decoy") == 1){
        data <- data[data$decoy == 0, ]
    }
    data1.c <- dcast(data, comparison1, value.var = column.values, 
                     fun.aggregate = fun_aggregate)
    data1.c[data1.c == 0] <- NA
    data1.sd <- apply(data1.c[, 2:dim(data1.c)[2]], 1, 
                      function(x) sd(x, na.rm = TRUE))
    data1.mean <- apply(data1.c[, 2:dim(data1.c)[2]], 1, 
                        function(x) mean(x, na.rm = TRUE))
    data1.c$cv <- data1.sd/data1.mean
    mean1.cv <- mean(data1.c$cv, na.rm = TRUE)
    median1.cv <- median(data1.c$cv, na.rm = TRUE)

    data2.c <- dcast(data, comparison2, value.var = column.values, 
                     fun.aggregate = fun_aggregate)
    data2.c[data2.c == 0] <- NA
    n_vars2 <- length(all.vars(comparison2))

    data2.sd <- apply(data2.c[, n_vars2:dim(data2.c)[2]], 1, 
                      function(x) sd(x, na.rm = TRUE))
    data2.mean <- apply(data2.c[, n_vars2:dim(data2.c)[2]], 1, 
                        function(x) mean(x,
        na.rm = TRUE))
    data2.c$cv <- data2.sd/data2.mean
    mean2.cv <- mean(data2.c$cv, na.rm = TRUE)
    median2.cv <- median(data2.c$cv, na.rm = TRUE)

    data2.c$rep <- paste(data2.c[, 1], data2.c[, 2])
    data2.c$scope <- "replicate"
    data1.c$rep <- data1.c[, 1]
    data1.c$scope <- "total"

    data.comb <- rbind(data1.c[, c("rep", "cv", "scope")], data2.c[, c("rep", "cv",
        "scope")])
    data.comb$scope <- factor(data.comb$scope, 
                              levels = c("total", "replicate"))
    p <- ggplot(na.omit(data.comb), aes_string(x = "scope", y = "cv")) +
        geom_violin(scale = "area") +
        xlab("") +
        theme(axis.text.x = element_text(size = 8, angle = 90, hjust = 1,
                                         vjust = 0.5)) +
        labs(title = title) +
        theme_bw()
    if (label) {
        p <- p +
          stat_summary(
            fun.data = function(x) data.frame("y" = max(x) * 0.75,
                                              label = paste("median cv:\n", 
                                                            signif(median(x, na.rm = TRUE), 
                                                                   digits = 2))),
            geom = "text")
    }
    if (boxplot) {
        p <- p + geom_boxplot(width = 0.1, outlier.shape = NA)
    }
    print(p)

    median <- aggregate(data.comb[, "cv"], by = list(data.comb$scope),
                        FUN = function(x) median(x, na.rm = TRUE))
    colnames(median) <- c("scope", "median_cv")
    mean <- aggregate(data.comb[, "cv"], by = list(data.comb$scope),
                      FUN = function(x) mean(x, na.rm = TRUE))
    colnames(mean) <- c("scope", "mean_cv")
    mode <- aggregate(data.comb[, "cv"], by = list(data.comb$scope),
                      FUN = function(x) {
                        d <- density(x, na.rm = TRUE)
                        i <- which.max(d$y)
                        return(d$x[i])
                      })
    colnames(mode) <- c("scope", "mode_cv")

    cv_table <- merge(mode, merge(mean, median, by = "scope"), by = "scope")
    return(list(data.comb, cv_table))
}

#' Assess overall FDR in annotated OpenSWATH/pyProphet output table in
#' dependence of m_score cutoff
#'
#' This function estimates the assay, peptide and protein FDR over a multi-run
#' OpenSWATH/pyProphet output table. It counts target and decoy assays (unique
#' transition_group_id), peptides (unique FullPeptideName) and proteins (unique
#' ProteinName) in dependence of the m-score cutoff (1e-2 to 1e-20).
#' To arrive from decoy counts at an estimation of the false discovery rate
#' (false positives among the targets remaining at a given mscore cutoff) the
#' ratio of false positives to true negatives (decoys) (FFT) must be
#' supplied. It is estimated for each run individually by pyProphet and
#' contained in the pyProphet statistics [Injection_name]_full_stat.csv. As an
#' approximation, the FFTs of multiple runs are averaged and supplied as
#' argument FFT. For further details see the Vignette Section 1.3 and
#' 4.1. Protein FDR control on peak group quality level is a very strict filter
#' and should be handled with caution.
#' FDR is calculated as FDR = (TN*FFT/T); TN=decoys, T=targets, FFT=see above
#'
#' @param data  Data table that is produced by the OpenSWATH/pyProphet workflow
#' @param FFT Ratio of false positives to true negatives, q-values from
#'   [Injection_name]_full_stat.csv in pyProphet stats output. As an
#'   approximation, the q-values of multiple runs are averaged and supplied as
#'   argument FFT. Numeric from 0 to 1. Defaults to 1, the most conservative
#'   value (1 Decoy indicates 1 False target).
#' @param n_range  I am also not certain what this is, nor why 20 is the
#'   optimal default value, but I think the idea is to set up a series of mscore
#'   thresholds.
#' @param output Choose output type. "pdf_csv" creates the output as files in
#'   the working directory, "Rconsole" triggers delivery of the output to the
#'   console enabling further computation or custom plotting / output.
#' @param plot  Logical, whether or not to create plots from the results (using
#'   the associated method plot.fdr_table()
#' @param filename  Optional, modifying the basename of the result files if
#'   applicable.
#' @param score_col Column that contains the score. Default. m_score 
#' @return Returns a list of class "fdr_table". If output "pdf_csv" and plot =
#'   TRUE were chosen, report files are written to the working folder.
#' @author Moritz Heusel
#' @examples{
#'  data("OpenSWATH_data", package="SWATH2stats")
#'  data("Study_design", package="SWATH2stats")
#'  data <- sample_annotation(OpenSWATH_data, Study_design)
#'  assess_fdr_overall(data, FFT=0.7, output="Rconsole", plot=TRUE,
#'                     filename="Testoutput_assess_fdr_overall")
#' }
#' @importFrom utils write.csv
#' @export

assess_fdr_overall <- function(data, 
                               FFT = 1, 
                               n_range = 20, 
                               output = "pdf_csv", 
                               plot = TRUE,
                               filename = "FDR_report_overall", 
                               score_col = "m_score") {
    score_col <- JPP_update(data, score_col)

    mscore_levels <- 10^-seq(n_range)

    # create vectors to store count results
    target.assays <- NULL
    decoy.assays <- NULL
    target.peptides <- NULL
    decoy.peptides <- NULL
    target.proteins <- NULL
    decoy.proteins <- NULL

    # loop over IDs with different m_score thresholds counting targets & decoys
    for (i in seq_len(length(mscore_levels))) {
        target.assays[i] <- length(unique(data[data$decoy == FALSE & data[, score_col] <=
            mscore_levels[i], c("transition_group_id")]))
        decoy.assays[i] <- length(unique(data[data$decoy == TRUE & data[, score_col] <=
            mscore_levels[i], c("transition_group_id")]))
        target.peptides[i] <- length(unique(data[data$decoy == FALSE & data[, score_col] <=
            mscore_levels[i], c("FullPeptideName")]))
        decoy.peptides[i] <- length(unique(data[data$decoy == TRUE & data[, score_col] <=
            mscore_levels[i], c("FullPeptideName")]))
        target.proteins[i] <- length(unique(data[data$decoy == FALSE & data[, score_col] <=
            mscore_levels[i], c("ProteinName")]))
        decoy.proteins[i] <- length(unique(data[data$decoy == TRUE & data[, score_col] <=
            mscore_levels[i], c("ProteinName")]))
    }

    # calculate false target fraction at cutoff (FDR) by decoy fraction * FFT
    assay.fdr <- (decoy.assays/target.assays) * FFT
    peptide.fdr <- (decoy.peptides/target.peptides) * FFT
    protein.fdr <- (decoy.proteins/target.proteins) * FFT
    # calculate estimated number of true identifications among the target hits
    true.target.assays <- target.assays - (decoy.assays * FFT)
    true.target.peptides <- target.peptides - (decoy.peptides * FFT)
    true.target.proteins <- target.proteins - (decoy.proteins * FFT)


  fdr_table <- structure(list(
    mscore_cutoff = mscore_levels,
    target.assays = target.assays,
    decoy.assays = decoy.assays,
    assay.fdr = assay.fdr,
    true.target.assays = true.target.assays,
    target.peptides = target.peptides,
    decoy.peptides = decoy.peptides,
    peptide.fdr = peptide.fdr,
    true.target.peptides = true.target.peptides,
    target.proteins = target.proteins,
    decoy.proteins = decoy.proteins,
    protein.fdr = protein.fdr,
    true.target.proteins = true.target.proteins),
        class = "fdr_table")

    if (isTRUE(plot)) {
        plot.fdr_table(fdr_table, filename = filename, output = output)
    }

    if (output == "Rconsole") {
        return(fdr_table)
    }

    if (output == "pdf_csv") {
        fdr_table_csv <- cbind(mscore_levels, target.assays, decoy.assays, assay.fdr,
            true.target.assays, target.peptides, decoy.peptides, peptide.fdr, true.target.peptides,
            target.proteins, decoy.proteins, protein.fdr, true.target.proteins)
        write.csv(fdr_table_csv, file = paste(filename, "table.csv", sep = "_"),
            row.names = TRUE)
        message(filename, "table.csv written to working folder", "\n")
    }

}

#' Filter annotated OpenSWATH/pyProphet output table to achieve a high FDR
#' quality data matrix with controlled overall protein FDR and quantitative
#' values for all peptides mapping to these high-confidence proteins (up to a
#' desired overall peptide level FDR quality).
#'
#' This function controls the protein FDR over a multi-run OpenSWATH/pyProphet
#' output table and filters all quantitative values to a desired overall/global
#' peptide FDR level.
#' It first finds a suitable m-score cutoff to minimally achieve a desired
#' global FDR quality on a protein master list based on the function
#' mscore4protfdr. It then finds a suitable m-score cutoff to minimally achieve
#' a desired global FDR quality on peptide level based on the function
#' mscore4pepfdr. Finally, it reports all the peptide quantities derived based
#' on the peptide level cutoff for only those peptides mapping to the protein
#' master list. It further summarizes the protein and peptide numbers remaining
#' after the filtering and evaluates the individual run FDR qualities of the
#' peptides (and quantitation events) selected.
#'
#' @param data  Annotated OpenSWATH/pyProphet data table.
#' @param FFT  Ratio of false positives to true negatives, q-values from
#'   [Injection_name]_full_stat.csv in pyProphet stats output. As an
#'   approximation, the q-values of multiple runs are averaged and supplied as
#'   argument FFT. Numeric from 0 to 1. Defaults to 1, the most conservative
#'   value (1 Decoy indicates 1 False target). For further details see the
#'   Vignette Section 1.3 and 4.1.
#' @param overall_protein_fdr_target FDR target for the protein master list for
#'   which quantitative values down to the less strict peptide_fdr criterion
#'   will be kept/reported. Defaults to 0.02.
#' @param upper_overall_peptide_fdr_limit  Option to relax or tighten the false
#'   discovery rate limit.
#' @param rm_decoy Logical T/F, whether decoy entries should be removed after
#'   the analysis. Defaults to TRUE. Can be useful to disable to track the
#'   influence on decoy fraction by further filtering steps such as requiring 2
#'   peptides per protein.
#' @param score_col Defines the column from which to retrieve the m_score. 
#'    If you use JPP (Rosenberger, Bludau et al. 2017) this can be used to 
#'    select between Protein and transition_group m_score.
#' @return Returns a data frame with the filtered data.
#' @author Moritz Heusel
#' @examples
#'  data("OpenSWATH_data", package="SWATH2stats")
#'  data("Study_design", package="SWATH2stats")
#'  data <- sample_annotation(OpenSWATH_data, Study_design)
#'  data.fdr.filtered<-filter_mscore_fdr(data, FFT=0.7, 
#'                                       overall_protein_fdr_target=0.02,
#'                                       upper_overall_peptide_fdr_limit=0.1)
#' @export
filter_mscore_fdr <- function(data, 
                              FFT = 1, 
                              overall_protein_fdr_target = 0.02, 
                              upper_overall_peptide_fdr_limit = 0.05,
                              rm_decoy = TRUE, 
                              score_col = "m_score") {
    score_col <- JPP_update(data, score_col)

    mscore4protfdr_target <- mscore4protfdr(data, FFT, fdr_target = overall_protein_fdr_target)

    if (is.na(mscore4protfdr_target)) {
        stop("The overall_protein_fdr_target cannot be reached in this dataset.\n
         Consider using a higher accepted FDR criterion. \n
         Check mProphet models for target-decoy separation.")
    }

    # Initiate reporting to console
    message("filter_mscore_fdr is filtering the data...", "\n")
    message("-----------------------------------------------------------", "\n")
    message("finding m-score cutoff to achieve desired protein FDR in protein master list..",
        "\n")
    # Create master list at strict protein level FDR criterion
    protein_master_list <- unique(subset(data, data[, score_col] <= mscore4protfdr_target)$ProteinName)

    # Pre-Filter data based on upper_overall_peptide_fdr_limit
    message("finding m-score cutoff to achieve desired global peptide FDR..", "\n")
    data.f1 <- subset(data, data[, score_col] <= mscore4pepfdr(data, FFT, fdr_target = upper_overall_peptide_fdr_limit))

    # Filter prefiltered data down to entries mapping to the protein_master_list
    data.f2 <- subset(data.f1, data.f1$ProteinName %in% protein_master_list)

    # count remaining entries
    proteins <- length(protein_master_list)
    proteins.t <- length(unique(data.f2[data.f2$decoy == FALSE, c("ProteinName")]))
    proteins.d <- length(unique(data.f2[data.f2$decoy == TRUE, c("ProteinName")]))
    total.peptides <- length(unique(data.f1$FullPeptideName))
    total.peptides.t <- length(unique(data.f1[data.f2$decoy == FALSE, c("FullPeptideName")]))
    total.peptides.d <- length(unique(data.f1[data.f2$decoy == TRUE, c("FullPeptideName")]))
    mapping.peptides <- length(unique(data.f2$FullPeptideName))
    mapping.peptides.t <- length(unique(data.f2[data.f2$decoy == FALSE, c("FullPeptideName")]))
    mapping.peptides.d <- length(unique(data.f2[data.f2$decoy == TRUE, c("FullPeptideName")]))


    # print some numbers about the filtering results
    message("-------------------------------------------------------------", "\n")
    message("Proteins selected: ", "\n", "Total proteins selected: ", proteins, "\n",
        "Thereof target proteins: ", proteins.t, "\n", "Thereof decoy proteins: ",
        proteins.d, "\n")
    message("Peptides mapping to these protein entries selected:", "\n", "Total mapping peptides: ",
        mapping.peptides, "\n", "Thereof target peptides: ", mapping.peptides.t,
        "\n", "Thereof decoy peptides: ", mapping.peptides.d, "\n")
    message("Total peptides selected from:", "\n", "Total peptides: ", total.peptides,
        "\n", "Thereof target peptides: ", total.peptides.t, "\n", "Thereof decoy peptides: ",
        total.peptides.d, "\n")
    message("-------------------------------------------------------------", "\n")

    # test if all runs contain a decoy after peptide FDR filering in order to
    # calculate the local FDR
    n.run_id <- length(unique(data.f1$run_id))
    n.run_id_decoy <- length(unique(data.f1[data.f1$decoy == TRUE & data.f1[, score_col] <=
        0.01, ]$run_id))
    if (n.run_id == n.run_id_decoy) {

        fdr_cube <- assess_fdr_byrun(data.f1, FFT, output = "Rconsole", plot = FALSE)

        message("-------------------------------------------------------------",
            "\n")
        message("Individual run FDR quality of the peptides selected from:", "\n")
        message(signif(mean(fdr_cube[8, , 1]), 3))
    }
    if (n.run_id != n.run_id_decoy) {
        message("Individual run FDR quality of the peptides was not calculated",
            "\n", "as not every run contains a decoy.", "\n")
    }

    # return filtered data with or without decoy entries
    if (rm_decoy == FALSE) {
        message("The decoys have NOT been removed from the returned data", "\n")
        return(data.f2)
    } else {
        message("The decoys have been removed from the returned data", "\n")
        return(data.f2[data.f2$decoy == FALSE, ])
    }
}

#' S3 plot function for results of class "fdr_table" as produced by e.g. the
#' function assess_fdr_overall()
#'
#' This function created standard plots from results of class "fdr_table" as
#' produced by e.g. the function assess_fdr_overall() visualizig ID numbers in
#' dependence of estimated FDR and also estimated FDR in dependence of m_score
#' cutoff.
#'
#' @param x  List of class "fdr_table" as produced e.g. by the function
#'   assess_fdr_overall() from this package.
#' @param output Choose output type. "pdf_csv" creates the output as files in
#'   the working directory, "Rconsole" triggers delivery of the output to the
#'   console enabling further computation or custom plotting / output.
#' @param filename  Basename for output files to be created (if output =
#'   "pdf_csv" has been selected).
#' @param ...  Extra arguments passed on to functions inside this.
#' @return  Plots in Rconsole or report files.
#' @author Moritz Heusel
#' @examples{
#'  data("OpenSWATH_data", package="SWATH2stats")
#'  data("Study_design", package="SWATH2stats")
#'  data <- sample_annotation(OpenSWATH_data, Study_design)
#'  x <- assess_fdr_overall(data, FFT=0.7, output="Rconsole", plot=FALSE)
#'  plot(x, output="Rconsole", filename="Assess_fdr_overall_testplot")
#'  }
#' @importFrom graphics par lines legend mtext points axis grid abline
#' @importFrom grDevices pdf dev.off
#' @export
plot.fdr_table <- function(x, 
                           output = "Rconsole", 
                           filename = "FDR_report_overall",
                           ...) {
    ## Plot and create output from ID-FDR report (x) ## Plot 1: Target/true target
    ## curves as estimated by decoy counting + FFT-correction

    # Save previous par(mfrow) settings and restore upon exit (BioConductor
    # suggestion)
    par.mfrow.old <- par(mfrow = par()$mfrow)
    on.exit(par(par.mfrow.old), add = TRUE)
    par.mar.old <- par(mar = par()$mar)
    on.exit(par(par.mar.old), add = TRUE)

    # Open pdf if output as pdf was desired by user
    if (output == "pdf_csv") {
        pdf(file = paste0(filename, ".pdf"), height = 6, width = 10)
        par(mfrow = c(1, 3), mar = c(13, 4, 15, 2) + 0.1)
    }

    # plot 1.1 Assay-level sensitivity
    plot(x$assay.fdr, x$target.assays, 
         xlab = "assay FDR", ylab = "# of assays",
         type = "l", lty = 2, 
         xlim = c(0, max(x$assay.fdr, na.rm = TRUE)), 
         ylim = (c(0, max(x$target.assays))))
    lines(x$assay.fdr, x$true.target.assays, xlim = c(0, max(x$assay.fdr)))
    legend("topleft", legend = c("all targets", "true targets"), 
           cex = 0.5, lty = c(2,1))

    # plot 1.2 Peptide-level sensitivity
    plot(x$peptide.fdr, x$target.peptides, 
         xlab = "peptide FDR", ylab = "# of peptides",
         type = "l", lty = 2, 
         xlim = c(0, max(x$peptide.fdr, na.rm = TRUE)), 
         ylim = (c(0, max(x$target.peptides))))
    lines(x$peptide.fdr, x$true.target.peptides, 
          xlim = c(0, max(x$peptide.fdr)))
    legend("topleft", legend = c("all targets", "true targets"), 
           cex = 0.5, lty = c(2, 1))

    if (output == "pdf_csv") {
        mtext("SWATH2stats global FDR & sensitivity report of OpenSWATH/pyProphet results",
            line = 8, cex = 1.5)
        mtext("Overall sensitivity:", line = 3, adj = 0.5, cex = 1.1)
    }

    # plot 1.3 Protein-level sensitivity
    plot(x$protein.fdr, x$target.proteins, 
         xlab = "protein FDR", ylab = "# of proteins",
         type = "l", lty = 2, 
         xlim = c(0, max(x$protein.fdr, na.rm = TRUE)),
         ylim = (c(0, max(x$target.proteins))))
    lines(x$protein.fdr, x$true.target.proteins, 
          xlim = c(0, max(x$protein.fdr)))
    legend("topleft", legend = c("all targets", "true targets"), 
           cex = 0.5, lty = c(2, 1))

    # Plot 2: Global m_score adjustment and connectivity to global FDR quality 
    # levels as estimated by decoy counting + FFT-correction
    xlimit <- -1 * (length(x$assay.fdr) + 1)

    par(mar = c(5, 8, 4, 8) + 0.1, mfrow = c(1, 1))
    plot(log10(x$mscore_cutoff), x$assay.fdr, axes = FALSE, 
         ylim = c(0, 1.1 * max(x$assay.fdr, na.rm = TRUE)), 
        main = "Global m-score cutoff connectivity to FDR quality",
        xlab = "", ylab = "", 
        type = "l", col = "black", 
        xlim = c(xlimit, 0))
    points(log10(x$mscore_cutoff), x$assay.fdr, pch = 20, col = "black")
    axis(2, ylim = c(0, 1.1 * max(x$assay.fdr, na.rm = TRUE)), 
         col = "black", lwd = 2)
    
    mtext(2, text = "Assay FDR", line = 2)
    par(new = TRUE)
    plot(log10(x$mscore_cutoff), x$peptide.fdr, 
         axes = FALSE, 
         ylim = c(0, 1.1 * max(x$peptide.fdr, na.rm = TRUE)), 
         xlab = "", ylab = "", type = "l", lwd = 2, col = "red", main = "",
         xlim = c(xlimit, 0), lty = 2)
    points(log10(x$mscore_cutoff), x$peptide.fdr, pch = 20, col = "red")
    axis(2, ylim = c(0, 1.1 * max(x$peptide.fdr, na.rm = TRUE)), 
         col = "red", lwd = 2, line = 3.5)
    
    mtext(2, text = "Peptide FDR", col = "red", line = 5.5)
    par(new = TRUE)
    plot(log10(x$mscore_cutoff), x$protein.fdr, axes = FALSE, 
         ylim = c(0, 1.1 * max(x$protein.fdr, na.rm = TRUE)), 
         col = "blue", xlab = "", ylab = "", type = "l", lty = 3,
         main = "", xlim = c(xlimit, 0), lwd = 2)
    points(log10(x$mscore_cutoff), x$protein.fdr, col = "blue", pch = 20)
    axis(4, ylim = c(0, 1.1 * max(x$protein.fdr, na.rm = TRUE)), 
         col = "blue", lwd = 2)
    mtext(4, text = "Protein FDR", col = "blue", line = 2)
    axis(1, at = seq(min(log10(x$mscore_cutoff)), max(log10(x$mscore_cutoff))))
    grid()
    abline(v = seq(min(log10(x$mscore_cutoff)), max(log10(x$mscore_cutoff))), 
           col = "grey", lty = 3)
    mtext(1, text = "log10(m_score cutoff)", col = "black", line = 2)
    
    if (output == "pdf_csv") {
        dev.off()
        message(filename, ".pdf written to working folder", "\n")
    }
}

#' Find m_score cutoff to reach a desired FDR on peptide level (over the entire
#' OpenSWATH/pyProphet output table)
#'
#' This function estimates the m_score cutoff required in a dataset to reach a
#' given overall peptide level FDR.
#' It counts target and decoy peptides (unique FullPeptideName) at high
#' resolution across the m_score cutoffs and reports a useful m_score cutoff -
#' peptide FDR pair close to the supplied fdr_target level over the entire
#' dataset. The m_score cutoff is returned by the function and can be used in
#' the context of the filtering functions, e.g.:
#' data.pepFDR2pc<-filter_mscore(data, mscore4pepfdr(data, fdr_target=0.02))
#' To arrive from decoy counts at an estimation of the false discovery rate
#' (false positives among the targets remaining at a given mscore cutoff) the
#' ratio of false positives to true negatives (decoys) (FFT) must be
#' supplied. It is estimated for each run individually by pyProphet and
#' contained in the pyProphet statistics [Injection_name]_full_stat.csv. As an
#' approximation, the FFTs of multiple runs are averaged and supplied as
#' argument FFT. For further details see the Vignette Section 1.3 and 4.1.
#' For FDR evaluations on assay and protein level, please refer to functions
#' mscore4assayfdr and mscore4protfdr
#'
#' @param data Annotated OpenSWATH/pyProphet data table. See function
#'   sample_annotation from this package.
#' @param FFT  Ratio of false positives to true negatives, q-values from
#'   [Injection_name]_full_stat.csv in pyProphet stats output. As an
#'   approximation, the q-values of multiple runs are averaged and supplied as
#'   argument FFT. Numeric from 0 to 1. Defaults to 1, the most conservative
#'   value (1 Decoy indicates 1 False target).
#' @param fdr_target  FDR target, numeric, defaults to 0.01. An m_score cutoff
#'   achieving an FDR < fdr_target will be selected.
#'   Calculated as FDR = (TN*FFT/T); TN=decoys, T=targets, FFT=see above.
#' @param mscore.col column in the data containing the m score data.
#' @return Returns the m_score cutoff selected to arrive at the desired FDR
#' @author Moritz Heusel
#' @examples
#'  data("OpenSWATH_data", package="SWATH2stats")
#'  data("Study_design", package="SWATH2stats")
#'  data <- sample_annotation(OpenSWATH_data, Study_design)
#'  chosen <- mscore4pepfdr(data, FFT=0.7, fdr_target=0.01)
#' @export
mscore4pepfdr <- function(data, FFT = 1, 
                          fdr_target = 0.01, 
                          mscore.col = "m_score") {

    mscore.col <- JPP_update(data, mscore.col)

    # generate high resolution mscore levels to assess mscore cutoff for a given
    # fdr_target
    mscore_levels_highres = 10^-(c(seq(2, 20, 0.05)))
    target.peptides.highres <- NULL
    decoy.peptides.highres <- NULL
    for (i in seq_len(length(mscore_levels_highres))) {
        target.peptides.highres[i] <- length(unique(data[data$decoy == FALSE & data[,
            mscore.col] <= mscore_levels_highres[i], c("FullPeptideName")]))
        decoy.peptides.highres[i] <- length(unique(data[data$decoy == TRUE & data[,
            mscore.col] <= mscore_levels_highres[i], c("FullPeptideName")]))
    }
    peptide.fdr.highres <- (decoy.peptides.highres/target.peptides.highres) * FFT

    # pick mscore cutoff closest to (<=) fdr_target % peptide FDR & filter data
    # accordingly
    mscore_chosen <- mscore_levels_highres[peptide.fdr.highres <= fdr_target][1]
    peptide_fdr_chosen <- peptide.fdr.highres[peptide.fdr.highres <= fdr_target][1]
    message("Target peptide FDR:", fdr_target, "\n")
    message("Required overall m-score cutoff:", signif(mscore_chosen, digits = 5),
        "\n", "achieving peptide FDR =", signif(peptide_fdr_chosen, digits = 3),
        "\n")
    return(mscore_chosen)
}

#' Annotate the SWATH data with the sample information
#'
#' For statistical analysis and filtering the measurements need to be annotated
#' with Filename, Condition, BioReplicate, and Run. This functions takes this
#' information from a txt file containing this meta-data.
#'
#' Given dataframes of TRIC processed data and sample annotations, mash them
#' together into something appropriate for downstream analyses.
#'
#' This performs some quick sanity checks on the data and annotations and
#' creates the 'Condition', 'BioReplicate', and 'Run' columns along with other
#' columns expected by MSstats/OpenSWATH.
#'
#' @param data A data frame containing SWATH data.
#' @param sample_annotation A data frame containing the columns: Filename,
#'   Condition, BioReplicate, Run. The values contained in the column filename
#'   have to be present in the filename of the SWATH data.
#' @param data_type Option to specify the format of the table, if the column
#'   names from an OpenSWATH output or MSstats table are used.
#' @param column_file Option to specify the column name where the injection 
#'    file is specified. Default is set to "filename".
#' @param change_run_id Option to choose if the run_id column shall be
#'   reassigned to a unique value combining the values of Condition,
#'   BioReplicate and Run. (Option only possible if data is of format
#'   "OpenSWATH")
#' @param verbose Option to turn on reporting on which filename it is working
#'   on.
#' @return Returns a dataframe with each row annotated for the study design
#' @author Peter Blattmann
#' @examples
#'  data("OpenSWATH_data", package="SWATH2stats")
#'  data("Study_design", package="SWATH2stats")
#'  data <- SWATH2stats::sample_annotation(OpenSWATH_data, Study_design, verbose=TRUE)
#'  summary(data)
#' @importFrom methods is
#' @export
sample_annotation <- function(data, 
                              sample_annotation, 
                              data_type = "OpenSWATH",
                              column_file = "filename",
                              change_run_id = TRUE, 
                              verbose = FALSE) {
    ## annotate sample needs a txt file with the columns Filename, Condition,
    ## BioReplicate, Run. In Filename a unique string contained in File must be
    ## contained.
    if (!is(data, "data.frame")) {
        stop("Input data is not a data.frame")
    }

    if (!(column_file %in% colnames(data))) {
        stop("Column for filename is not present in data file")
    }
    if (nlevels(factor(sample_annotation$Filename)) != nlevels(factor(data[, column_file]))) {
        warning("The number of sample annotation condition and filenames in data are not balanced.",
            "\n", "Different filenames in sample annotation file: ", nlevels(factor(sample_annotation$Filename)),
            "\n", "Different filenames in data file: ", nlevels(factor(data[, column_file])))
    }

    for (i in seq_len(nrow(sample_annotation))) {
        n.found <- grep(sample_annotation[i, "Filename"], sample_annotation[, "Filename"])
        if (length(n.found) > 1) {
            stop("The values in the column filename are not unique and will lead to erroneous results because the following string matches to multiple different filenames: ",
                sample_annotation[i, "Filename"])
        }
    }

    if (data_type %in% c("OpenSWATH", "MSstats")) {

        for (i in levels(factor(sample_annotation$Filename))) {
            if (verbose) {
                message(i)
            }

            coord <- grep(i, data[, column_file], fixed = TRUE)

            if (length(coord) == 0) {
                warning("No measurement value found for this sample in the data file: ",
                  print(i))
            }

            data.subset <- sample_annotation[which(i == sample_annotation$Filename),
                ]
            data[coord, "Condition"] <- data.subset[, "Condition"]
            data[coord, "BioReplicate"] <- data.subset[, "BioReplicate"]
            data[coord, "Run"] <- data.subset[, "Run"]
        }

        # select column names
        if (data_type == "OpenSWATH") {
            add.colnames <- colnames(data)[!(colnames(data) %in% c("ProteinName",
                "FullPeptideName", "Charge", "aggr_Fragment_Annotation", 
                "aggr_Peak_Area", "Condition", "BioReplicate", "Run"))]

            sel.colnames <- c("ProteinName", "FullPeptideName", "Charge", 
                              "aggr_Fragment_Annotation", "aggr_Peak_Area", 
                              "Condition", "BioReplicate", "Run", add.colnames)
        }
        if (data_type == "MSstats") {
            add.colnames <- colnames(data)[!(colnames(data) %in% c("ProteinName",
                "PeptideSequence", "PrecursorCharge", "FragmentIon", 
                "ProductCharge", "IsotopeLabelType", "Condition", 
                "BioReplicate", "Run", "Intensity"))]

            sel.colnames <- c("ProteinName", "PeptideSequence", 
                              "PrecursorCharge", "FragmentIon", "ProductCharge", 
                              "IsotopeLabelType", "Condition", "BioReplicate", 
                              "Run", "Intensity", add.colnames)
        }
        data <- data[, sel.colnames]

        if (change_run_id) {
            data$run_id <- paste(data$Condition, data$BioReplicate, data$Run, 
                                 sep = "_")
        }

        return(data)
    } else (print("Type of data not recognized"))
}

#' @name MSstats_data
#' @docType data
#' @title Testing dataset in MSstats format. 
#' @description A small table with the column names corresponding to the MSstats format. 
#'  This data is intended only to test functions. 
#' @author Peter Blattmann
NULL

#' @name OpenSWATH_data
#' @aliases data
#' @docType data
#' @title Testing dataset from OpenSWATH.
#' @description A small selection of the data obtained from the iPortal pipeline for an 
#'  SWATH/DIA experiment with perturbations relating to cholesterol regulation. 
#'  Protein and Peptides have been anonymized as the data is unpublished.
#'  The FDR version of the test data contains modified (lowered) decoy peak 
#'  group m_scores to simulate FDR behaviour of a large dataset.
#' @author Peter Blattmann
NULL

#' @name Spyogenes
#' @docType data
#' @title S.pyogenes example data.
#' 
#' @description A table containing SWATH-MS data from S.pyogenes.
#'  This table was generated from the original data deposited on PeptideAtlas 
#'  (PASS00289, file "rawOpenSwathResults_1pcnt_only.tsv") by selecting only 
#'  the column necessary for the SWATH2stats. 
#'
#' @references Rost, H. L., et al. (2014). OpenSWATH enables automated, 
#'   targeted analysis of data-independent acquisition MS data. 
#'   Nat Biotechnol 32(3): 219-223. 
NULL


#' @name Study_design
#' @docType data
#' @title A table containing the meta-data defining the study design of the OpenSWATH data.
#' 
#' @description This table contains a unique identifier in the column "Filename"
#'  corresponding to the filename in the SWATH data.
#'  In the column "Condition" the perturbation performed is described.
#'  In the column "BioReplicate" the biological replicate is indicated.
#'  In the column "Run" a unique identifier for each injection.
#'  Technical injections would have different Run numbers but the same 
#'  BioReplicate number.
#' 
#' @author Peter Blattmann
NULL

#' Reduce columns of OpenSWATH data
#'
#' This function selects the columns from the standard OpenSWATH output to
#' column needed for MSstats, aLFQ and mapDIA.
#'
#' @note A basic set of columns are defined in the function and are used if no column
#'   names are indicated.
#' @note The column.names can be omitted and then the following columns are
#'   selected that are needed for MSstats and mapDIA analysis: ProteinName,
#'   FullPeptideName, Sequence, Charge, aggr_Fragment_Annotation,
#'   aggr_Peak_Area, filename, m_score, decoy, Intensity, RT.
#'   This function should be ommitted if the data is analyzed afterwards with
#'   the aLFQ or imsbInfer package that needs further columns.
#'
#' @param data  A data frame containing SWATH data.
#' @param column.names A vector of column names that can be selected.
#' @return Returns a data frame with the selected columns.
#' @author Peter Blattmann
#' @examples
#'  data("OpenSWATH_data", package="SWATH2stats")
#'  data("Study_design", package="SWATH2stats")
#'  data <- sample_annotation(OpenSWATH_data, Study_design)
#'  data.filtered <- reduce_OpenSWATH_output(data)
#' @export
reduce_OpenSWATH_output <- function(data, 
                                    column.names = NULL) {
    if (is.null(column.names)) {
        if ("transition_group_id_m_score" %in% colnames(data)) {
            column.names <- c("ProteinName", "FullPeptideName", "Sequence", 
                              "Charge","aggr_Fragment_Annotation", 
                              "aggr_Peak_Area", "filename", 
                              "transition_group_id_m_score", 
                              "ProteinName_m_score", "decoy", "Intensity", 
                              "RT", "run_id", "transition_group_id")
        }
        if ("m_score" %in% colnames(data)) {
            column.names <- c("ProteinName", "FullPeptideName", "Sequence", 
                              "Charge", "aggr_Fragment_Annotation", 
                              "aggr_Peak_Area", "filename", "m_score",
                              "decoy", "Intensity", "RT", "run_id", 
                              "transition_group_id")
        }

    }
    if (length(column.names) > length(column.names[column.names %in% colnames(data)])) {
        col.names.missing <- column.names[!column.names %in% colnames(data)]
        warning("These columns are missing from the data:", paste(unlist(col.names.missing),
            collapse = ", "))

    }
    # Keep only required columns for MSStats and mapDIA
    if (length(column.names) == length(column.names[column.names %in% colnames(data)])) {
        data.filtered <- data[, column.names]
        return(data.filtered)
    }
}

#' Get data ready for use by MSstats.
#'
#' Though SWATH2stats uses very similar format as MSstats, some coercion is 
#' required to convert the data into the format for MSstats.
#'
#' This functions selects the columns necessary for MSstats and renames them if
#' necessary.
#'
#' The necessary columns are selected and three columns renamed:
#' FullPeptideName -> PeptideSequence
#' Charge -> PrecursorCharge
#' filename -> File
#'
#' @param data  A data frame containing SWATH data.
#' @param replace_values Option to indicate if negative and 0 values should be replaced with NA.
#' @param replace_colnames Option to indicate if column names should be renamed
#'   and columns reduced to the necessary columns for MSstats.
#' @param replace_unimod Option to indicate if Unimod Identifier should be
#'   replaced from ":" to "_".
#' @return Returns a data frame in the appropriate format for MSstats.
#' @references Choi M, Chang CY, Clough T, Broudy D, Killeen T, MacLean B, Vitek
#'   O. MSstats: an R package for statistical analysis of quantitative mass
#'   spectrometry-based proteomic experiments.Bioinformatics. 2014 Sep
#'   1;30(17):2524-6. doi: 10.1093/bioinformatics/btu305.
#' @author Peter Blattmann
#' @examples
#'  data("OpenSWATH_data", package="SWATH2stats")
#'  data("Study_design", package="SWATH2stats")
#'  data <- sample_annotation(OpenSWATH_data, Study_design)
#'  data.filtered.decoy <- filter_mscore(data, 0.01)
#'  raw <- disaggregate(data.filtered.decoy)
#'  data.mapDIA <- convert4MSstats(raw)
#' @export

convert4MSstats <- function(data, 
                            replace_values = TRUE, 
                            replace_colnames = TRUE,
                            replace_unimod = TRUE) {

    MSstats_columns <- c("ProteinName", "PeptideSequence", "PrecursorCharge", 
                         "FragmentIon", "ProductCharge", "IsotopeLabelType", 
                         "Intensity", "BioReplicate", "Condition", "Run")

    if (isTRUE(replace_colnames)) {
        colnames(data) <- gsub("FullPeptideName", "PeptideSequence", colnames(data))
        colnames(data) <- gsub("^Charge$", "PrecursorCharge", colnames(data))
        colnames(data) <- gsub("filename", "File", colnames(data))
    }
    missing_columns <- MSstats_columns[!(MSstats_columns %in% colnames(data))]
    if (length(missing_columns) > 0) {
        message("One or several columns required by MSstats were not in the data. 
                The columns were created and filled with NAs.\nMissing columns: ",
            paste(unlist(missing_columns), collapse = ", "))

        if ("IsotopeLabelType" %in% missing_columns) {
            message("IsotopeLabelType was filled with light.")
        }

        data[, missing_columns] <- NA

        if ("IsotopeLabelType" %in% missing_columns) {
            data[, "IsotopeLabelType"] <- "light"
        }
        if ("PrecursorCharge" %in% missing_columns) {
            data[, "PrecursorCharge"] <- gsub(".*_([[:digit:]])$", "\\1", 
                                              data[,"FragmentIon"])
        }
    }
    data <- data[, MSstats_columns]

    if (isTRUE(replace_values)) {
        # replace negative values to 0 and 0 to NA
        if (sum(data$Intensity < 0, na.rm = TRUE) > 0) {
            data[data$Intensity < 0, "Intensity"] <- 0
            warning("Negative intensity values were replaced by NA")
        }
        if (sum(data$Intensity == 0, na.rm = TRUE) > 0) {
            data[data$Intensity %in% 0, "Intensity"] <- NA
            warning("Intensity values that were 0, were replaced by NA")
        }
    }

    if (isTRUE(replace_unimod)) {
        # replace UniMod: to UniMod_
        data$PeptideSequence <- gsub("UniMod:", "UniMod_", data$PeptideSequence)
        data$FragmentIon <- gsub("UniMod:", "UniMod_", data$FragmentIon)
    }
    return(data)
}

#' Select alternate m_score column in JPP data and avert user
#' 
#' The output from JPP (Rosenberger, Bludau et al. 2017) has not anymore the 
#' m_score column but an ProteinName_m_score and transition_group_id_m_score. 
#' To make the users aware this function tests if the m_score column still 
#' exists and selects as default the transition_group_id_m_score column.
#' 
#' @param data Data table that is produced by the OpenSWATH/pyProphet workflow
#' @param mscore_col Defines the column from which to retrieve the m_score. 
#'    If you use JPP (Rosenberger, Bludau et al. 2017) this can be used to 
#'    select between Protein and transition_group m_score.
#' @return Returns the mscore_col that might have been changed to 
#'    transition_group_id_m_score and gives a message to the user.
#' @author Peter Blattmann
#' @examples{
#' data("OpenSWATH_data", package="SWATH2stats")
#' JPP_update(OpenSWATH_data, "m_score")
#' }
#' @export
JPP_update <- function(data, mscore_col) {
    if (!(mscore_col %in% colnames(data))) {
        if ("transition_group_id_m_score" %in% colnames(data)) {
            mscore_col <- "transition_group_id_m_score"
            message("The transition_group_id_m_score was used as mscore. 
                    If you want to use Protein q-value please change option 
                    in the function.")
        } else {
            stop("Column for m_score column doesn't exist in data.")
        }
    }
    return(mscore_col)
}

#'  Convert table into the format for ROPECA
#'
#' This functions selects the columns necessary for ROPECA.
#'
#' @param data   A data frame containing SWATH data.
#' @return Returns a data frame in the appropriate format for ROPECA.
#' @note The table must not contain any technical replica, the intensity of
#'   technical replica is averaged. This function requires the package reshape2.
#' @author Peter Blattmann
#' @references Suomi, T. and Elo L.L. (2017). "Enhanced differential expression
#'   statistics for data-independent acquisition proteomics" Scientific Reports
#'   7, Article number: 5869.doi:10.1038/s41598-017-05949-y
#' @examples{
#'  data("OpenSWATH_data", package="SWATH2stats")
#'  data("Study_design", package="SWATH2stats")
#'  data <- sample_annotation(OpenSWATH_data, Study_design)
#'  data.filtered.decoy <- filter_mscore(data, 0.01)
#'  data.PECA <- convert4PECA(data.filtered.decoy)
#' }
#' @importFrom reshape2 dcast
#' @export

convert4PECA <- function(data) {

    columns <- c("ProteinName", "FullPeptideName", "Charge", "Intensity", 
                 "BioReplicate", "Condition", "RT")

    missing_columns <- columns[!(columns %in% colnames(data))]
    if (length(missing_columns) > 0) {
        warning("One or several columns required by mapDIA were not in the data and filled with NAs.
            Missing columns: ",
            paste(unlist(missing_columns), collapse = ", "))

        data[, missing_columns] <- NA
    }

    data$FullPeptideName_Charge <- paste(data$FullPeptideName, data$Charge, sep = "_")

    # # replace UniMod: to UniMod_ data$PeptideSequence <- gsub(':', '_',
    # data$PeptideSequence) data$FragmentIon <- gsub(':', '_', data$FragmentIon)

    data.wide <- dcast(data, ProteinName + FullPeptideName_Charge ~ Condition +
        BioReplicate, fun.aggregate = length, value.var = "Intensity")

    if (sum(data.wide[, 4:dim(data.wide)[2]] > 1) > 0) {
        duplicates.cols <- apply(data.wide[, 4:dim(data.wide)[2]],
            2, function(x) sum(x > 1))
        duplicates.cols <- names(duplicates.cols[duplicates.cols > 0])
        duplicates.rows <- apply(data.wide[, 4:dim(data.wide)[2]],
            1, function(x) sum(x > 1))
        duplicates.rows <- data.wide[duplicates.rows > 0, "FullPeptideName_Charge"]

        warning("Data contains several intensity values per condition\n\n", "in the following columns: ",
            paste(duplicates.cols, collapse = ", "), "\n\n", "and in the following rows: ",
            paste(duplicates.rows, collapse = ", "))
    }

    data.out <- dcast(data, ProteinName + FullPeptideName_Charge ~ Condition + BioReplicate,
        fun.aggregate = mean, value.var = "Intensity")

    data.out <- droplevels(data.out)

    return(data.out)
}

#' Convert table into the format for mapDIA
#'
#' This functions selects the columns necessary for mapDIA.
#'
#' @param data  A data frame containing SWATH data.
#' @param RT   Option to export the retention times.
#' @return Returns a data frame in the appropriate format for mapDIA.
#' @note The table must not contain any technical replica, the intensity of
#'   technical replica is averaged. This function requires the package
#'   reshape2.
#' @references Teo, G., et al. (2015). "mapDIA: Preprocessing and statistical
#'   analysis of quantitative proteomics data from data independent acquisition
#'   mass spectrometry." J Proteomics 129: 108-120.
#' @author Peter Blattmann
#' @examples{
#'  data("OpenSWATH_data", package="SWATH2stats")
#'  data("Study_design", package="SWATH2stats")
#'  data <- sample_annotation(OpenSWATH_data, Study_design)
#'  data.filtered.decoy <- filter_mscore(data, 0.01)
#'  raw <- disaggregate(data.filtered.decoy)
#'  data.mapDIA <- convert4mapDIA(raw, RT=TRUE)
#'  }
#' @importFrom reshape2 dcast
#' @importFrom stats aggregate
#' @export

convert4mapDIA <- function(data, RT = FALSE) {

    mapDIA_columns <- c("ProteinName", "PeptideSequence", "FragmentIon", 
                      "Intensity", "BioReplicate", "Condition", "RT")

    missing_columns <- mapDIA_columns[!(mapDIA_columns %in% colnames(data))]
    if (length(missing_columns) > 0) {
        warning("One or several columns required by mapDIA were not in the data and filled with NAs.
        Missing columns: ",  paste(unlist(missing_columns), collapse = ", "))
        data[, missing_columns] <- NA
    }

    # replace UniMod: to UniMod_
    data$PeptideSequence <- gsub(":", "_", data$PeptideSequence)
    data$FragmentIon <- gsub(":", "_", data$FragmentIon)

    data.wide.test <- dcast(data, ProteinName + PeptideSequence + FragmentIon ~
        Condition + BioReplicate, fun.aggregate = length, value.var = "Intensity")

    if (sum(data.wide.test[, 4:dim(data.wide.test)[2]] > 1) > 0) {
        col.names.repl <- apply(data.wide.test[, 4:dim(data.wide.test)[2]],
            2, function(x) sum(x > 1))
        col.names.repl <- names(col.names.repl[col.names.repl > 0])
        row.names.repl <- apply(data.wide.test[, 4:dim(data.wide.test)[2]],
            1, function(x) sum(x > 1))
        row.names.repl <- data.wide.test[row.names.repl > 0, "FragmentIon"]

        warning("Data contains several intensity values per condition\n\n", "in the following columns: ",
            paste(col.names.repl, collapse = ", "), "\n\n", "and in the following rows: ",
            paste(row.names.repl, collapse = ", "))
    }

    data.wide <- dcast(data, ProteinName + PeptideSequence + FragmentIon ~ Condition +
        BioReplicate, fun.aggregate = mean, value.var = "Intensity")
    if (RT) {
        RTs <- unique(aggregate(data[, c(which(colnames(data) == "RT"))], by = list(data$FragmentIon),
            FUN = mean, na.rm = TRUE))
        colnames(RTs) <- c("FragmentIon", "RT")
        RTs$RT <- RTs$RT/60
        data.wide <- merge(data.wide, RTs, by = "FragmentIon", all.x = TRUE)
        data.wide <- data.wide[, c("ProteinName", "PeptideSequence", "FragmentIon",
            colnames(data.wide)[!(colnames(data.wide) %in% c("ProteinName",
                "PeptideSequence", "FragmentIon", "RT"))], "RT")]
    }
    return(data.wide)
}

### R code from vignette source 'SWATH2stats_vignette.Rnw'

###################################################
### code chunk number 1: SWATH2stats_vignette.Rnw:48-49
###################################################
options(width=70)


###################################################
### code chunk number 2: SWATH2stats_vignette.Rnw:54-57 (eval = FALSE)
###################################################
## if (!require("BiocManager"))
##     install.packages("BiocManager")
## BiocManager::install("SWATH2stats")


###################################################
### code chunk number 3: SWATH2stats_vignette.Rnw:61-62
###################################################
library(SWATH2stats)


###################################################
### code chunk number 4: SWATH2stats_vignette.Rnw:71-76
###################################################
data('OpenSWATH_data', package='SWATH2stats')
data <- OpenSWATH_data

data('Study_design', package='SWATH2stats')
head(Study_design)


###################################################
### code chunk number 5: SWATH2stats_vignette.Rnw:81-92 (eval = FALSE)
###################################################
## # set working directory
## setwd('~/Documents/MyWorkingDirectory/')
## 
## # Define input data file (e.g. OpenSWATH_output_file.txt)
## file.name <- 'OpenSWATH_output_file.txt'
## 
## # File name for annotation file
## annotation.file <- 'Study_design_file.txt'
## 
## # load data
## data <- data.frame(fread(file.name, sep='\t', header=TRUE))


###################################################
### code chunk number 6: SWATH2stats_vignette.Rnw:97-101 (eval = FALSE)
###################################################
## # consult the manual page.
## help(import_data)
## # rename the columns
## data <- import_data(data)


###################################################
### code chunk number 7: SWATH2stats_vignette.Rnw:108-113
###################################################
# reduce number of columns
data <- reduce_OpenSWATH_output(data)

# remove the iRT peptides (or other proteins)
data <- data[grep('iRT', data$ProteinName, invert=TRUE),]


###################################################
### code chunk number 8: SWATH2stats_vignette.Rnw:119-126 (eval = FALSE)
###################################################
## # list number and different Files present
## nlevels(factor(data$filename))
## levels(factor(data$filename))
## 
## # load the study design table from the indicated file
## Study_design <- read.delim2(annotation.file,
##                             dec='.', sep='\t', header=TRUE)


###################################################
### code chunk number 9: SWATH2stats_vignette.Rnw:131-140
###################################################
# annotate data
data.annotated <- sample_annotation(data, Study_design)

head(unique(data$ProteinName))
# OPTIONAL: for human, shorten Protein Name to remove non-unique information
#(sp|Q9GZL7|WDR12_HUMAN --> Q9GZL7)
data$ProteinName <- gsub('sp\\|([[:alnum:]]+)\\|[[:alnum:]]*_HUMAN',
                         '\\1', data$ProteinName)
head(unique(data$ProteinName))


###################################################
### code chunk number 10: SWATH2stats_vignette.Rnw:149-150
###################################################
count_analytes(data.annotated)


###################################################
### code chunk number 11: SWATH2stats_vignette.Rnw:156-163
###################################################
# Plot correlation of intensity
correlation <- plot_correlation_between_samples(data.annotated, column.values = "Intensity")
head(correlation)

# Plot correlation of retention times
correlation <- plot_correlation_between_samples(data.annotated, column.values = "RT")



###################################################
### code chunk number 12: SWATH2stats_vignette.Rnw:169-178
###################################################
# plot variation of transitions for each condition across replicates
variation <- plot_variation(data.annotated)
head(variation[[2]])

# plot variation of summed signal for Peptides for each condition across replicates
variation <- plot_variation(data.annotated,
               Comparison = FullPeptideName + Condition ~ BioReplicate,
               fun.aggregate = sum)



###################################################
### code chunk number 13: SWATH2stats_vignette.Rnw:184-188
###################################################
# plot variation of transitions for each condition within replicates
# compared to total variation
variation <- plot_variation_vs_total(data.annotated)
head(variation[[2]])


###################################################
### code chunk number 14: SWATH2stats_vignette.Rnw:198-201 (eval = FALSE)
###################################################
## protein_matrix <- write_matrix_proteins(data,
##                       filename = "SWATH2stats_overview_matrix_proteinlevel",
##                       rm.decoy = FALSE)


###################################################
### code chunk number 15: SWATH2stats_vignette.Rnw:208-211 (eval = FALSE)
###################################################
## peptide_matrix <- write_matrix_peptides(data,
##                       filename = "SWATH2stats_overview_matrix_peptidelevel",
##                       rm.decoy = FALSE)


###################################################
### code chunk number 16: SWATH2stats_vignette.Rnw:233-234
###################################################
assess_decoy_rate(data)


###################################################
### code chunk number 17: SWATH2stats_vignette.Rnw:239-247
###################################################
# count decoys and targets on assay, peptide and protein level
# and report FDR at a range of m_score cutoffs
assess_fdr_overall(data, FFT = 0.7, output = "pdf_csv", plot = TRUE,
                   filename='assess_fdr_overall_testrun')

# The results can be reported back to R for further calculations
overall_fdr_table <- assess_fdr_overall(data, FFT = 0.7,
                                        output = "Rconsole")


###################################################
### code chunk number 18: SWATH2stats_vignette.Rnw:252-255
###################################################
# create plots from fdr_table
plot(overall_fdr_table, output = "Rconsole",
               filename = "FDR_report_overall")


###################################################
### code chunk number 19: SWATH2stats_vignette.Rnw:260-268
###################################################
# count decoys and targets on assay, peptide and protein level per run
# and report FDR at a range of m_score cutoffs
assess_fdr_byrun(data, FFT = 0.7, output = "pdf_csv", plot = TRUE,
                 filename='assess_fdr_byrun_testrun')

# The results can be reported back to R for further calculations
byrun_fdr_cube <- assess_fdr_byrun(data, FFT = 0.7,
                                   output = "Rconsole")


###################################################
### code chunk number 20: SWATH2stats_vignette.Rnw:273-276
###################################################
# create plots from fdr_table
plot(byrun_fdr_cube, output = "Rconsole",
              filename = "FDR_report_overall")


###################################################
### code chunk number 21: SWATH2stats_vignette.Rnw:284-287
###################################################
# select and return a useful m_score cutoff in order
# to achieve the desired FDR quality for the entire table
mscore4assayfdr(data, FFT = 0.7, fdr_target=0.01)


###################################################
### code chunk number 22: SWATH2stats_vignette.Rnw:292-295
###################################################
# select and return a useful m_score cutoff
# in order to achieve the desired FDR quality for the entire table
mscore4pepfdr(data, FFT = 0.7, fdr_target=0.02)


###################################################
### code chunk number 23: SWATH2stats_vignette.Rnw:302-305
###################################################
# select and return a useful m_score cutoff in order
# to achieve the desired FDR quality for the entire table
mscore4protfdr(data, FFT = 0.7, fdr_target=0.02)


###################################################
### code chunk number 24: SWATH2stats_vignette.Rnw:317-318
###################################################
data.filtered.mscore <- filter_mscore(data.annotated, 0.01)


###################################################
### code chunk number 25: SWATH2stats_vignette.Rnw:323-325
###################################################
data.filtered.mscore <- filter_mscore_freqobs(data.annotated, 0.01, 0.8,
                                              rm.decoy=FALSE)


###################################################
### code chunk number 26: SWATH2stats_vignette.Rnw:330-331 (eval = FALSE)
###################################################
## data.filtered.mscore <- filter_mscore_condition(data.annotated, 0.01, 3)


###################################################
### code chunk number 27: SWATH2stats_vignette.Rnw:338-341
###################################################
data.filtered.fdr <- filter_mscore_fdr(data, FFT=0.7,
                                   overall_protein_fdr_target = 0.03,
                                   upper_overall_peptide_fdr_limit = 0.05)


###################################################
### code chunk number 28: SWATH2stats_vignette.Rnw:347-349
###################################################
data <- filter_proteotypic_peptides(data.filtered.mscore)
data.all <- filter_all_peptides(data.filtered.mscore)


###################################################
### code chunk number 29: SWATH2stats_vignette.Rnw:355-356
###################################################
data.filtered.max <- filter_on_max_peptides(data.filtered.mscore, 5)


###################################################
### code chunk number 30: SWATH2stats_vignette.Rnw:361-362
###################################################
data.filtered.max.min <- filter_on_min_peptides(data.filtered.max, 2)


###################################################
### code chunk number 31: SWATH2stats_vignette.Rnw:374-376
###################################################
data.transition <- disaggregate(data)
head(data.transition)


###################################################
### code chunk number 32: SWATH2stats_vignette.Rnw:379-381 (eval = FALSE)
###################################################
## write.csv(data.transition, file='transition_level_output.csv',
##           row.names=FALSE, quote=FALSE)


###################################################
### code chunk number 33: SWATH2stats_vignette.Rnw:387-388
###################################################
data.python <- convert4pythonscript(data)


###################################################
### code chunk number 34: SWATH2stats_vignette.Rnw:391-392 (eval = FALSE)
###################################################
## write.table(data.python, file="input.tsv", sep="\t", row.names=FALSE, quote=FALSE)


###################################################
### code chunk number 35: SWATH2stats_vignette.Rnw:400-402 (eval = FALSE)
###################################################
## data.transition <- data.frame(fread('output.csv',
##                                     sep=',', header=TRUE))


###################################################
### code chunk number 36: SWATH2stats_vignette.Rnw:409-413 (eval = FALSE)
###################################################
## MSstats.input <- convert4MSstats(data.transition)
## 
## library(MSstats)
## quantData <- dataProcess(MSstats.input)


###################################################
### code chunk number 37: SWATH2stats_vignette.Rnw:420-432 (eval = FALSE)
###################################################
## aLFQ.input <- convert4aLFQ(data.transition)
## 
## library(aLFQ)
## prots <- ProteinInference(aLFQ.input, peptide_method = 'top',
##                           peptide_topx = 3,
##                           peptide_strictness = 'loose',
##                           peptide_summary = 'mean',
##                           transition_topx = 3,
##                           transition_strictness = 'loose',
##                           transition_summary = 'sum',
##                           fasta = NA, model = NA,
##                           combine_precursors = FALSE)


###################################################
### code chunk number 38: SWATH2stats_vignette.Rnw:437-439
###################################################
mapDIA.input <- convert4mapDIA(data.transition, RT =TRUE)
head(mapDIA.input)


###################################################
### code chunk number 39: SWATH2stats_vignette.Rnw:442-444 (eval = FALSE)
###################################################
## write.table(mapDIA.input, file='mapDIA.txt', quote=FALSE,
##           row.names=FALSE, sep='\t')


###################################################
### code chunk number 40: SWATH2stats_vignette.Rnw:449-451
###################################################
PECA.input <- convert4PECA(data)
head(PECA.input)


###################################################
### code chunk number 41: SWATH2stats_vignette.Rnw:456-462 (eval = FALSE)
###################################################
## library(PECA)
## group1 <- c("Hela_Control_1", "Hela_Control_2", "Hela_Control_3")
## group2 <- c("Hela_Treatment_1", "Hela_Treatment_2", "Hela_Treatment_3")
## 
## # PECA_df
## results <- PECA_df(PECA.input, group1, group2, id="ProteinName", test = "rots")


###################################################
### code chunk number 42: SWATH2stats_vignette.Rnw:468-475 (eval = FALSE)
###################################################
## data.annotated.full <- sample_annotation(OpenSWATH_data, Study_design)
## data.annotated.full <- filter_mscore(data.annotated.full,
##                                      mscore4assayfdr(data.annotated.full, 0.01))
## data.annotated.full$decoy <- 0 ### imsbInfer needs the decoy column
## 
## library(imsbInfer)
## specLib <- loadTransitonsMSExperiment(data.annotated.full)

#' Select all proteins that are supported by peptides.
#'
#' This function can be used as opposed to the function 
#' "filter_proteotypic_peptides()". This function counts all proteins (including
#' proteins supported by non proteo-typic (i.e. shared) peptides). 
#' All peptides (incl. non proteotypic peptides are selected. 
#' For the proteins supported by proteotypic peptide the "1/" in front
#' of the identifier is removed to facilitate further data processing. 
#' The protein identifier of shared peptides needs to be separated by a slash "/".
#'
#' @param data  A data frame containing SWATH data.
#' @return  Returns a data frame with the data from both proteotypic and
#'   non-proteotypic peptides.
#' @author Peter Blattmann
#' @examples{
#'  data("OpenSWATH_data", package="SWATH2stats")
#'  data("Study_design", package="SWATH2stats")
#'  data <- sample_annotation(OpenSWATH_data, Study_design)
#'  data.filtered.decoy <- filter_mscore(data, 0.01)
#'  data.all <- filter_all_peptides(data.filtered.decoy)
#'  }
#' @importFrom utils head
#' @export
filter_all_peptides <- function(data) {

    data.proteins <- gsub("^[[:digit:]]\\/", "", data$ProteinName)
    data.proteins <- unlist(strsplit(as.character(data.proteins), "\\/"))
    data.proteins <- unique(data.proteins)

    message("Number of proteins detected: ", length(data.proteins), 
            "\n", "Protein identifiers: ",
        paste(head(data.proteins), collapse = ", "))

    # Delete non-unique features (unique features have '1/' in front of protein name)
    data$ProteinName <- gsub("^1\\/", "", data$ProteinName)

    return(data)
}

utils::globalVariables(c("Var1", "Var2", "value"))

#' Plots the correlation between injections.
#'
#' This function plots the Pearson's and Spearman correlation between
#' samples. If decoys are present these are removed before plotting.
#'
#' @param data Data frame that is produced by the OpenSWATH/pyProphet workflow.
#' @param column_values   Indicates the columns for which the correlation is
#'   assessed. This can be the Intensity or Signal, but also the retention
#'   time.
#' @param comparison  The comparison for assessing the variability. Default is
#'   to assess the variability per transition_group_id over the different
#'   Condition and Replicates. Comparison is performed using the dcast()
#'   function of the reshape2 package.
#' @param fun_aggregate  If for the comparison values have to be aggregated one
#'   needs to provide the function here.
#' @param label  Option to print correlation value in the plot.
#' @param ... Further arguments passed to methods.
#' @return Plots in Rconsole a correlation heatmap and returns the data frame
#'   used to do the plotting.
#' @author Peter Blattmann
#' @examples{
#'  data("OpenSWATH_data", package="SWATH2stats")
#'  data("Study_design", package="SWATH2stats")
#'  data <- sample_annotation(OpenSWATH_data, Study_design)
#'  information <- plot_correlation_between_samples(data)
#' }
#' @importFrom reshape2 dcast melt
#' @importFrom ggplot2 aes geom_tile scale_fill_gradient xlab ylab labs 
#'  geom_text theme theme_bw scale_x_discrete scale_y_discrete
#' @importFrom stats cor
#' @export
plot_correlation_between_samples <- function(data, 
                                             column_values = "Intensity",
                                             comparison = transition_group_id ~ Condition + BioReplicate,
                                             fun_aggregate = NULL, 
                                             label = TRUE, ...) {
    if (sum(colnames(data) == "decoy") == 1) {
        data <- data[data$decoy == 0, ]
    }
    data.c <- dcast(data, comparison, value.var = column_values, fun.aggregate = fun_aggregate)
    dep.vars <- length(all.vars(comparison[[2]]))
    indep.vars <- length(all.vars(comparison[[3]]))
    
    pearson.cor <- cor(data.c[, (dep.vars + 1):dim(data.c)[2]], 
                       use = "pairwise.complete.obs", method = "pearson")
    pearson.cor[lower.tri(pearson.cor)] <- NA
    
    spearman.cor <- cor(data.c[, (dep.vars + 1):dim(data.c)[2]], 
                        use = "pairwise.complete.obs", method = "spearman")
    spearman.cor[upper.tri(spearman.cor, diag = TRUE)] <- NA
    
    pearson.cor <- melt(pearson.cor)
    pearson.cor$method <- "pearson"
    spearman.cor <- melt(spearman.cor)
    spearman.cor$method <- "spearman"
    
    data.plot <- rbind(pearson.cor, spearman.cor)
    data.plot <- data.plot[!is.na(data.plot$value), ]
    
    if (label) {
        p <- (ggplot(data.plot, aes(x = Var2, y = Var1, fill = value)) +
                  geom_tile() +
                  scale_fill_gradient(low = "white", high = "red",
                                      name = "Correlation\n[R or rho]") +
                  xlab("") +
                  ylab("") +
                  labs(title = paste0(column_values, 
                                      " correlation between samples:\nPearson (upper triangle) and Spearman correlation (lower triangle)")) +
                  geom_text(aes_string(label = "round(data.plot$value, digits = 2)")) +
                  scale_x_discrete(expand = c(0, 0)) +
                  scale_y_discrete(limits = rev(levels(data.plot$Var1)),
                                   expand = c(0, 0)) +
                  theme_bw() +
                  theme(plot.title = element_text(hjust = 0.5, vjust = 1),
                        axis.text.x = element_text(angle = 90, hjust = 1)))
    } else {
        p <- (ggplot(data.plot, aes_string(x = "Var2", y = "Var1", fill = "value")) +
                  geom_tile() +
                  scale_fill_gradient(low = "white", high = "red",
                                      name = "Correlation\n[R or rho]") +
                  xlab("") +
                  ylab("") +
                  labs(title = paste(column_values, "correlation between samples:\nPearson (upper triangle) and Spearman correlation (lower triangle)")) +
                  scale_x_discrete(expand = c(0, 0)) +
                  scale_y_discrete(limits = rev(levels(data.plot$Var1)), expand = c(0, 0)) +
                  theme_bw() +
                  theme(plot.title = element_text(hjust = 0.5, vjust = 1),
                        axis.text.x = element_text(angle = 90, hjust = 1)))
    }
    print(p)
    return(data.plot)
}

#' S3 plot function for FDR assessment result arrays 
#'
#' This function creates standard plots from result arrays as produced by
#' e.g. the function assess_fdr_byrun(), visualizig assay, peptide and protein
#' level FDR for each run at m-score cutoffs 1e-2 and 1e-3. Furthermore, Target
#' and Decoy ID numbers are visualized.
#'
#' @param x Array of by-run FDR assessment results as produced e.g. by the
#'  function assess_fdr_byrun() from this package.
#' @param output Choose output type. "pdf_csv" creates the output as files in
#'  the working directory, "Rconsole" triggers delivery of the output to the
#'  console enabling further computation and/or custom plotting / output.
#' @param filename  Basename for output files to be created (if output =
#'  "pdf_csv" has been selected).
#' @param plot_mscore_levels  Define m-score levels to plot the estimated FDR
#'  results.
#' @param ...  Extra arguments passed on to functions inside this.
#' @return Plots in Rconsole or report files.
#' @author Moritz Heusel
#' @examples{
#'  data("OpenSWATH_data", package="SWATH2stats")
#'  data("Study_design", package="SWATH2stats")
#'  data <- sample_annotation(OpenSWATH_data, Study_design)
#'  x <- assess_fdr_byrun(data, FFT=0.7, output="Rconsole", plot=FALSE)
#'  retlist <- plot(x, output="Rconsole", filename="Assess_fdr_byrun_testplot",
#'                           plot_mscore_levels=0.01)
#'  }
#' @importFrom graphics par barplot mtext
#' @importFrom grDevices pdf dev.off
#' @export
plot.fdr_cube <- function(x, 
                          output = "Rconsole", 
                          filename = "FDR_report_byrun",
                          plot_mscore_levels = c(0.01, 0.001), ...){
    for (i in plot_mscore_levels) {
        k.mscore <- which(dimnames(x)[[3]] == i)
        k.mscore.label <- as.numeric(dimnames(x)[[3]][k.mscore])

        # write pdf reports
        run_ids <- unlist(dimnames(x)[2])

        # Save previous par(mfrow) settings and restore upon exit (BioConductor
        # suggestion)
        par.mfrow.old <- par(mfrow = par()$mfrow)
        on.exit(par(par.mfrow.old), add = TRUE)

        if (output == "pdf_csv") {
            pdf(file = paste0(filename, "_", format(k.mscore.label, scientific = TRUE),".pdf"))
            par(mfrow = c(3, 2))
            title <- NULL
        }

        if (output == "Rconsole") {
            title <- paste0("mscore_", format(k.mscore.label, scientific = TRUE))
        }

        # Assay level plots
        barplot(x[seq_len(2), , k.mscore], 
                ylim = c(0, 1.2 * max(x[seq_len(2), , k.mscore], na.rm = TRUE)), 
                ylab = "# of assays", legend = TRUE, 
                args.legend = list(x = "topleft", cex = 0.5), names.arg = run_ids, 
                cex.names = 0.5, las = 2, main = title)
        barplot(x[4, , k.mscore], 
                ylim = c(0, 1.2 * max(x[4, , k.mscore], na.rm = TRUE)),
                ylab = row.names(x)[4], names.arg = run_ids, 
                cex.names = 0.5, las = 2, main = title)
        # Peptide level plots
        barplot(x[5:6, , k.mscore], 
                ylim = c(0, 1.2 * max(x[5:6, , k.mscore], na.rm = TRUE)),
                ylab = "# of peptides", legend = TRUE, 
                args.legend = list(x = "topleft",cex = 0.5), 
                names.arg = run_ids, cex.names = 0.5, las = 2, main = title)
        barplot(x[8, , k.mscore], 
                ylim = c(0, 1.2 * max(x[8, , k.mscore], na.rm = TRUE)),
                ylab = row.names(x)[8], names.arg = run_ids, cex.names = 0.5, las = 2,
                main = title)
        # Protein level plots
        barplot(x[9:10, , k.mscore], 
                ylim = c(0, 2 * max(x[9:10, , k.mscore], na.rm = TRUE)),
                ylab = "# of proteins", legend = TRUE, 
                args.legend = list(x = "topleft", cex = 0.5), 
                names.arg = run_ids, cex.names = 0.5, las = 2, main = title)
        barplot(x[12, , k.mscore], 
                ylim = c(0, 1.2 * max(x[12, , k.mscore], na.rm = TRUE)),
                ylab = row.names(x)[12], 
                names.arg = run_ids, cex.names = 0.5, las = 2,
                main = title)

        if (output == "pdf_csv") {
            par(mfrow = c(1, 1))
            mtext(paste0("FDR by run target/decoy ID report (m_score <= ", 
                        format(k.mscore.label, scientific = TRUE), ")"), 
                  line = 2, adj = 0.5)
            dev.off()
            message(filename, ".pdf report plots written to working folder", "\n")
        }
    }
}

---
title: 'SWATH2stats example script'
output: 
  pdf_document: 
    keep_tex: yes
vignette: >
  %\VignetteIndexEntry{SWATH2stats example script}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---

Example R code showing the usage of the SWATH2stats package. The data processed is the publicly available dataset of *S.pyogenes* (Röst et al. 2014) (http://www.peptideatlas.org/PASS/PASS00289). The results file 'rawOpenSwathResults\_1pcnt\_only.tsv' can be found on PeptideAtlas (ftp://PASS00289@ftp.peptideatlas.org/../Spyogenes/results/).
This is a R Markdown file, showing the result of processing this data. The lines shaded in grey represent the R code executed during this analysis.

The SWATH2stats package can be directly installed from Bioconductor using the commands below (http://bioconductor.org/packages/devel/bioc/html/SWATH2stats.html).


```{r, eval=FALSE}
if (!require("BiocManager"))
    install.packages("BiocManager")
BiocManager::install("SWATH2stats")
```

# Part 1: Loading and annotation

Load the SWATH-MS example data from the package, this is a reduced file in order to limit the file size of the package. 

```{r}
library(SWATH2stats)
library(data.table)
data('Spyogenes', package = 'SWATH2stats')
```

Alternatively the original file downloaded from the Peptide Atlas can be loaded from the working directory.

```{r, eval=FALSE}
data <- data.frame(fread('rawOpenSwathResults_1pcnt_only.tsv', sep='\t', header=TRUE))
```

Extract the study design information from the file names. Alternatively, the study design table can  be provided as an external table.

```{r, tidy=TRUE}
Study_design <- data.frame(Filename = unique(data$align_origfilename))
Study_design$Filename <- gsub('.*strep_align/(.*)_all_peakgroups.*', '\\1', Study_design$Filename)
Study_design$Condition <- gsub('(Strep.*)_Repl.*', '\\1', Study_design$Filename)
Study_design$BioReplicate <- gsub('.*Repl([[:digit:]])_.*', '\\1', Study_design$Filename)
Study_design$Run <- seq_len(nrow(Study_design))
head(Study_design)
```

The SWATH-MS data is annotated using the study design table.
```{r}
data.annotated <- sample_annotation(data, Study_design, column_file = "align_origfilename")
```

Remove the decoy peptides for a subsequent inspection of the data.
```{r}
data.annotated.nodecoy <- subset(data.annotated, decoy==FALSE)
```


\newpage

# Part 2: Analyze correlation, variation and signal
Count the different analytes for the different injections.
```{r}
count_analytes(data.annotated.nodecoy)
```

Plot the correlation of the signal intensity.
```{r, fig.height=2.5, fig.width = 6}
correlation <- plot_correlation_between_samples(data.annotated.nodecoy, column.values = 'Intensity')
```

Plot the correlation of the delta_rt, which is the deviation of the retention time from the expected retention time.
```{r, fig.height=2.5, fig.width = 6}
correlation <- plot_correlation_between_samples(data.annotated.nodecoy, column.values = 'delta_rt')
```
\newpage 


Plot the variation of the signal across replicates.
```{r, fig.height=2.5, fig.width = 5.5}
variation <- plot_variation(data.annotated.nodecoy)
variation[[2]]
```

Plot the total variation versus variation within replicates.
```{r, fig.height=2.5, fig.width = 5.5}
variation_total <- plot_variation_vs_total(data.annotated.nodecoy)
variation_total[[2]]
```

Calculate the summed signal per peptide and protein across samples.
```{r}
peptide_signal <- write_matrix_peptides(data.annotated.nodecoy)
protein_signal <- write_matrix_proteins(data.annotated.nodecoy)
head(protein_signal)
```
\newpage


# Part 3: FDR estimation
Estimate the overall FDR across runs using a target decoy strategy.
```{r, fig.height = 3.5}
par(mfrow = c(1, 3))
fdr_target_decoy <- assess_fdr_overall(data.annotated, n_range = 10, 
                                       FFT = 0.25, output = 'Rconsole')

```
According to this FDR estimation one would need to filter the data with a lower mscore threshold to reach an overall protein FDR of 5%. 
```{r}
mscore4protfdr(data, FFT = 0.25, fdr_target = 0.05)
```

# Part 4: Filtering
Filter data for values that pass the 0.001 mscore criteria in at least two replicates of one condition.
```{r}
data.filtered <- filter_mscore_condition(data.annotated, 0.001, n_replica = 2)
```

Select only the 10 peptides showing strongest signal per protein.
```{r}
data.filtered2 <- filter_on_max_peptides(data.filtered, n_peptides = 10)
```
\newpage 


Filter for proteins that are supported by at least two peptides.
```{r}
data.filtered3 <- filter_on_min_peptides(data.filtered2, n_peptides = 2)
```


# Part 5: Conversion
Convert the data into a transition-level format (one row per transition measured). 

```{r}
data.transition <- disaggregate(data.filtered3)
```

Convert the data into the format required by MSstats.
```{r}
MSstats.input <- convert4MSstats(data.transition)
head(MSstats.input)
```

Convert the data into the format required by mapDIA.
```{r}
mapDIA.input <- convert4mapDIA(data.transition)
head(mapDIA.input)
```

Convert the data into the format required by aLFQ.
```{r}
aLFQ.input <- convert4aLFQ(data.transition)
head(aLFQ.input)
```

Session info on the R version and packages used.
```{r}
sessionInfo()
```

%\VignetteIndexEntry{SWATH2stats package Vignette}
%\VignettePackage{SWATH2stats}
\documentclass[a4paper]{article}
\usepackage{authblk}
\makeatletter
\renewcommand\paragraph{\@startsection{paragraph}{4}{\z@}%
            {-2.5ex\@plus -1ex \@minus -.25ex}%
            {1.25ex \@plus .25ex}%
            {\normalfont\normalsize\bfseries}}
\makeatother
\setcounter{secnumdepth}{4} % how many sectioning levels to assign numbers to
\setcounter{tocdepth}{4}    % how many sectioning levels to show in ToC

\title{SWATH2stats}
\author{Peter Blattmann}
\author{Moritz Heusel}
\author{Ruedi Aebersold}
\affil{Institute of Molecular Systems Biology, Department of Biology, ETH Zurich, Switzerland}
\begin{document}

\maketitle


\SweaveOpts{concordance=TRUE}
\DefineVerbatimEnvironment{Sinput}{Verbatim} {fontsize=\small}
\DefineVerbatimEnvironment{Soutput}{Verbatim}{fontsize=\small}
\DefineVerbatimEnvironment{Scode}{Verbatim}{fontsize=\small}

This vignette describes how the different functions from the SWATH2stats package can be applied. The functions from the SWATH2stats package are intended to be used on SWATH data that has been generated by the OpenSWATH pipeline. The SWATH2stats package provides functions to annotate such SWATH data with experimental meta-data, perform initial data analysis, perform a false-discovery rate (FDR) estimation, perform filtering, and to convert the SWATH data into a format readable by downstream statistical and quantification software tools such as MSstats, aLFQ, mapDIA or imsbInfer. The SWATH2stats package thus represents a link between the OpenSWATH pipeline and the downstream analysis packages MSstats, aLFQ, mapDIA, or imsbInfer. The SWATH2stats package was programmed and intended for use by researchers in proteomics working with SWATH data without extensive programming skills, but with basic R knowledge.

\tableofcontents
\section{Introduction}
\subsection{SWATH-MS data analysis via open source tools}
SWATH-MS as an implementation of data-independent acquisition (DIA) mass spectrometry is an emerging proteomic approach that allows systematic quantification of peptides in complex samples (Gillet et al. 2012, Venable et al. 2004). The acquired mass spectra can be queried for the presence and quantity of peptide analytes using the open-source OpenSWATH pipeline. The OpenSWATH pipeline consists of the OpenSWATH software (Roest et al. 2014) coupled to statistical validation using the mProphet algorithm (Reiter et al. 2011), or its re-implementation pyProphet (Teleman et al. 2015). OpenSWATH extracts ion chromatograms of both the peptide precursor and the fragment ions and quantifies peak groups. It then generates scores for how well a given candidate peak group corresponds to an analyte from a spectral or assay library (Roest et al. 2014). mProphet uses a machine learning algorithm to identify an optimal linear combination of these scores (d-score) to discriminate targets from decoys. In addition, it fits a function to the distribution of the d-scores for the decoy peptides, that is used as the null distribution. This null distribution is then used to calculate a q-value/m-score of each peakgroup (Storrey et al. 2003, Reiter et al. 2011). Hence, filtering the results with an m-score of 0.01 results in an FDR of 1\% of the target assays within this run.

\subsection{The usage of SWATH2stats in the open-source SWATH-MS data analysis workflow}
This package creates a link between the OpenSWATH/mProphet .tsv output table and popular downstream tools for statistical and advanced data analysis. With very large assay libraries (Rosenberger et al. 2014) the SWATH results can become too large to analyse and process via tools such as Microsoft Excel. Therefore this package offers functionality to annotate the data with the study design (such as condition, biological and unique MS run id), perform initial data analysis, and offers substantial filtering capabilities. The data can be filtered based on frequency of observation among the samples, number of sibling peptides of a protein entry or directly on the FDR as estimated by the mProphet model (m-score, equivalent to q-value; for details see previous section or Reiter et al. 2011). Furthermore the package features estimation of global false discovery rates according to the target-decoy rationale (Elias and Gygi 2008, Kaell et al. 2008). The last step is the conversion to formats that can be read directly by the downstream analysis tools MSstats (Choi et al. 2014), mapDIA (Teo et al. unpublished), aLFQ (Rosenberger et al. 2014),  and imsbInfer (Wolski et al. unpublished).

\subsection{Implementation of a target-decoy strategy to estimate false target discovery rates (FDR)}
Mass-spectrometry-based proteomic experiments produce large amounts of data that require statistical validation. In the SWATH2stats package a target-decoy strategy was implemented to estimate the FDR (Elias and Gygi, 2007). The target-decoy strategy relies on the assumption that the decoys have the same characteristics (distribution of their scores) as the false targets. The FDR among the targets is estimated as the ratio of decoy peptides passing a certain score threshold divided by the total number of targets passing the same score threshold (Choi and Nesvizhskii, 2008). The usage of the target-decoy strategy for SWATH data and to estimate peptide and protein-level FDR has not been extensively tested yet. The target-decoy strategy has been tested to estimate protein-level FDR in DDA data and has been shown to result in more conservative FDR estimates compared to a hypergeometric model-based approach (Reiter et al. 2009). A target-decoy approach was implemented in SWATH2stats because it allows i.) estimation of an FDR over multiple runs and ii.) allows to directly assess the selectivity of a given filter for likely true (target) over false (decoy) data points.\\
In contrast to the naive target-decoy approach counting the number of decoys, a correction factor can be supplied to many FDR estimation functions in the SWATH2stats package. For example, a correction needs to be applied to correct for the fraction of false targets (FFT). Similar correction factors have been used to adjust FDR estimation in DDA data (PIT: Kaell et al. 2008, p(-): Keller et al. 2002). In the functions, the FFT defaults to 1 to perform a naive target-decoy counting strategy without FFT correction, which will result in an overestimation of the FDR. For a more accurate estimation of the FDR, a FFT correction factor can be provided that corrects for the ratio of false targets to decoys. The number of decoys counted is multiplied with the FFT correction factor. The rationale is that for example if 50\% of the samples are true targets, the number of true negative targets that are modeled by the decoy distribution is around 50\% lower than the decoy distribution. Therefore 2 decoy hits passing a certain m-score threshold suggest only one false positive datapoint passing the same threshold. The ratio of true negative (false) targets compared to all targets (FFT) can for example be obtained from the mProphet model statistics (Injection\_name]\_full\_stat.csv (column 1 line 2 corresponding to the maximal q-value). \\
Alternatively, the FFT can conservatively be approximated by the fraction of assays in the library that do not pass an m-score threshold of e.g. 0.01 (corresponding to ~ 1 \% model FDR). For example, acquiring a full cell lysate and searching the data using the combined assay library (200k assays, Rosenberger et al. 2014), 50k assays are typically identified with m-score <= 0.01. Hence a FFT of 0.75 can be estimated. If a full lysate is searched by a sample-specific assay library (e.g. 70k assays) and 40k assays were identified with m-score <=0.01, a FFT of 0.57 can be estimated.\\



\section{Loading and annotating the data}
\subsection{Installing SWATH2stats}
<<echo=false>>=
options(width=70)
@

To install the SWATH2stats package the following commands can be executed within R (after package has been accepted to Bioconductor).

<<eval=FALSE>>=
if (!require("BiocManager"))
    install.packages("BiocManager")
BiocManager::install("SWATH2stats")
@

The SWATH2stats package can now be loaded.
<<>>=
library(SWATH2stats)
@

\subsection{Loading the data}

The example data, that is included in the package, consists of a reduced OpenSWATH output file generated from Hela cells. To avoid making the file of the SWATH2stats package too large, only a fraction of a typical SWATH data table is included as example data. The example data contains data for 9 proteins, 5 decoy-proteins and a set of peptides for retention time calibration (labelled as iRT\_protein). In total the data contains 284 peptides for which quantitative data has been extracted from 6 different samples measured on an ABSciex TripleTOF 5600 mass spectrometer and analyzed with the OpenSWATH + pyProphet workflow (Roest et al. 2014, Teleman et al. 2015). These 6 samples consist of biological triplicates of Hela cells grown under control condition and Hela cells that have been perturbed by inhibiting cholesterol synthesis. \\
The experimental design is described in a table called Study\_design that is included in the package. This file that contains the study design information needs to be a table with the following columns:  Filename, Condition, BioReplicate, Run (see below). For correct assignment of identifiers into the Run, BioReplicate and Condition column for MSstats, please consult their manual. The values in the column \texttt{Filename} have to be unique for every injection file and will be matched to the OpenSWATH output in the column \texttt{align\_origfilename} (caution: this matching is case sensitive).\\
The example SWATH data and the study design table can be loaded from the package with the function \texttt{data()}.

<<>>=
data('OpenSWATH_data', package='SWATH2stats')
data <- OpenSWATH_data

data('Study_design', package='SWATH2stats')
head(Study_design)
@

Alternatively, the data can be loaded from your working directory. In the code below you see an example how to load the required files from your working directory (code is not executed here but just as an example). As input data for SWATH2stats it is recommended to use the quantitative data matrix after OpenSWATH, pyProphet, and TRIC analysis (see www.openswath.org).

<<eval=FALSE>>=
# set working directory
setwd('~/Documents/MyWorkingDirectory/')

# Define input data file (e.g. OpenSWATH_output_file.txt)
file.name <- 'OpenSWATH_output_file.txt'

# File name for annotation file
annotation.file <- 'Study_design_file.txt'

# load data
data <- data.frame(fread(file.name, sep='\t', header=TRUE))
@

If the file is in a different format the column names have to be renamed accordingly. For this the function \texttt{import\_data} can perform this. For the requirements for each column please consult the manual page.

<<eval=FALSE>>=
# consult the manual page.
help(import_data)
# rename the columns
data <- import_data(data)
@


The function \texttt{reduce\_openSWATH\_output} can be executed to reduce the number of columns from the OpenSWATH result table. This function reduces the number of columns to the ones necessary for MSstats, mapDIA, aLFQ. However for other packages such as imsbInfer all the columns need to be kept and this function should be omitted.
The next command shows how specific proteins or peptides can be removed from the data. As an example the iRT peptides (peptides for retention time calibration) were removed.

<<>>=
# reduce number of columns
data <- reduce_OpenSWATH_output(data)

# remove the iRT peptides (or other proteins)
data <- data[grep('iRT', data$ProteinName, invert=TRUE),]
@

\subsection{Annotating the data}
With the first two commands the number of files in the OpenSWATH data and the names of these files can be printed. This can be helpful to generate the study design table (The script can be executed until here and then the annotation file generated with a text editor). See above for a description of the exact format and column names required for the study design table.

<<eval=FALSE>>=
# list number and different Files present
nlevels(factor(data$filename))
levels(factor(data$filename))

# load the study design table from the indicated file
Study_design <- read.delim2(annotation.file,
                            dec='.', sep='\t', header=TRUE)
@

With the function \texttt{sample\_annotation} the data is annotated with the meta-data contained in the study design table. The next commands can be used to shorten the protein names and remove repetitive and non-unique parts of the Protein name as shown by the example removing some parts of the identifier keeping only the unique SwissProt accession identifier (example see below).

<<>>=
# annotate data
data.annotated <- sample_annotation(data, Study_design)

head(unique(data$ProteinName))
# OPTIONAL: for human, shorten Protein Name to remove non-unique information
#(sp|Q9GZL7|WDR12_HUMAN --> Q9GZL7)
data$ProteinName <- gsub('sp\\|([[:alnum:]]+)\\|[[:alnum:]]*_HUMAN',
                         '\\1', data$ProteinName)
head(unique(data$ProteinName))
@

\section{Analyze data}
In order to analyze the data we provide different functions to assess the variation or correlation between samples or to calculated the summed signal per peptide and protein. This can be used to quickly assess the overall similarity of the injections or see what the signal for a peptide or protein of interest is.

\subsection{Count analytes}
With the function \texttt{count\_analytes} the number of transitions, peptides and proteins can be counted across the different injections. This can be helpful for assessing if a certain injection produced considerably less identifications and what the mean number of identified transitions, peptides or proteins per sample is.

<<>>=
count_analytes(data.annotated)
@

\subsection{Plot correlation between samples}
With the function \texttt{plot\_correlation\_between\_samples} the Pearson and Spearman correlation is calculated between the different injections and plotted in a heatmap. This can be used to spot injections that show very different signal or also retention times.

<<>>=
# Plot correlation of intensity
correlation <- plot_correlation_between_samples(data.annotated, column.values = "Intensity")
head(correlation)

# Plot correlation of retention times
correlation <- plot_correlation_between_samples(data.annotated, column.values = "RT")

@

\subsection{Plot variation}
With the function \texttt{plot\_variation} the coefficient of variation of the signal for the different transitions per condition across replicates is plotted. The coefficient of variation is calculated as the standard deviation divided by the mean of the signal. In order to do different comparisons, the optional parameters can be altered from the default values. For example the coefficient of variation of the summed signal for each peptide can be plotted as shown below. The function uses the cast function from the reshape2 package and the comparison needs to specified accordingly.

<<>>=
# plot variation of transitions for each condition across replicates
variation <- plot_variation(data.annotated)
head(variation[[2]])

# plot variation of summed signal for Peptides for each condition across replicates
variation <- plot_variation(data.annotated,
               Comparison = FullPeptideName + Condition ~ BioReplicate,
               fun.aggregate = sum)

@

\subsection{Plot variation within replicates versus total variation}
With the function \texttt{plot\_variation\_vs\_total} the coefficient of variation of the signal within replicates can be compared to the variation across all samples. This can serve as an assessment if the variation within technical or biological replicates is indeed smaller than the overall variation. Also for this function the signal for which the variation is plotted and the comparison can be changed by altering the default input options.

<<>>=
# plot variation of transitions for each condition within replicates
# compared to total variation
variation <- plot_variation_vs_total(data.annotated)
head(variation[[2]])
@


\subsection{Results on protein level}
SWATH2stats can write a protein-level summary matrix showing the summed signals of protein (unique \texttt{ProteinName} identifiers) over the MS runs (unique \texttt{run\_id}) using the function \texttt{write\_matrix\_proteins}.
It calculates the sum of all transition intensities per assay, all charge states per peptide, and all peptides for the different protein groups. Note that this function does not select consistently quantified peptides, or a certain number of highest intense peptides, and therefore the summed signal should be used with caution as a measure of protein abundance or to compare protein abundance between runs. For other quantitative protein inference strategies, the R package aLFQ can be used (Rosenberger et al. 2014, see below). For testing differential expression we recommend the downstream tools MSstats and mapDIA (Choi et al. 2014, Teo et al. 2015).\\

Writing the overview matrix of summed intensities per protein entry per MS run:

<<eval=FALSE>>=
protein_matrix <- write_matrix_proteins(data,
                      filename = "SWATH2stats_overview_matrix_proteinlevel",
                      rm.decoy = FALSE)
@

\subsection{Results on peptide level}
SWATH2stats can also write a peptide-level summary matrix showing the summed signals of peptide (unique \texttt{FullPeptideName} identifiers) over the MS runs (unique \texttt{run\_id}) using the function \texttt{write\_matrix\_peptides}.
It calculates the sum of all transition intensities per assay and all charge states per peptide.

<<eval=FALSE>>=
peptide_matrix <- write_matrix_peptides(data,
                      filename = "SWATH2stats_overview_matrix_peptidelevel",
                      rm.decoy = FALSE)
@




\section{FDR estimation}
Mass-spectrometry-based proteomic experiments produce large amounts of data, requiring statistical validation of the obtained results. Large multi-run proteomics studies are prone to the accumulation of false positive identifications and the statistical significance scores must therefore be normalized accordingly(Benjamini and Hochberg, 1995).\\
This chapter describes first the functionality of SWATH2stats to estimate and visualize the global false discovery rate in OpenSWATH/mProphet result tables and second the functionality to obtain m-score thresholds (peak group level mProphet-estimated FDR quality) to control FDR on a global level.\\
\\
Assays are identified by unique identifiers in the column \texttt{transition\_group\_id} of the SWATH data table, peptides by unique identifiers in the column \texttt{\texttt{FullPeptideName}} and protein(group)s by unique identifiers in the column \texttt{ProteinName}. Different MS injections (also termed runs) are identified based on a unique entry in the column run\_id.


\subsection{FDR: Overview and visualization}
SWATH2stats supplies three functions to assess and visualize the false discovery rate in multi-run SWATH data. These functions are useful to get an overview on the relationship between false discovery rate and m-score thresholds. A suitable m-score threshold can subsequently be used to filter the data with the filtering functions described in the next chapter.\\
The FDR within the results passing a given score cutoff is evaluated as explained in the introduction:\\
\\
FDR = (number of decoys * FFT)/(number of targets)\\

Application of the decoy-counting-based FDR assessment functions in interplay with the meta-data filters can help the researcher in selecting an efficient strategy to establish highest possible data quality for downstream analyses. By counting the decoys before and after application of a filter, the selectivity of a given filter for likely true (target) over false (decoy) data can be estimated.\\
With a first basic function \texttt{assess\_decoy\_rate} the overall number of decoy peptides can be counted in the data:

<<>>=
assess_decoy_rate(data)
@

The function \texttt{assess\_fdr\_overall} creates a global assessment of decoy rates (and estimated FDR) on assay, peptide and protein level. Results are reported by default as .csv table and visualized in a .pdf report. Setting the output option to "Rconsole" reports the results back to R. Included in the pdf report are plots showing the estimated global FDR in relation to the m-score threshold. Because false-positive hits accumulate over different runs, the false discovery rate estimated by this function will be higher than if assessed within each run individually.

<<>>=
# count decoys and targets on assay, peptide and protein level
# and report FDR at a range of m_score cutoffs
assess_fdr_overall(data, FFT = 0.7, output = "pdf_csv", plot = TRUE,
                   filename='assess_fdr_overall_testrun')

# The results can be reported back to R for further calculations
overall_fdr_table <- assess_fdr_overall(data, FFT = 0.7,
                                        output = "Rconsole")
@

The function \texttt{plot.fdr\_table} allows to create the report plots from this overall fdr table.

<<>>=
# create plots from fdr_table
plot(overall_fdr_table, output = "Rconsole",
               filename = "FDR_report_overall")
@

The function \texttt{assess\_fdr\_byrun} investigates the decoy rate or FDR in individual runs and by default reports the results in a .csv table and .pdf file. Setting the output option to "Rconsole" reports back the results to R. This function is used if the FDR for different injections should be estimated separately.

<<>>=
# count decoys and targets on assay, peptide and protein level per run
# and report FDR at a range of m_score cutoffs
assess_fdr_byrun(data, FFT = 0.7, output = "pdf_csv", plot = TRUE,
                 filename='assess_fdr_byrun_testrun')

# The results can be reported back to R for further calculations
byrun_fdr_cube <- assess_fdr_byrun(data, FFT = 0.7,
                                   output = "Rconsole")
@

The function \texttt{plot.\_fdr\_cube} allows to create the report plots from this by-run fdr cube.

<<>>=
# create plots from fdr_table
plot(byrun_fdr_cube, output = "Rconsole",
              filename = "FDR_report_overall")
@

\subsection{Identification of useful m-score cutoffs to satisfy desired FDR criteria}

SWATH2stats supplies three functions for the identification of useful m-score cutoffs to satisfy FDR criteria on assay, peptide and protein level over many different runs. These functions return an m-score value, which can be used to filter the data of these different runs in order to obtain a desired overall FDR. The following functions report an m-score cutoff to achieve a strict global FDR target.\\
The function \texttt{mscore4assayfdr} reports an m-score cutoff to achieve a desired overall (global) assay FDR:

<<>>=
# select and return a useful m_score cutoff in order
# to achieve the desired FDR quality for the entire table
mscore4assayfdr(data, FFT = 0.7, fdr_target=0.01)
@

The function \texttt{mscore4pepfdr} reports an m-score cutoff to achieve a desired overall (global) peptide FDR:

<<>>=
# select and return a useful m_score cutoff
# in order to achieve the desired FDR quality for the entire table
mscore4pepfdr(data, FFT = 0.7, fdr_target=0.02)
@

The function \texttt{mscore4protfdr} reports an m-score cutoff to achieve a desired overall (global) protein FDR. Protein FDR control on peak group quality level is a very strict filter and should be handled with caution.
Alternatively, a function \texttt{filter\_mscore\_fdr} is described below applying a two-tiered filtering approach.
\\

<<>>=
# select and return a useful m_score cutoff in order
# to achieve the desired FDR quality for the entire table
mscore4protfdr(data, FFT = 0.7, fdr_target=0.02)
@

\section{Filtering the data}
In this chapter the SWATH data is filtered based on the study design or desired global FDR criteria to be achieved.
By setting the option rm.decoy=FALSE, the decoy peptides can be kept in the data in order to evaluate the selectivity of a given filter for likely true (target) over false (decoy) data by decoy counting with the functions described in the previous chapter.

Before converting the data for statistical analysis the rm.decoy option is set to 'TRUE' in order to remove any decoy peptides and proteins from the data.\\

\subsection{Filter on m-score}
The function \texttt{filter\_mscore} removes all measured peak groups that are above a certain m-score value. The number of rows removed by the function is indicated.

<<>>=
data.filtered.mscore <- filter_mscore(data.annotated, 0.01)
@

The function \texttt{filter\_mscore\_freqobs} takes into account how many times in the different injection runs a peak group has been confidently (as defined by the m-score threshold) identified. This is useful in large data of many different replicates. For example the data for a certain precursor that has been confidently identified in most of the replicates but does not pass the threshold in one replicate still should be kept for statistical analysis. The function \texttt{filter\_mscore\_freqobs} can be used to filter for precursors that were observed with a certain m-score threshold and frequency across the samples. In the following example, precursors passing an m-score threshold of 0.01 in 80 \% of the replicates are selected. The option rm.decoy is set to FALSE to keep the decoys for subsequent FDR assessment.

<<>>=
data.filtered.mscore <- filter_mscore_freqobs(data.annotated, 0.01, 0.8,
                                              rm.decoy=FALSE)
@

The function \texttt{filter\_mscore\_condition} selects only precursors that have passed a certain m-score threshold in a minimum number of replicates for the same condition (as defined by the study design table). In contrast to the previous function, this selects precursors that are confidently identified a certain number of times within a condition as opposed to being identified a certain number of times across all samples.

<<eval=FALSE>>=
data.filtered.mscore <- filter_mscore_condition(data.annotated, 0.01, 3)
@


In order to reach a compromise between a very stringent m-score filter controlling the global protein FDR, and keeping valid peptide quantifications in the data, we introduce here a two-tiered filtering approach with the function \texttt{filter\_mscore\_fdr}. This uses a similar approach as implemented for extracting quantitative data from multi-run DDA data sets (Fermin et al. 2011). In the first step, an m-score cutoff is identified to reach a desired protein-level FDR. All proteins for which one peakgroup passes this strict m-score cutoff criterion are collected in a protein master list. The original data is then filtered i.) for the proteins present in the master list and ii.) filtered for all peptide quantifications passing an m-score cutoff to achieve a desired global peptide-level FDR. Note that the m-score cutoff to filter the protein list will typically be more stringent than the second m-score cutoff to filter the peptides.


<<>>=
data.filtered.fdr <- filter_mscore_fdr(data, FFT=0.7,
                                   overall_protein_fdr_target = 0.03,
                                   upper_overall_peptide_fdr_limit = 0.05)
@

\subsection{Filter on proteotypic peptides}
The function \texttt{filter\_proteotypic\_peptides} selects only data that is based on proteotypic peptides (peptides only contained in one protein and marked by "1/" in the beginning of the protein identifier). These functions also remove the '1' in front of the protein identifier from proteotypic peptides.

<<>>=
data <- filter_proteotypic_peptides(data.filtered.mscore)
data.all <- filter_all_peptides(data.filtered.mscore)
@

\subsection{Filter on number of peptides per protein}
With the function \texttt{filter\_on\_max\_peptides} the peptides showing the strongest signal over the entire table can be selected (top n approach). Removing the lower intense peptides for a protein can make the statistical analysis faster or result in more accurate quantification of proteins under the assumption that quantification of more intense peptides is more robust.

<<>>=
data.filtered.max <- filter_on_max_peptides(data.filtered.mscore, 5)
@

Conversely maybe only data for proteins with a minimum number of supporting peptides should be selected. With the function \texttt{filter\_on\_min\_peptides} only the proteins for which at least a certain number of peptides have been measured are selected. This filter can also be powerful to remove false positive hits from the data as these are enriched in the fraction of single hits. FDR assessment based on decoy counting may still be valid after such filtering (Reiter et al. 2009).

<<>>=
data.filtered.max.min <- filter_on_min_peptides(data.filtered.max, 2)
@



\section{Conversion of data for other tools}
Dedicated programs or tools exist to asssess the statistical significance of a regulated protein or estimate the absolute quantity thereof. A selection of such tools are listed in Section \ref{sec:software_tools}. To facilitate the analyses of SWATH data with these tools, SWATH2stats contains several functions that convert the data into the required format. 

\subsection{Convert results to transition-level format}
\subsubsection{Conversion within R}
Some tools work based on transition-level data. With the function \texttt{disaggregate} the SWATH data is changed from a table where one row corresponds to one quantified peak group to a table where one row corresponds to one measured transition.

<<>>=
data.transition <- disaggregate(data)
head(data.transition)
@

<<eval=FALSE>>=
write.csv(data.transition, file='transition_level_output.csv',
          row.names=FALSE, quote=FALSE)
@

\subsubsection{Conversion using a python script}
For very large SWATH data it is faster to use a custom-made python script to transform the data from a peptide-level format to a transition-level format. With the function \texttt{convert4pythonscript} the necessary columns are selected and the nomenclature for modified peptides is changed. Subsequently the data is written to disk.

<<>>=
data.python <- convert4pythonscript(data)
@

<<eval=FALSE>>=
write.table(data.python, file="input.tsv", sep="\t", row.names=FALSE, quote=FALSE)
@

The .tsv table can be transformed into a transition level table using a python script (as example featurealigner2msstats\_withRT.py from msproteomicstools which is available in the scripts folder of the package).

\texttt{python ./featurealigner2msstats.py input.csv output.csv}

Afterwards the generated .csv table is loaded again into R.
<<eval=FALSE>>=
data.transition <- data.frame(fread('output.csv',
                                    sep=',', header=TRUE))
@


\subsection{MSstats}
In order to use the data in the R Bioconductor package \texttt{MSstats} (Choi et al. 2014), the transition-level data needs to be converted using the function \texttt{convert4MSstats}. Afterwards the data can directly be processed using the \texttt{MSstats} package as shown here by application of the function \texttt{dataProcess} from the MSstats package.

<<eval=FALSE>>=
MSstats.input <- convert4MSstats(data.transition)

library(MSstats)
quantData <- dataProcess(MSstats.input)
@

\subsection{aLFQ}
The package \texttt{aLFQ} (Rosenberger et al. 2014) can read the original OpenSWATH output. Alternatively the  \texttt{aLFQ} package can also be applied to the filtered and annotated data from the  \texttt{SWATH2stats} package.
To convert the data after filtering to the format for aLFQ, the function \texttt{convert4aLFQ} is applied to the transition-level data.

<<eval=FALSE>>=
aLFQ.input <- convert4aLFQ(data.transition)

library(aLFQ)
prots <- ProteinInference(aLFQ.input, peptide_method = 'top',
                          peptide_topx = 3,
                          peptide_strictness = 'loose',
                          peptide_summary = 'mean',
                          transition_topx = 3,
                          transition_strictness = 'loose',
                          transition_summary = 'sum',
                          fasta = NA, model = NA,
                          combine_precursors = FALSE)
@

\subsection{mapDIA}
In order to convert the data into the  format for the mapDIA program (Teo et al. 2015), the function \texttt{convert4mapDIA} is used. Technical replicates included in the data are not taken into account by mapDIA. Therefore the function \texttt{convert4mapDIA} averages the SWATH data from technical replicates if such exist. 
<<>>=
mapDIA.input <- convert4mapDIA(data.transition, RT =TRUE)
head(mapDIA.input)
@

<<eval=FALSE>>=
write.table(mapDIA.input, file='mapDIA.txt', quote=FALSE,
          row.names=FALSE, sep='\t')
@

\subsection{PECA}
In order to convert the data into the  format for the PECA tool (Suomi et al. 2015), the function \texttt{convert4PECA} is used. If both biological and technical replicates exist in the data, technical replicates are not taken into account by PECA. Therefore the function \texttt{convert4PECA} averages the SWATH data from technical replicates. As the PECA tool does not use transition-level data, the \texttt{convert4PECA} function is applied on the peptide-level data. 
<<>>=
PECA.input <- convert4PECA(data)
head(PECA.input)
@

For DIA data, the reproducibility-optimized test statistic (ROTS) is the suggested statistic to be used within PECA (Suomi et al. 2017).

<<eval=FALSE>>=
library(PECA)
group1 <- c("Hela_Control_1", "Hela_Control_2", "Hela_Control_3")
group2 <- c("Hela_Treatment_1", "Hela_Treatment_2", "Hela_Treatment_3")

# PECA_df
results <- PECA_df(PECA.input, group1, group2, id="ProteinName", test = "rots")
@

\subsection{imsbInfer}
The package imsbInfer needs all the columns from the OpenSWATH output, therefore the function \texttt{reduce\_OpenSWATH\_output} needs to be omitted in the workflow (see above). If the package imsbInfer should be used after SWATH2stats, a decoy column needs to be added as exemplified below if it has been removed by the filtering functions.

<<eval=FALSE>>=
data.annotated.full <- sample_annotation(OpenSWATH_data, Study_design)
data.annotated.full <- filter_mscore(data.annotated.full,
                                     mscore4assayfdr(data.annotated.full, 0.01))
data.annotated.full$decoy <- 0 ### imsbInfer needs the decoy column

library(imsbInfer)
specLib <- loadTransitonsMSExperiment(data.annotated.full)
@

\section{Acknowledgments}
We want to acknowledge Koh  Ching Chiek, and Dr. Olga Schubert for help in testing the usability of this package and helpful comments to implementation and description. We want to acknowledge George Rosenberger for discussion and advice to bioinformatic questions and the OpenSWATH + pyProphet workflow. We acknowledge Tomi Suomi for assistance in how to best interface SWATH2stats to the PECA package. 

\section{Software and tools}
\label{sec:software_tools}
\texttt{OpenSWATH}: http://www.openswath.org\\
\texttt{MSstats}: MSstats is available on Bioconductor (www.bioconductor.org) or on www.mssstats.org\\
\texttt{aLFQ}: aLFQ is available on CRAN (https://cran.r-project.org)\\
\texttt{mapDIA}: mapDIA is available on Sourceforge (http://sourceforge.net/projects/mapdia/)\\
\texttt{PECA}: PECA is available on Bioconductor (www.bioconductor.org)\\
\texttt{imsbInfer}: imsbInfer is available on Github (https://github.com/wolski/imsbInfer)\\
\texttt{msproteomicstools}: https://github.com/msproteomicstools\\

\section{References}
Benjamini Y., and Hochberg Y. (1995). Controlling the False Discovery Rate: a Practical and Powerful Approach to Multiple Testing. J. R. Statist. Soc. B, 57(1), 289-300.\\
\\
Choi H., and Nesvizhskii A.I. (2008). False discovery rates and related statistical concepts in mass spectrometry-based proteomics. Journal of Proteome Research, 7(1), 47-50.\\
\\
Choi M., et al. (2014). MSstats: an R package for statistical analysis of quantitative mass spectrometry-based proteomic experiments. Bioinformatics 30(17): 2524-2526.\\
 \\
Elias J.E., and Gygi S.P. (2007). Target-decoy search strategy for increased confidence in large-scale protein identifications by mass spectrometry. Nature Methods, 4(3), 207-214.\\
\\
Fermin D. et al. (2011). Abacus: A computational tool for extracting and pre-processing spectral count data for label-free quantitative proteomic analysis. Proteomics, 11(7), 1340-1345.\\
\\
Gillet, L., et al. (2012). Targeted data extraction of the MS/MS spectra generated by data-independent acquisition: a new concept for consistent and accurate proteome analysis. Mol Cell Proteomics 11(6).\\
 \\
Kaell L. et al. (2008). Assigning significance to peptides identified by tandem mass spectrometry using decoy databases. Journal of Proteome Research, 7(1), 29-34.\\
\\
Nesvizhskii, A. I. (2010). A survey of computational methods and error rate estimation procedures for peptide and protein identification in shotgun proteomics. Journal of Proteomics, 73(11), 2092-123.\\
\\
Reiter L. et al. (2009). Protein identification false discovery rates for very large proteomics data sets generated by tandem mass spectrometry. Molecular and Cellular Proteomics : MCP, 8(11), 2405-17.\\
\\
Reiter L. et al. (2011). mProphet: automated data processing and statistical validation for large-scale SRM experiments. Nature Methods, 8(5), 430-5.\\
\\
Rosenberger G., et al. (2014). A repository of assays to quantify 10,000 human proteins by SWATH-MS.' Sci Data 1: 140031.\\
 \\
Rosenberger G., et al. (2014). aLFQ: an R-package for estimating absolute protein quantities from label-free LC-MS/MS proteomics data. Bioinformatics 30(17): 2511-2513.\\
 \\
Rost H.L., et al. (2014). OpenSWATH enables automated, targeted analysis of data-independent acquisition MS data. Nat Biotechnol 32(3): 219-223.\\
 \\
Suomi T., Corthals G., Nevalainen O.S., and Elo L.L. (2015). Using Peptide-Level Proteomics Data for Detecting Differentially Expressed Proteins. J Proteome Res. Nov 6;14(11):4564-70. doi: 10.1021/acs.jproteome.5b00363.\\
\\
Suomi, T. and Elo L.L. (2017). Enhanced differential expression statistics for data-independent acquisition proteomics" Scientific Reports 7, Article number:
5869.doi:10.1038/s41598-017-05949-y\\
\\
Teo G., et al. (2015). mapDIA: Preprocessing and statistical analysis of quantitative proteomics data from data independent acquisition mass spectrometry. J Proteomics 129: 108-120.\\
  \\
Teleman J., et al. (2015). DIANA--algorithmic improvements for analysis of data-independent acquisition MS data. Bioinformatics 31(4): 555-562.\\
 \\
Venable J.D., et al. (2004). Automated approach for quantitative analysis of complex peptide mixtures from tandem mass spectra. Nat Methods 1(1): 39-45.\\
\\
\\

\end{document}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{MSstats_data}
\alias{MSstats_data}
\title{Testing dataset in MSstats format.}
\description{
A small table with the column names corresponding to the MSstats format. 
 This data is intended only to test functions.
}
\author{
Peter Blattmann
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convert_protein_ids.R
\name{load_mart}
\alias{load_mart}
\title{Establish connection to biomaRt database}
\usage{
load_mart(species, ensembl.path = "www.ensembl.org", mart, verbose = FALSE)
}
\arguments{
\item{species}{The species of the protein identifiers in the term used by
biomaRt (e.g. "hsapiens_gene_ensembl", "mmusculus_gene_ensembl",
"drerio_gene_ensembl", etc.)}

\item{ensembl.path}{Ensembl host to connect to. Default: www.ensembl.org}

\item{mart}{The type of mart (e.g. "ENSEMBL_MART_ENSEMBL", etc.)}

\item{verbose}{print a summary of the ensembl connection.}
}
\value{
Connection for performing biomart queries.
}
\description{
This function establishes a connection to a biomart database.
}
\examples{
{
 data_table <- data.frame(Protein = c("Q01581", "P49327", "2/P63261/P60709"),
                          Abundance = c(100, 3390, 43423))
 mart <- convert_protein_ids(data_table)
 }
}
\author{
Peter Blattmann
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convert4MSstats.R
\name{convert4MSstats}
\alias{convert4MSstats}
\title{Get data ready for use by MSstats.}
\usage{
convert4MSstats(
  data,
  replace_values = TRUE,
  replace_colnames = TRUE,
  replace_unimod = TRUE
)
}
\arguments{
\item{data}{A data frame containing SWATH data.}

\item{replace_values}{Option to indicate if negative and 0 values should be replaced with NA.}

\item{replace_colnames}{Option to indicate if column names should be renamed
and columns reduced to the necessary columns for MSstats.}

\item{replace_unimod}{Option to indicate if Unimod Identifier should be
replaced from ":" to "_".}
}
\value{
Returns a data frame in the appropriate format for MSstats.
}
\description{
Though SWATH2stats uses very similar format as MSstats, some coercion is 
required to convert the data into the format for MSstats.
}
\details{
This functions selects the columns necessary for MSstats and renames them if
necessary.

The necessary columns are selected and three columns renamed:
FullPeptideName -> PeptideSequence
Charge -> PrecursorCharge
filename -> File
}
\examples{
 data("OpenSWATH_data", package="SWATH2stats")
 data("Study_design", package="SWATH2stats")
 data <- sample_annotation(OpenSWATH_data, Study_design)
 data.filtered.decoy <- filter_mscore(data, 0.01)
 raw <- disaggregate(data.filtered.decoy)
 data.mapDIA <- convert4MSstats(raw)
}
\references{
Choi M, Chang CY, Clough T, Broudy D, Killeen T, MacLean B, Vitek
  O. MSstats: an R package for statistical analysis of quantitative mass
  spectrometry-based proteomic experiments.Bioinformatics. 2014 Sep
  1;30(17):2524-6. doi: 10.1093/bioinformatics/btu305.
}
\author{
Peter Blattmann
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sample_annotation.R
\name{sample_annotation}
\alias{sample_annotation}
\title{Annotate the SWATH data with the sample information}
\usage{
sample_annotation(
  data,
  sample_annotation,
  data_type = "OpenSWATH",
  column_file = "filename",
  change_run_id = TRUE,
  verbose = FALSE
)
}
\arguments{
\item{data}{A data frame containing SWATH data.}

\item{sample_annotation}{A data frame containing the columns: Filename,
Condition, BioReplicate, Run. The values contained in the column filename
have to be present in the filename of the SWATH data.}

\item{data_type}{Option to specify the format of the table, if the column
names from an OpenSWATH output or MSstats table are used.}

\item{column_file}{Option to specify the column name where the injection 
file is specified. Default is set to "filename".}

\item{change_run_id}{Option to choose if the run_id column shall be
reassigned to a unique value combining the values of Condition,
BioReplicate and Run. (Option only possible if data is of format
"OpenSWATH")}

\item{verbose}{Option to turn on reporting on which filename it is working
on.}
}
\value{
Returns a dataframe with each row annotated for the study design
}
\description{
For statistical analysis and filtering the measurements need to be annotated
with Filename, Condition, BioReplicate, and Run. This functions takes this
information from a txt file containing this meta-data.
}
\details{
Given dataframes of TRIC processed data and sample annotations, mash them
together into something appropriate for downstream analyses.

This performs some quick sanity checks on the data and annotations and
creates the 'Condition', 'BioReplicate', and 'Run' columns along with other
columns expected by MSstats/OpenSWATH.
}
\examples{
 data("OpenSWATH_data", package="SWATH2stats")
 data("Study_design", package="SWATH2stats")
 data <- SWATH2stats::sample_annotation(OpenSWATH_data, Study_design, verbose=TRUE)
 summary(data)
}
\author{
Peter Blattmann
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filter_mscore_fdr.R
\name{filter_mscore_fdr}
\alias{filter_mscore_fdr}
\title{Filter annotated OpenSWATH/pyProphet output table to achieve a high FDR
quality data matrix with controlled overall protein FDR and quantitative
values for all peptides mapping to these high-confidence proteins (up to a
desired overall peptide level FDR quality).}
\usage{
filter_mscore_fdr(
  data,
  FFT = 1,
  overall_protein_fdr_target = 0.02,
  upper_overall_peptide_fdr_limit = 0.05,
  rm_decoy = TRUE,
  score_col = "m_score"
)
}
\arguments{
\item{data}{Annotated OpenSWATH/pyProphet data table.}

\item{FFT}{Ratio of false positives to true negatives, q-values from
[Injection_name]_full_stat.csv in pyProphet stats output. As an
approximation, the q-values of multiple runs are averaged and supplied as
argument FFT. Numeric from 0 to 1. Defaults to 1, the most conservative
value (1 Decoy indicates 1 False target). For further details see the
Vignette Section 1.3 and 4.1.}

\item{overall_protein_fdr_target}{FDR target for the protein master list for
which quantitative values down to the less strict peptide_fdr criterion
will be kept/reported. Defaults to 0.02.}

\item{upper_overall_peptide_fdr_limit}{Option to relax or tighten the false
discovery rate limit.}

\item{rm_decoy}{Logical T/F, whether decoy entries should be removed after
the analysis. Defaults to TRUE. Can be useful to disable to track the
influence on decoy fraction by further filtering steps such as requiring 2
peptides per protein.}

\item{score_col}{Defines the column from which to retrieve the m_score. 
If you use JPP (Rosenberger, Bludau et al. 2017) this can be used to 
select between Protein and transition_group m_score.}
}
\value{
Returns a data frame with the filtered data.
}
\description{
This function controls the protein FDR over a multi-run OpenSWATH/pyProphet
output table and filters all quantitative values to a desired overall/global
peptide FDR level.
It first finds a suitable m-score cutoff to minimally achieve a desired
global FDR quality on a protein master list based on the function
mscore4protfdr. It then finds a suitable m-score cutoff to minimally achieve
a desired global FDR quality on peptide level based on the function
mscore4pepfdr. Finally, it reports all the peptide quantities derived based
on the peptide level cutoff for only those peptides mapping to the protein
master list. It further summarizes the protein and peptide numbers remaining
after the filtering and evaluates the individual run FDR qualities of the
peptides (and quantitation events) selected.
}
\examples{
 data("OpenSWATH_data", package="SWATH2stats")
 data("Study_design", package="SWATH2stats")
 data <- sample_annotation(OpenSWATH_data, Study_design)
 data.fdr.filtered<-filter_mscore_fdr(data, FFT=0.7, 
                                      overall_protein_fdr_target=0.02,
                                      upper_overall_peptide_fdr_limit=0.1)
}
\author{
Moritz Heusel
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/write_matrix_proteins.R
\name{write_matrix_proteins}
\alias{write_matrix_proteins}
\title{Writes out an overview matrix of summed signals per protein identifier
(lines) over run_id(columns).}
\usage{
write_matrix_proteins(
  data,
  write_csv = FALSE,
  fun_aggregate = "sum",
  filename = "SWATH2stats_overview_matrix_proteinlevel.csv",
  rm_decoy = FALSE
)
}
\arguments{
\item{data}{A data frame containing annotated OpenSWATH/pyProphet data.}

\item{write_csv}{Option to determine if table should be written 
automatically into csv file.}

\item{fun_aggregate}{What function to use when aggregating the set of 
intensities (sum or mean)?. Default: sum.}

\item{filename}{File base name of the .csv matrix written out to the working
folder}

\item{rm_decoy}{Logical whether decoys will be removed from the data
matrix. Defaults to FALSE. It's sometimes useful to know how decoys behave
across a dataset and how many you allow into your final table with the
current filtering strategy.}
}
\value{
the peptides as a matrix, also output .csv matrix is written to the
  working folder
}
\description{
Writes out an overview matrix on protein level of a supplied (unfiltered or
filtered) OpenSWATH results data frame. The protein quantification is achieved
by summing the areas under all 6 transitions per precursor, summing all
precursors per FullPeptideName and all FullPeptideName signals per
ProteinName entry.
This function does not select consistently quantified or top peptides but
sums all signals availabe that may or may not originate from the same set of
peptides across different runs. A more detailed overview can be generated
using the function write_matrix_peptides().
Peptide selection can be achieved upstream using e.g. the functions
filter_mscore_requant(), filter_on_max_peptides() and
filter_on_min_peptides().
}
\examples{
{
 data("OpenSWATH_data", package="SWATH2stats")
 data("Study_design", package="SWATH2stats")
 data <- sample_annotation(OpenSWATH_data, Study_design)
 written <- write_matrix_proteins(data)
 }
}
\author{
Moritz Heusel
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{Study_design}
\alias{Study_design}
\title{A table containing the meta-data defining the study design of the OpenSWATH data.}
\description{
This table contains a unique identifier in the column "Filename"
 corresponding to the filename in the SWATH data.
 In the column "Condition" the perturbation performed is described.
 In the column "BioReplicate" the biological replicate is indicated.
 In the column "Run" a unique identifier for each injection.
 Technical injections would have different Run numbers but the same 
 BioReplicate number.
}
\author{
Peter Blattmann
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{OpenSWATH_data}
\alias{OpenSWATH_data}
\alias{data}
\title{Testing dataset from OpenSWATH.}
\description{
A small selection of the data obtained from the iPortal pipeline for an 
 SWATH/DIA experiment with perturbations relating to cholesterol regulation. 
 Protein and Peptides have been anonymized as the data is unpublished.
 The FDR version of the test data contains modified (lowered) decoy peak 
 group m_scores to simulate FDR behaviour of a large dataset.
}
\author{
Peter Blattmann
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/count_analytes.R
\name{count_analytes}
\alias{count_analytes}
\title{Counts the analytes across the different injections}
\usage{
count_analytes(
  data,
  column_levels = c("transition_group_id", "FullPeptideName", "ProteinName"),
  column_by = "run_id",
  rm_decoy = TRUE
)
}
\arguments{
\item{data}{A data frame containing SWATH data.}

\item{column_levels}{Columns in which the number of unique identifiers 
should be counted.}

\item{column_by}{Column for which the different identifiers should be counted
for, e.g. for the different injections.}

\item{rm_decoy}{Option to not remove decoy before counting.}
}
\value{
Returns a data frame with the count of the different identifiers per
  e.g. injection.
}
\description{
This functions counts the number of different peakgroups, peptides and
proteins in different injections.
}
\examples{
{
 data("OpenSWATH_data", package="SWATH2stats")
 data("Study_design", package="SWATH2stats")
 data <- sample_annotation(OpenSWATH_data, Study_design)
 counts <- count_analytes(data)
 }
}
\author{
Peter Blattmann
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.fdr_table.R
\name{plot.fdr_table}
\alias{plot.fdr_table}
\title{S3 plot function for results of class "fdr_table" as produced by e.g. the
function assess_fdr_overall()}
\usage{
\method{plot}{fdr_table}(x, output = "Rconsole", filename = "FDR_report_overall", ...)
}
\arguments{
\item{x}{List of class "fdr_table" as produced e.g. by the function
assess_fdr_overall() from this package.}

\item{output}{Choose output type. "pdf_csv" creates the output as files in
the working directory, "Rconsole" triggers delivery of the output to the
console enabling further computation or custom plotting / output.}

\item{filename}{Basename for output files to be created (if output =
"pdf_csv" has been selected).}

\item{...}{Extra arguments passed on to functions inside this.}
}
\value{
Plots in Rconsole or report files.
}
\description{
This function created standard plots from results of class "fdr_table" as
produced by e.g. the function assess_fdr_overall() visualizig ID numbers in
dependence of estimated FDR and also estimated FDR in dependence of m_score
cutoff.
}
\examples{
{
 data("OpenSWATH_data", package="SWATH2stats")
 data("Study_design", package="SWATH2stats")
 data <- sample_annotation(OpenSWATH_data, Study_design)
 x <- assess_fdr_overall(data, FFT=0.7, output="Rconsole", plot=FALSE)
 plot(x, output="Rconsole", filename="Assess_fdr_overall_testplot")
 }
}
\author{
Moritz Heusel
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reduce_OpenSWATH_output.R
\name{reduce_OpenSWATH_output}
\alias{reduce_OpenSWATH_output}
\title{Reduce columns of OpenSWATH data}
\usage{
reduce_OpenSWATH_output(data, column.names = NULL)
}
\arguments{
\item{data}{A data frame containing SWATH data.}

\item{column.names}{A vector of column names that can be selected.}
}
\value{
Returns a data frame with the selected columns.
}
\description{
This function selects the columns from the standard OpenSWATH output to
column needed for MSstats, aLFQ and mapDIA.
}
\note{
A basic set of columns are defined in the function and are used if no column
  names are indicated.

The column.names can be omitted and then the following columns are
  selected that are needed for MSstats and mapDIA analysis: ProteinName,
  FullPeptideName, Sequence, Charge, aggr_Fragment_Annotation,
  aggr_Peak_Area, filename, m_score, decoy, Intensity, RT.
  This function should be ommitted if the data is analyzed afterwards with
  the aLFQ or imsbInfer package that needs further columns.
}
\examples{
 data("OpenSWATH_data", package="SWATH2stats")
 data("Study_design", package="SWATH2stats")
 data <- sample_annotation(OpenSWATH_data, Study_design)
 data.filtered <- reduce_OpenSWATH_output(data)
}
\author{
Peter Blattmann
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{Spyogenes}
\alias{Spyogenes}
\title{S.pyogenes example data.}
\description{
A table containing SWATH-MS data from S.pyogenes.
 This table was generated from the original data deposited on PeptideAtlas 
 (PASS00289, file "rawOpenSwathResults_1pcnt_only.tsv") by selecting only 
 the column necessary for the SWATH2stats.
}
\references{
Rost, H. L., et al. (2014). OpenSWATH enables automated, 
  targeted analysis of data-independent acquisition MS data. 
  Nat Biotechnol 32(3): 219-223.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filter_all_peptides.R
\name{filter_all_peptides}
\alias{filter_all_peptides}
\title{Select all proteins that are supported by peptides.}
\usage{
filter_all_peptides(data)
}
\arguments{
\item{data}{A data frame containing SWATH data.}
}
\value{
Returns a data frame with the data from both proteotypic and
  non-proteotypic peptides.
}
\description{
This function can be used as opposed to the function 
"filter_proteotypic_peptides()". This function counts all proteins (including
proteins supported by non proteo-typic (i.e. shared) peptides). 
All peptides (incl. non proteotypic peptides are selected. 
For the proteins supported by proteotypic peptide the "1/" in front
of the identifier is removed to facilitate further data processing. 
The protein identifier of shared peptides needs to be separated by a slash "/".
}
\examples{
{
 data("OpenSWATH_data", package="SWATH2stats")
 data("Study_design", package="SWATH2stats")
 data <- sample_annotation(OpenSWATH_data, Study_design)
 data.filtered.decoy <- filter_mscore(data, 0.01)
 data.all <- filter_all_peptides(data.filtered.decoy)
 }
}
\author{
Peter Blattmann
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_variation.R
\name{plot_variation}
\alias{plot_variation}
\title{Plots the coefficient of variation for different replicates.}
\usage{
plot_variation(
  data,
  column.values = "Intensity",
  comparison = transition_group_id + Condition ~ BioReplicate,
  fun_aggregate = NULL,
  label = FALSE,
  title = "cv across conditions",
  boxplot = TRUE,
  ...
)
}
\arguments{
\item{data}{Data frame that is produced by the OpenSWATH/pyProphet workflow.}

\item{column.values}{Indicates the columns for which the variation is
assessed. This can be the Intensity or Signal, but also the retention
time.}

\item{comparison}{The comparison for assessing the variability. Default is to
assess the variability per transition_group_id and Condition over the
different Replicates. Comparison is performed using the dcast() function of
the reshape2 package.}

\item{fun_aggregate}{If for the comparison values have to be aggregated one
needs to provide the function here.}

\item{label}{Option to print value of median cv.}

\item{title}{Title of plot. Default: "cv across conditions"}

\item{boxplot}{Logical. If boxplot should be plotted. Default: TRUE}

\item{...}{further arguments passed to method.}
}
\value{
Returns a list with the data and calculated cv and a table that
  summarizes the mean, median and mode cv per Condition (if Condition is
  contained in the comparison). In addition it plots in Rconsole a violin
  plot with the observed coefficient of variations.
}
\description{
This function plots the coefficient of variation within replicates for a
given value. If decoys are present these are removed before plotting.
}
\examples{
{
 data("OpenSWATH_data", package="SWATH2stats")
 data("Study_design", package="SWATH2stats")
 data <- sample_annotation(OpenSWATH_data, Study_design)
 var_summary <- plot_variation(data)
 }
}
\author{
Peter Blattmann
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convert_protein_ids.R
\name{convert_protein_ids}
\alias{convert_protein_ids}
\title{Convert protein ids}
\usage{
convert_protein_ids(
  data_table,
  column_name = "Protein",
  species = "hsapiens_gene_ensembl",
  host = "https://www.ensembl.org",
  mart = "ENSEMBL_MART_ENSEMBL",
  ID1 = "uniprotswissprot",
  ID2 = "hgnc_symbol",
  id.separator = "/",
  copy_nonconverted = TRUE,
  verbose = FALSE
)
}
\arguments{
\item{data_table}{A data frame or file name.}

\item{column_name}{The column name where the original protein identifiers are present.}

\item{species}{The species of the protein identifiers in the term used by
biomaRt (e.g. "hsapiens_gene_ensembl", "mmusculus_gene_ensembl",
"drerio_gene_ensembl", etc.)}

\item{host}{Path of the biomaRt database (e.g. "www.ensembl.org",
"dec2017.archive.ensembl.org").}

\item{mart}{The type of mart (e.g. "ENSEMBL_MART_ENSEMBL", etc.)}

\item{ID1}{The type of the original protein identifiers
(e.g. "uniprotswissprot", "ensembl_peptide_id").}

\item{ID2}{The type of the converted protein identifiers
(e.g. "hgnc_symbol", "mgi_symbol", "external_gene_name").}

\item{id.separator}{Separator between protein identifiers of shared
peptides.}

\item{copy_nonconverted}{Option defining if the identifiers that cannot be
converted should be copied.}

\item{verbose}{Option to write a file containing the version of the
database used.}
}
\value{
The data frame with an added column of the converted protein
  identifiers.
}
\description{
This function renames protein ids in a data frame or file
}
\note{
Protein identifiers from shared peptides should be separated by a
  forward slash. The host of archived ensembl databases can be introduced as
  well (e.g. "dec2017.archive.ensembl.org")
}
\examples{
 \dontrun{
  data_table <- data.frame(
       "Protein" = c("Q01581", "P49327", "2/P63261/P60709"),
       "Abundance" = c(100, 3390, 43423))
  convert_protein_ids(data_table)
}
}
\author{
Peter Blattmann
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convert4PECA.R
\name{convert4PECA}
\alias{convert4PECA}
\title{Convert table into the format for ROPECA}
\usage{
convert4PECA(data)
}
\arguments{
\item{data}{A data frame containing SWATH data.}
}
\value{
Returns a data frame in the appropriate format for ROPECA.
}
\description{
This functions selects the columns necessary for ROPECA.
}
\note{
The table must not contain any technical replica, the intensity of
  technical replica is averaged. This function requires the package reshape2.
}
\examples{
{
 data("OpenSWATH_data", package="SWATH2stats")
 data("Study_design", package="SWATH2stats")
 data <- sample_annotation(OpenSWATH_data, Study_design)
 data.filtered.decoy <- filter_mscore(data, 0.01)
 data.PECA <- convert4PECA(data.filtered.decoy)
}
}
\references{
Suomi, T. and Elo L.L. (2017). "Enhanced differential expression
  statistics for data-independent acquisition proteomics" Scientific Reports
  7, Article number: 5869.doi:10.1038/s41598-017-05949-y
}
\author{
Peter Blattmann
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/validate_columns.R
\name{validate_columns}
\alias{validate_columns}
\title{Validate columns for a data.frame}
\usage{
validate_columns(data, columns, verbose = FALSE)
}
\arguments{
\item{data}{data.frame to check for columns.}

\item{columns}{List of column names to be checked if they exist.}

\item{verbose}{Logical if message should be printed. Default = FALSE}
}
\value{
Returns list of columns that are present
}
\description{
This function looks at the different possible column names given and chooses 
the one present in a data.frame. If none of the column names fit or if 
multiple names fit the function stops with an appropriate error  message.
The functions returns a list with the column names existing that can be used.
}
\examples{
{
 validate_columns(cars, list(Speed = c("speed")))
 
 # if out of two possible column one exists
 validate_columns(cars, list(Speed = c("speed", "velocity")))
 validate_columns(cars, list(Speed = c("speed", "velocity")), verbose = TRUE)
 
 }
}
\author{
Peter Blattmann
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convert_protein_ids.R
\name{add_genesymbol}
\alias{add_genesymbol}
\title{Adds gene symbols to a table}
\usage{
add_genesymbol(
  data_table,
  gene_ID_table,
  column_name = "Protein",
  ID1 = "uniprotswissprot",
  ID2 = "hgnc_symbol",
  id.separator = "/",
  copy_nonconverted = TRUE
)
}
\arguments{
\item{data_table}{A data frame or file name.}

\item{gene_ID_table}{A table to match gene identifiers against}

\item{column_name}{The column name where the original protein identifiers
are present.}

\item{ID1}{The type of the original protein identifiers
(e.g. "uniprotswissprot", "ensembl_peptide_id").}

\item{ID2}{The type of the converted protein identifiers
(e.g. "hgnc_symbol", "mgi_symbol", "external_gene_name").}

\item{id.separator}{Separator between protein identifiers of shared
peptides.}

\item{copy_nonconverted}{Option defining if the identifiers that cannot be
converted should be copied.}
}
\value{
Returns the data frame with an added column of the converted protein identifiers.
}
\description{
Gather gene symbols from biomart and add them to a data frame.
}
\note{
Protein identifiers from shared peptides should be separated by a forward
slash. The host of archived ensembl databases can be introduced as well
(e.g. "dec2017.archive.ensembl.org")
}
\examples{
{
  gene_ID_table <- data.frame(uniprotswissprot = c("Q01581", "P49327", "P60709"), 
                              hgnc_symbol = c("HMGCS1", "FASN", "ACTB"))
  data_table <- data.frame(Protein = c("Q01581", "P49327", "2/P63261/P60709"), 
                           Abundance = c(100, 3390, 43423))
  add_genesymbol(data_table, gene_ID_table)
  }
}
\author{
Peter Blattmann
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/JPP_update.R
\name{JPP_update}
\alias{JPP_update}
\title{Select alternate m_score column in JPP data and avert user}
\usage{
JPP_update(data, mscore_col)
}
\arguments{
\item{data}{Data table that is produced by the OpenSWATH/pyProphet workflow}

\item{mscore_col}{Defines the column from which to retrieve the m_score. 
If you use JPP (Rosenberger, Bludau et al. 2017) this can be used to 
select between Protein and transition_group m_score.}
}
\value{
Returns the mscore_col that might have been changed to 
   transition_group_id_m_score and gives a message to the user.
}
\description{
The output from JPP (Rosenberger, Bludau et al. 2017) has not anymore the 
m_score column but an ProteinName_m_score and transition_group_id_m_score. 
To make the users aware this function tests if the m_score column still 
exists and selects as default the transition_group_id_m_score column.
}
\examples{
{
data("OpenSWATH_data", package="SWATH2stats")
JPP_update(OpenSWATH_data, "m_score")
}
}
\author{
Peter Blattmann
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/removeDecoyProteins.R
\name{rmDecoyProt}
\alias{rmDecoyProt}
\title{Subfunction to remove decoys}
\usage{
rmDecoyProt(x, decoy_string = "DECOY")
}
\arguments{
\item{x}{proteinname string to query.}

\item{decoy_string}{String defining a decoy}
}
\value{
returns string without elements containing the decoy string
}
\description{
Subfunction to remove decoys
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filter_on_min_peptides.R
\name{filter_on_min_peptides}
\alias{filter_on_min_peptides}
\title{Filter openSWATH output for proteins that are identified by a minimum of n independent peptides.}
\usage{
filter_on_min_peptides(
  data,
  n_peptides,
  protein_col = "ProteinName",
  peptide_col = c("Peptide.Sequence", "FullPeptideName"),
  rm.decoy = TRUE
)
}
\arguments{
\item{data}{Data table that is produced by the OpenSWATH/iPortal workflow.}

\item{n_peptides}{Number of minimal number of peptide IDs associated with a
protein ID in order to be kept in the dataset.}

\item{protein_col}{Column with protein identifiers. Default: ProteinName}

\item{peptide_col}{Column with peptide identifiers. Default: Peptide.Sequence or FullPeptideName}

\item{rm.decoy}{Option to remove the decoys during filtering.}
}
\value{
Returns the filtered data frame with only peptides that map to
  proteins with >= n_peptides peptides.
}
\description{
This function removes entries mapping to proteins that are identified by less
than n_peptides.
Removing single-hit proteins from an analysis can significantly increase the
sensitivity under strict protein fdr criteria, as evaluated by
e.g. assess_fdr_overall.
}
\examples{
{
 data("OpenSWATH_data", package="SWATH2stats")
 data("Study_design", package="SWATH2stats")
 data <- sample_annotation(OpenSWATH_data, Study_design)
 data.filtered <- filter_mscore_freqobs(data, 0.01,0.8)
 data.max <- filter_on_max_peptides(data.filtered, 5)
 data.min.max <- filter_on_min_peptides(data.max, 3)
 }
}
\author{
Moritz Heusel, Peter Blattmann
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mscore4pepfdr.R
\name{mscore4pepfdr}
\alias{mscore4pepfdr}
\title{Find m_score cutoff to reach a desired FDR on peptide level (over the entire
OpenSWATH/pyProphet output table)}
\usage{
mscore4pepfdr(data, FFT = 1, fdr_target = 0.01, mscore.col = "m_score")
}
\arguments{
\item{data}{Annotated OpenSWATH/pyProphet data table. See function
sample_annotation from this package.}

\item{FFT}{Ratio of false positives to true negatives, q-values from
[Injection_name]_full_stat.csv in pyProphet stats output. As an
approximation, the q-values of multiple runs are averaged and supplied as
argument FFT. Numeric from 0 to 1. Defaults to 1, the most conservative
value (1 Decoy indicates 1 False target).}

\item{fdr_target}{FDR target, numeric, defaults to 0.01. An m_score cutoff
achieving an FDR < fdr_target will be selected.
Calculated as FDR = (TN*FFT/T); TN=decoys, T=targets, FFT=see above.}

\item{mscore.col}{column in the data containing the m score data.}
}
\value{
Returns the m_score cutoff selected to arrive at the desired FDR
}
\description{
This function estimates the m_score cutoff required in a dataset to reach a
given overall peptide level FDR.
It counts target and decoy peptides (unique FullPeptideName) at high
resolution across the m_score cutoffs and reports a useful m_score cutoff -
peptide FDR pair close to the supplied fdr_target level over the entire
dataset. The m_score cutoff is returned by the function and can be used in
the context of the filtering functions, e.g.:
data.pepFDR2pc<-filter_mscore(data, mscore4pepfdr(data, fdr_target=0.02))
To arrive from decoy counts at an estimation of the false discovery rate
(false positives among the targets remaining at a given mscore cutoff) the
ratio of false positives to true negatives (decoys) (FFT) must be
supplied. It is estimated for each run individually by pyProphet and
contained in the pyProphet statistics [Injection_name]_full_stat.csv. As an
approximation, the FFTs of multiple runs are averaged and supplied as
argument FFT. For further details see the Vignette Section 1.3 and 4.1.
For FDR evaluations on assay and protein level, please refer to functions
mscore4assayfdr and mscore4protfdr
}
\examples{
 data("OpenSWATH_data", package="SWATH2stats")
 data("Study_design", package="SWATH2stats")
 data <- sample_annotation(OpenSWATH_data, Study_design)
 chosen <- mscore4pepfdr(data, FFT=0.7, fdr_target=0.01)
}
\author{
Moritz Heusel
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.fdr_cube.R
\name{plot.fdr_cube}
\alias{plot.fdr_cube}
\title{S3 plot function for FDR assessment result arrays}
\usage{
\method{plot}{fdr_cube}(
  x,
  output = "Rconsole",
  filename = "FDR_report_byrun",
  plot_mscore_levels = c(0.01, 0.001),
  ...
)
}
\arguments{
\item{x}{Array of by-run FDR assessment results as produced e.g. by the
function assess_fdr_byrun() from this package.}

\item{output}{Choose output type. "pdf_csv" creates the output as files in
the working directory, "Rconsole" triggers delivery of the output to the
console enabling further computation and/or custom plotting / output.}

\item{filename}{Basename for output files to be created (if output =
"pdf_csv" has been selected).}

\item{plot_mscore_levels}{Define m-score levels to plot the estimated FDR
results.}

\item{...}{Extra arguments passed on to functions inside this.}
}
\value{
Plots in Rconsole or report files.
}
\description{
This function creates standard plots from result arrays as produced by
e.g. the function assess_fdr_byrun(), visualizig assay, peptide and protein
level FDR for each run at m-score cutoffs 1e-2 and 1e-3. Furthermore, Target
and Decoy ID numbers are visualized.
}
\examples{
{
 data("OpenSWATH_data", package="SWATH2stats")
 data("Study_design", package="SWATH2stats")
 data <- sample_annotation(OpenSWATH_data, Study_design)
 x <- assess_fdr_byrun(data, FFT=0.7, output="Rconsole", plot=FALSE)
 retlist <- plot(x, output="Rconsole", filename="Assess_fdr_byrun_testplot",
                          plot_mscore_levels=0.01)
 }
}
\author{
Moritz Heusel
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transform_MSstats_OpenSWATH.R
\name{transform_MSstats_OpenSWATH}
\alias{transform_MSstats_OpenSWATH}
\title{Transforms column names to OpenSWATH column names}
\usage{
transform_MSstats_OpenSWATH(data)
}
\arguments{
\item{data}{A data frame containing the SWATH data in the MSstats format}
}
\value{
The data frame in the appropriate format.
}
\description{
This functions transforms the column names from a data frame in MSstats
format to a data frame with column names used by the OpenSWATH output. The
original table needs to contain at least the 10 columns defined by MSstats:
ProteinName, PeptideSequence, PrecursorCharge, Fragmentation, ProductCharge,
IsotopeLabelType, Condition, BioReplicate, Run, Intensity.)
}
\examples{
 data("MSstats_data", package="SWATH2stats")
 transformed <- transform_MSstats_OpenSWATH(MSstats_data)
}
\references{
Choi M, Chang CY, Clough T, Broudy D, Killeen T, MacLean B, Vitek
  O. MSstats: an R package for statistical analysis of quantitative mass
  spectrometry-based proteomic experiments.Bioinformatics. 2014 Sep
  1;30(17):2524-6. doi: 10.1093/bioinformatics/btu305.
}
\author{
Peter Blattmann
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assess_fdr_overall.R
\name{assess_fdr_overall}
\alias{assess_fdr_overall}
\title{Assess overall FDR in annotated OpenSWATH/pyProphet output table in
dependence of m_score cutoff}
\usage{
assess_fdr_overall(
  data,
  FFT = 1,
  n_range = 20,
  output = "pdf_csv",
  plot = TRUE,
  filename = "FDR_report_overall",
  score_col = "m_score"
)
}
\arguments{
\item{data}{Data table that is produced by the OpenSWATH/pyProphet workflow}

\item{FFT}{Ratio of false positives to true negatives, q-values from
[Injection_name]_full_stat.csv in pyProphet stats output. As an
approximation, the q-values of multiple runs are averaged and supplied as
argument FFT. Numeric from 0 to 1. Defaults to 1, the most conservative
value (1 Decoy indicates 1 False target).}

\item{n_range}{I am also not certain what this is, nor why 20 is the
optimal default value, but I think the idea is to set up a series of mscore
thresholds.}

\item{output}{Choose output type. "pdf_csv" creates the output as files in
the working directory, "Rconsole" triggers delivery of the output to the
console enabling further computation or custom plotting / output.}

\item{plot}{Logical, whether or not to create plots from the results (using
the associated method plot.fdr_table()}

\item{filename}{Optional, modifying the basename of the result files if
applicable.}

\item{score_col}{Column that contains the score. Default. m_score}
}
\value{
Returns a list of class "fdr_table". If output "pdf_csv" and plot =
  TRUE were chosen, report files are written to the working folder.
}
\description{
This function estimates the assay, peptide and protein FDR over a multi-run
OpenSWATH/pyProphet output table. It counts target and decoy assays (unique
transition_group_id), peptides (unique FullPeptideName) and proteins (unique
ProteinName) in dependence of the m-score cutoff (1e-2 to 1e-20).
To arrive from decoy counts at an estimation of the false discovery rate
(false positives among the targets remaining at a given mscore cutoff) the
ratio of false positives to true negatives (decoys) (FFT) must be
supplied. It is estimated for each run individually by pyProphet and
contained in the pyProphet statistics [Injection_name]_full_stat.csv. As an
approximation, the FFTs of multiple runs are averaged and supplied as
argument FFT. For further details see the Vignette Section 1.3 and
4.1. Protein FDR control on peak group quality level is a very strict filter
and should be handled with caution.
FDR is calculated as FDR = (TN*FFT/T); TN=decoys, T=targets, FFT=see above
}
\examples{
{
 data("OpenSWATH_data", package="SWATH2stats")
 data("Study_design", package="SWATH2stats")
 data <- sample_annotation(OpenSWATH_data, Study_design)
 assess_fdr_overall(data, FFT=0.7, output="Rconsole", plot=TRUE,
                    filename="Testoutput_assess_fdr_overall")
}
}
\author{
Moritz Heusel
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filter_on_max_peptides.R
\name{filter_on_max_peptides}
\alias{filter_on_max_peptides}
\title{Filter only for the highest intense peptides}
\usage{
filter_on_max_peptides(
  data,
  n_peptides,
  protein_col = "ProteinName",
  peptide_col = c("Peptide.Sequence", "FullPeptideName"),
  rm.decoy = TRUE
)
}
\arguments{
\item{data}{A data frame containing SWATH data with the column names:
ProteinNames, PeptideSequence, PrecursorCharge, Intensity.}

\item{n_peptides}{Maximum number of highest intense peptides to filter the
data on.}

\item{protein_col}{Column with protein identifiers. Default: ProteinName}

\item{peptide_col}{Column with peptide identifiers. Default: Peptide.Sequence or FullPeptideName}

\item{rm.decoy}{Option to remove the decoys during filtering.}
}
\value{
Returns a data frame of the filtered data.
}
\description{
In order to reduce the data, the data is filtered only for the proteins with
the highest intensity peptides.
}
\examples{
{
 data("OpenSWATH_data", package="SWATH2stats")
 data("Study_design", package="SWATH2stats")
 data <- sample_annotation(OpenSWATH_data, Study_design)
 data.filtered <- filter_mscore_freqobs(data, 0.01,0.8)
 data.max <- filter_on_max_peptides(data.filtered, 5)
 }
}
\author{
Peter Blattmann
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/removeDecoyProteins.R
\name{removeDecoyProteins}
\alias{removeDecoyProteins}
\title{Removes decoy proteins from the protein group label}
\usage{
removeDecoyProteins(data, column = "ProteinName", decoy_string = "DECOY")
}
\arguments{
\item{data}{A data frame containing SWATH data.}

\item{column}{Column to query for decoy string}

\item{decoy_string}{String defining a decoy. Default: DECOY}
}
\value{
Returns a data frame with changed protein labels
}
\description{
There exist peptides annotated as protein groups with
2/ProteinA/DECOY_ProteinB. However these are in principal proteotypic
peptides and should be annoated 1/ProteinA. This function changes these
labels accordingly. The subfunction rmDecoyProt removes the Decoy protein,
calling removeDecoyProteins also changes the nubmer before the protein group
accordingly.
}
\examples{
 data("OpenSWATH_data", package="SWATH2stats")
 data("Study_design", package="SWATH2stats")
 data <- sample_annotation(OpenSWATH_data, Study_design)
 data.filtered.decoy <- filter_mscore(data, 0.01)
 data.2 <- removeDecoyProteins(data.filtered.decoy)
}
\author{
Moritz Heusel
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mscore4protfdr.R
\name{mscore4protfdr}
\alias{mscore4protfdr}
\title{Find m_score cutoff to reach a desired FDR on protein level (over the entire
OpenSWATH/pyProphet output table)}
\usage{
mscore4protfdr(data, FFT = 1, fdr_target = 0.02, mscore.col = "m_score")
}
\arguments{
\item{data}{Annotated OpenSWATH/pyProphet data table. See function
sample_annotation from this package.}

\item{FFT}{Ratio of false positives to true negatives, q-values from
[Injection_name]_full_stat.csv in pyProphet stats output. As an
approximation, the q-values of multiple runs are averaged and supplied as
argument FFT. Numeric from 0 to 1. Defaults to 1, the most conservative
value (1 Decoy indicates 1 False target).}

\item{fdr_target}{FDR target, numeric, defaults to 0.01. An m_score cutoff
achieving an FDR < fdr_target will be selected.
Calculated as FDR = (TN*FFT/T); TN=decoys, T=targets, FFT=see above.}

\item{mscore.col}{Column containing the mscore data.}
}
\value{
Returns the m_score cutoff selected to arrive at the desired FDR
  quality.
}
\description{
This function estimates the m_score cutoff required in a dataset to reach a
given overall protein level FDR.
This filter is to be used with caution as the resulting quantitative matrix
is relatively sparse. It can be filled with quantitative values at a lower
FDR quality level.
It counts target and decoy peptides (unique ProteinName) at high resolution
across the m_score cutoffs and reports a useful m_score cutoff - peptide FDR
pair close to the supplied fdr_target level over the entire dataset. The
m_score cutoff is returned by the function and can be used in the context of
the filtering functions, e.g.:
data.protFDR5pc<-filter_mscore(data, mscore4protfdr(data, fdr_target=0.02))
To arrive from decoy counts at an estimation of the false discovery rate
(false positives among the targets remaining at a given mscore cutoff) the
ratio of false positives to true negatives (decoys) (FFT) must be
supplied. It is estimated for each run individually by pyProphet and
contained in the pyProphet statistics [Injection_name]_full_stat.csv. As an
approximation, the FFTs of multiple runs are averaged and supplied as
argument FFT. For further details see the Vignette Section 1.3 and 4.1.
For FDR evaluations on assay and peptide level, please refer to functions
mscore4assayfdr and mscore4pepfdr.
}
\examples{
 data("OpenSWATH_data", package="SWATH2stats")
 data("Study_design", package="SWATH2stats")
 data <- sample_annotation(OpenSWATH_data, Study_design)
 chosen <- mscore4protfdr(data, FFT=0.7, fdr_target=0.01)
}
\author{
Moritz Heusel
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_variation_vs_total.R
\name{plot_variation_vs_total}
\alias{plot_variation_vs_total}
\title{Plots the total variation versus variation within replicates}
\usage{
plot_variation_vs_total(
  data,
  column.values = "Intensity",
  comparison1 = transition_group_id ~ BioReplicate + Condition,
  comparison2 = transition_group_id + Condition ~ BioReplicate,
  fun_aggregate = NULL,
  label = FALSE,
  title = "coefficient of variation - total versus within replicates",
  boxplot = TRUE,
  ...
)
}
\arguments{
\item{data}{Data table that is produced by the OpenSWATH/pyProphet workflow.}

\item{column.values}{Indicates the columns for which the variation is
assessed. This can be the Intensity or Signal, but also the retention time.}

\item{comparison1}{The comparison for assessing the total
variability. Default is to assess the variability per transition_group_id
over the combination of Replicates and different Conditions.}

\item{comparison2}{The comparison for assessing the variability within the
replicates. Default is to assess the variability per transition_group_id
and Condition over the different Replicates.}

\item{fun_aggregate}{If depending on the comparison values have to be
aggregated one needs to provide the function here. (I think this should be
sum, yesno?)}

\item{label}{Option to print value of median cv.}

\item{title}{Title of plot. Default: "cv across conditions"}

\item{boxplot}{Logical. If boxplot should be plotted. Default: TRUE}

\item{...}{Arguments passed through, currently unused.}
}
\value{
Plots in Rconsole a violin plot comparing the total variation with
  the variation within replicates. In addition it returns the data frame from
  which the plotting is done and a table with the calculated mean, median and
  mode of the cv for the total or replicate data.
}
\description{
This function plots the total variation and the variation within replicates
for a given value. If decoys are present these are removed before plotting.
}
\examples{
{
 data("OpenSWATH_data", package="SWATH2stats")
 data("Study_design", package="SWATH2stats")
 data <- sample_annotation(OpenSWATH_data, Study_design)
 var_summary <- plot_variation_vs_total(data)
 }
}
\author{
Peter Blattmann
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/unifyProteinGroupLabels.R
\name{unifyProteinGroupLabels}
\alias{unifyProteinGroupLabels}
\title{Unify the protein group labels.}
\usage{
unifyProteinGroupLabels(data, column = "ProteinName")
}
\arguments{
\item{data}{A data frame containing SWATH data.}

\item{column}{Which column to use for unifying the protein group identifiers.}
}
\value{
Returns a data frame with the unififed protein labels.
}
\description{
Unify the protein group labels (2/ProteinA/ProteinB and 2/ProteinB/ProteinA)
to one common label (e.g. 2/ProteinA/ProteinB)
}
\examples{
 data("OpenSWATH_data", package="SWATH2stats")
 data("Study_design", package="SWATH2stats")
 data <- sample_annotation(OpenSWATH_data, Study_design)
 data.filtered.decoy <- filter_mscore(data, 0.01)
 data.unified <- unifyProteinGroupLabels(data.filtered.decoy)
}
\author{
Moritz Heusel
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assess_decoy_rate.R
\name{assess_decoy_rate}
\alias{assess_decoy_rate}
\title{Assess decoy rate in data}
\usage{
assess_decoy_rate(data, column = "FullPeptideName", column_decoy = "decoy")
}
\arguments{
\item{data}{A data frame that contains at least a column named 
"FullPeptideName" and "decoy".}

\item{column}{The column name of the Peptide identifier. 
Default: FullPeptideName.}

\item{column_decoy}{The column name of the decoy column. Default: decoy.}
}
\value{
Message detailing the number of decoys, non-decoys, and the ratio.
}
\description{
This function assesses the number of quantifications (typically peptides)
that are decoys (false-positive) versus true identifications.
}
\details{
A printout is generated to indicate the number of non-decoy, decoy peptides
and the rate of decoy vs non-decoy peptides. Unique peptides are counted, so
a precursor with different charge states is counted as one peptide. In the
column "decoy" the values need to be 1,0 or TRUE and FALSE.
}
\examples{
 data("OpenSWATH_data", package="SWATH2stats")
 data <- OpenSWATH_data
 assess_decoy_rate(data)
}
\author{
Peter Blattmann
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import_data.R
\name{import_data}
\alias{import_data}
\title{Transforms the column names from a data frame to the required format.}
\usage{
import_data(data)
}
\arguments{
\item{data}{A data frame containing the SWATH-MS data (one line per peptide
precursor quantified) but with different column names.}
}
\value{
Returns the data frame in the appropriate format.
}
\description{
This functions transforms the column names from a data frame from another
format to a data frame with column names used by the OpenSWATH output and
required for these functions. During executing of the function the
corresponding columns for each column in the data need to be selected. For
columns that do not corresond to a certain column 'not applicable' needs to
be selected and the column names are not changed.
}
\note{
List of column names of the OpenSWATH data:
ProteinName: Unique identifier for protein or proteingroup that the peptide
  maps to. Proteotypic peptides should be indicated by 1/ in order to be
  recognized as such by the function filter_proteotypic_peptides.
FullPeptideName: Unique identifier for the peptide.
Charge: Charge of the peptide precursor ion quantified.
Sequence: Naked peptide sequence without modifications.
aggr_Fragment_Annotation: aggregated annotation for the different Fragments
  quantified for this peptide. In the OpenSWATH results the different
  annotation in OpenSWATH are concatenated by a semicolon.
aggr_Peak_Area: aggregated Intensity values for the different Fragments
  quantified for this peptide. In the OpenSWATH results the aggregated Peak
  Area intensities are concatenated by a semicolon.
transition_group_id: A unique identifier for each transition group used.
decoy: Indicating with 1 or 0 if this transition group is a decoy.
m_score: Column containing the score that is used to estimate FDR or
  filter. M-score values of identified peak groups are equivalent to a
  q-value and thus typically are smaller than 0.01, depending on the
  confidence of identification (the lower the m-score, the higher the
  confidence).
Column containing the score that is used to estimate FDR or filter.
RT: Column containing the retention time of the quantified peak.
filename: Column containing the filename or a unique identifier for each
  injection.
Intensity: column containing the intensity value for each quantified peptide.
Columns needed for FDR estimation and filtering functions: ProteinName,
  FullPeptideName, transition_group_id, decoy, m_score
Columns needed for conversion to transition-level format (needed for MSStats
  and mapDIA input): aggr_Fragment_Annotation, aggr_Peak_Are
}
\examples{
 data('Spyogenes', package = 'SWATH2stats')
 head(data)
 str(data)
}
\author{
Peter Blattmann
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convert4mapDIA.R
\name{convert4mapDIA}
\alias{convert4mapDIA}
\title{Convert table into the format for mapDIA}
\usage{
convert4mapDIA(data, RT = FALSE)
}
\arguments{
\item{data}{A data frame containing SWATH data.}

\item{RT}{Option to export the retention times.}
}
\value{
Returns a data frame in the appropriate format for mapDIA.
}
\description{
This functions selects the columns necessary for mapDIA.
}
\note{
The table must not contain any technical replica, the intensity of
  technical replica is averaged. This function requires the package
  reshape2.
}
\examples{
{
 data("OpenSWATH_data", package="SWATH2stats")
 data("Study_design", package="SWATH2stats")
 data <- sample_annotation(OpenSWATH_data, Study_design)
 data.filtered.decoy <- filter_mscore(data, 0.01)
 raw <- disaggregate(data.filtered.decoy)
 data.mapDIA <- convert4mapDIA(raw, RT=TRUE)
 }
}
\references{
Teo, G., et al. (2015). "mapDIA: Preprocessing and statistical
  analysis of quantitative proteomics data from data independent acquisition
  mass spectrometry." J Proteomics 129: 108-120.
}
\author{
Peter Blattmann
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filter_proteotypic_peptides.R
\name{filter_proteotypic_peptides}
\alias{filter_proteotypic_peptides}
\title{Filter for proteins that are supported by proteotypic peptides.}
\usage{
filter_proteotypic_peptides(data, rm.decoy = TRUE)
}
\arguments{
\item{data}{A data frame containing SWATH data.}

\item{rm.decoy}{Option to remove the decoys during filtering.}
}
\value{
Returns a data frame with only the data supported by proteotypic
  peptides.
}
\description{
Peptides can match to several proteins. With this function proteotypic
peptides, peptides that are only contained in one protein are
selected. Additionally the number of proteins are counted and printed.
}
\examples{
 data("OpenSWATH_data", package="SWATH2stats")
 data("Study_design", package="SWATH2stats")
 data <- sample_annotation(OpenSWATH_data, Study_design)
 data.filtered.decoy <- filter_mscore(data, 0.01)
 data.all <- filter_proteotypic_peptides(data.filtered.decoy)
}
\author{
Peter Blattmann
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_correlation_between_samples.R
\name{plot_correlation_between_samples}
\alias{plot_correlation_between_samples}
\title{Plots the correlation between injections.}
\usage{
plot_correlation_between_samples(
  data,
  column_values = "Intensity",
  comparison = transition_group_id ~ Condition + BioReplicate,
  fun_aggregate = NULL,
  label = TRUE,
  ...
)
}
\arguments{
\item{data}{Data frame that is produced by the OpenSWATH/pyProphet workflow.}

\item{column_values}{Indicates the columns for which the correlation is
assessed. This can be the Intensity or Signal, but also the retention
time.}

\item{comparison}{The comparison for assessing the variability. Default is
to assess the variability per transition_group_id over the different
Condition and Replicates. Comparison is performed using the dcast()
function of the reshape2 package.}

\item{fun_aggregate}{If for the comparison values have to be aggregated one
needs to provide the function here.}

\item{label}{Option to print correlation value in the plot.}

\item{...}{Further arguments passed to methods.}
}
\value{
Plots in Rconsole a correlation heatmap and returns the data frame
  used to do the plotting.
}
\description{
This function plots the Pearson's and Spearman correlation between
samples. If decoys are present these are removed before plotting.
}
\examples{
{
 data("OpenSWATH_data", package="SWATH2stats")
 data("Study_design", package="SWATH2stats")
 data <- sample_annotation(OpenSWATH_data, Study_design)
 information <- plot_correlation_between_samples(data)
}
}
\author{
Peter Blattmann
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filter_mscore_freqobs.R
\name{filter_mscore_freqobs}
\alias{filter_mscore_freqobs}
\title{Filter OpenSWATH output table according to mscore.}
\usage{
filter_mscore_freqobs(
  data,
  mscore,
  percentage = NULL,
  rm.decoy = TRUE,
  mscore.col = "m_score"
)
}
\arguments{
\item{data}{A data frame containing SWATH data.}

\item{mscore}{Value that defines the mscore threshold according to which the
data will be filtered.}

\item{percentage}{Percentage in which replicas the transition has to reach
the mscore threshold.}

\item{rm.decoy}{Option to remove the decoys during filtering.}

\item{mscore.col}{Defines the column from which to retrieve the m_score. 
If you use JPP (Rosenberger, Bludau et al. 2017) this can be used to 
select between Protein and transition_group m_score.}
}
\value{
Returns a data frame with the filtered data.
}
\description{
This function filters the SWATH data according to the m_score value, as well
as to the number of occurence in the data.
}
\examples{
 data("OpenSWATH_data", package="SWATH2stats")
 data("Study_design", package="SWATH2stats")
 data <- sample_annotation(OpenSWATH_data, Study_design)
 data.filtered <- filter_mscore(data, 0.01)
 data.filtered <- filter_mscore_freqobs(data, 0.01, 0.8)
 data.filtered <- filter_mscore_condition(data, 0.01, 3)
}
\author{
Peter Blattmann
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assess_fdr_byrun.R
\name{assess_fdr_byrun}
\alias{assess_fdr_byrun}
\title{Assess assay, peptide and protein level FDR by run (for each MS_injection
separately) in OpenSWATH output table}
\usage{
assess_fdr_byrun(
  data,
  FFT = 1,
  n_range = 20,
  output = "pdf_csv",
  plot = TRUE,
  filename = "FDR_report_byrun",
  output_mscore_levels = c(0.01, 0.001),
  score_col = "m_score"
)
}
\arguments{
\item{data}{Annotated OpenSWATH/pyProphet output table. Refer to function
sample_annotation from this package for further information.}

\item{FFT}{Ratio of false positives to true negatives, q-values from
[Injection_name]_full_stat.csv in pyProphet stats output. As an
approximation, the q-values of multiple runs are averaged and supplied as
argument FFT. Numeric from 0 to 1. Defaults to 1, the most conservative
value (1 Decoy indicates 1 False target).}

\item{n_range}{Option to set the number of magnitude for which the m_score
threshold is decreased (e.g. n_range = 10, m-score from 0.1 until
10^-10)^.}

\item{output}{Choose output type. "pdf_csv" creates the output as files in
the working directory, "Rconsole" triggers delivery of the output to the
console enabling further computation or custom plotting / output.}

\item{plot}{Logical, whether or not to create plots from the results (using
the associated method plot.fdr_cube()}

\item{filename}{Modify the basename of the result files if set.}

\item{output_mscore_levels}{Define m-score levels to plot and write the
estimated FDR results.}

\item{score_col}{Column that contains the score. Default. m_score}
}
\value{
Returns an array of target/decoy identification numbers and
  calculated FDR values at different m-score cutoffs.
}
\description{
This function estimates the assay, peptide and protein FDR by run in an
OpenSWATH result table in dependence of a range of m_score cutoffs. The
results can be visualized and summarized by the associated method
plot.fdr_table().
It counts target and decoy assays (unique transition_group_id), peptides
(unique FullPeptideName) and proteins (unique ProteinName) in the OpenSWATH
output table in dependence of m-score cutoff, the useful m_score cutoff range
is evaluated for each dataset individually on the fly.
To arrive from decoy counts at an estimation of the false discovery rate
(false positives among the targets remaining at a given mscore cutoff) the
ratio of false positives to true negatives (decoys) (FFT) must be
supplied. It is estimated for each run individually by pyProphet and
contained in the pyProphet statistics [Injection_name]_full_stat.csv. As an
approximation, the FFTs of multiple runs are averaged and supplied as
argument FFT. For further details see the Vignette Section 1.3 and 4.1.
To assess fdr over the entire dataset, please refer to function
assess_fdr_overall. FDR is calculated as FDR = (TN*FFT/T); TN=decoys,
T=targets, FFT=see above.
}
\examples{
{
 data("OpenSWATH_data", package="SWATH2stats")
 data("Study_design", package="SWATH2stats")
 data <- sample_annotation(OpenSWATH_data, Study_design)
 assessed <- assess_fdr_byrun(data, FFT=0.7, output="Rconsole", plot=TRUE,
                              filename="Testoutput_assess_fdr_byrun")
 summary(assessed)
 }
}
\author{
Moritz Heusel
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/disaggregate.R
\name{disaggregate}
\alias{disaggregate}
\title{Transforms the SWATH data from a peptide- to a transition-level table.}
\usage{
disaggregate(data, all.columns = FALSE)
}
\arguments{
\item{data}{A data frame containing SWATH data.}

\item{all.columns}{Option that all columns are processed. Otherwise only the
columns typically needed for downstream analysis are processed.}
}
\value{
Returns a data frame containing the SWATH data in a transition-level
  table.
}
\description{
If the SWATH data should be analyzed on transition-level the data needs to be
tranformed from peptide-level table to a transition-level table (one row per
transition instead of one row per peptide). The columns
"aggr_Fragment_Annotation" and "aggr_Peak_Area" are disaggregated into the
new columns "Fragmentation" and "Intensity".
The following columns are renamed if they exist: FullPeptideName ->
PeptideSequence, Charge -> PrecursorCharge, Area -> Intensity, Fragment ->
Fragmentation, Sequence -> NakedSequence.
}
\examples{
{
 data("OpenSWATH_data", package="SWATH2stats")
 data("Study_design", package="SWATH2stats")
 data <- sample_annotation(OpenSWATH_data, Study_design)
 data.filtered.decoy <- filter_mscore(data, 0.01)
 raw <- disaggregate(data.filtered.decoy)
 }
}
\author{
Peter Blattmann
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convert4aLFQ.R
\name{convert4aLFQ}
\alias{convert4aLFQ}
\title{Convert table into the format expected by aLFQ.}
\usage{
convert4aLFQ(data, annotation = TRUE, check_transitions = TRUE)
}
\arguments{
\item{data}{A data frame containing the SWATH data in transition-level
format}

\item{annotation}{Option to indicate if the data has been annotated, i.e. if
the columns Condition, Replicate, Run are present. If option
is set to true it will write a new run_id as a string of the
combination of these three columns.}

\item{check_transitions}{Option if number of transitions should be checked.
As input only transition-level data should be used and
therefore this is checked. However, this makes the function
slow and herewith be omitted.}
}
\value{
Returns a data frame in the appropriate format for aLFQ.
}
\description{
This function selects the columns necessary for the aLFQ R package.
}
\examples{
{
 data("OpenSWATH_data", package="SWATH2stats")
 data("Study_design", package="SWATH2stats")
 data <- SWATH2stats::sample_annotation(OpenSWATH_data, Study_design, verbose=TRUE)
 data.filtered.decoy <- filter_mscore(data, 0.01)
 raw <- disaggregate(data.filtered.decoy)
 data.aLFQ <- convert4aLFQ(raw)
 }
}
\author{
Peter Blattmann
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convert4pythonscript.R
\name{convert4pythonscript}
\alias{convert4pythonscript}
\title{Convert data into the format for running a python script.}
\usage{
convert4pythonscript(data, replace.Unimod = TRUE)
}
\arguments{
\item{data}{A data frame containing SWATH data.}

\item{replace.Unimod}{Option to indicate if Unimod Identifier should be
replaced form ":"" to "_".}
}
\value{
Returns a data frame in the appropriate format to be used by a custom
  python script stored in the scripts folder.
}
\description{
This functions selects the columns suggested to run a python script to change
the data from peptide-level to transition-level.
}
\details{
The necessary columns are selected and the run column is renamed to filename
for the script. The intensities are taken from the column aggr_Peak_Area and
therefore the Intensity column is not exported.
}
\examples{
 data("OpenSWATH_data", package="SWATH2stats")
 data("Study_design", package="SWATH2stats")
 data <- sample_annotation(OpenSWATH_data, Study_design)
 data.filtered.decoy <- filter_mscore(data,0.01)
 data.pythonscript <- convert4pythonscript(data.filtered.decoy)
}
\author{
Peter Blattmann
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/write_matrix_peptides.R
\name{write_matrix_peptides}
\alias{write_matrix_peptides}
\title{Writes out an overview matrix of peptides mapping to a FDR quality controlled
protein master list at controlled global peptide FDR quality.}
\usage{
write_matrix_peptides(
  data,
  write_csv = FALSE,
  fun_aggregate = "sum",
  filename = "SWATH2stats_overview_matrix_peptidelevel.csv",
  rm_decoy = FALSE
)
}
\arguments{
\item{data}{A data frame containing annotated OpenSWATH/pyProphet data.}

\item{write_csv}{Option to determine if table should be written automatically
into csv file.}

\item{fun_aggregate}{What function to use when aggregating the set of 
intensities (sum or mean)?. Default: sum.}

\item{filename}{File base name of the .csv matrix written out to the working
folder.}

\item{rm_decoy}{Logical whether decoys will be removed from the data
matrix. Defaults to FALSE. It's sometimes useful to know how decoys behave
across a dataset and how many you allow into your final table with the
current filtering strategy.}
}
\value{
the peptides as a matrix! also output .csv matrix is written to the
  working folder.
}
\description{
Writes out an overview matrix on peptide level of a supplied (unfiltered or
prefiltered) OpenSWATH results data frame.
The peptide quantification is achieved by summing the areas under all 6
transitions per precursor and summing all precursors per FullPeptideName.
In order to keep the peptide-to-protein association, the FullPeptideName is
joined with the ProteinName.
}
\examples{
{
 data("OpenSWATH_data", package="SWATH2stats")
 data("Study_design", package="SWATH2stats")
 data <- sample_annotation(OpenSWATH_data, Study_design)
 written <- write_matrix_peptides(data)
 }
}
\author{
Moritz Heusel
}
